@@ -1,5 +1,5 @@
 /*
- * $Id: dq.c,v 1.31 2005/09/28 21:41:37 rmanfredi Exp $
+ * $Id: dq.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: dq.c,v 1.31 2005/09/28 21:41:37 rmanfredi Exp $");
+RCSID("$Id: dq.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include <math.h>			/* For pow() */
 
@@ -50,6 +50,11 @@ RCSID("$Id: dq.c,v 1.31 2005/09/28 21:41:37 rmanfredi Exp $");
 #include "vmsg.h"
 #include "search.h"
 #include "alive.h"
+#include "oob_proxy.h"
+#include "sockets.h"		/* For udp_active() */
+#include "settings.h"		/* For listen_addr() */
+#include "hosts.h"			/* For host_is_valid() */
+#include "share.h"			/* For query_strip_oob_flag() */
 
 #include "if/gnet_property_priv.h"
 
@@ -63,15 +68,18 @@ RCSID("$Id: dq.c,v 1.31 2005/09/28 21:41:37 rmanfredi Exp $");
 
 #include "lib/override.h"			/* Must be the last header included */
 
-#define DQ_MAX_LIFETIME		300000	/**< 5 minutes, in ms */
+#define DQ_MAX_LIFETIME		600000	/**< 10 minutes, in ms */
 #define DQ_PROBE_TIMEOUT  	1500	/**< 1.5 s extra per connection */
 #define DQ_PENDING_TIMEOUT 	1200	/**< 1.2 s extra per pending message */
 #define DQ_QUERY_TIMEOUT	3700	/**< 3.7 s */
 #define DQ_TIMEOUT_ADJUST	100		/**< 100 ms at each connection */
 #define DQ_MIN_TIMEOUT		1500	/**< 1.5 s at least between queries */
-#define DQ_LINGER_TIMEOUT	120000	/**< 2 minutes, in ms */
-#define DQ_STATUS_TIMEOUT	30000	/**< 30 s, in ms, to reply to query status */
+#define DQ_LINGER_TIMEOUT	180000	/**< 3 minutes, in ms */
+#define DQ_STATUS_TIMEOUT	40000	/**< 40 s, in ms, to reply to query status */
 #define DQ_MAX_PENDING		3		/**< Max pending queries we allow */
+#define DQ_MAX_STAT_TIMEOUT	2		/**< Max # of stat timeouts we allow */
+#define DQ_STAT_THRESHOLD	3		/**< Request status every 3 UP probed */
+#define DQ_MIN_FOR_GUIDANCE	20		/**< Request guidance if 20+ new results */
 
 #define DQ_LEAF_RESULTS		50		/**< # of results targetted for leaves */
 #define DQ_LOCAL_RESULTS	150		/**< # of results for local queries */
@@ -83,17 +91,11 @@ RCSID("$Id: dq.c,v 1.31 2005/09/28 21:41:37 rmanfredi Exp $");
 #define DQ_PERCENT_KEPT		5		/**< Assume 5% of results kept, worst case */
 
 #define DQ_MAX_TTL			5		/**< Max TTL we can use */
-#define DQ_AVG_ULTRA_NODES	2		/**< Avg # of ultranodes a leaf queries */
+#define DQ_AVG_ULTRA_NODES	3		/**< Avg # of ultranodes a leaf queries */
 
 #define DQ_MQ_EPSILON		2048	/**< Queues identical at +/- 2K */
 #define DQ_FUZZY_FACTOR		0.80	/**< Corrector for theoretical horizon */
 
-/**
- * Compute start of search string (which is NUL terminated) in query.
- * The "+2" skips the "speed" field in the query.
- */
-#define QUERY_TEXT(m)	((m) + sizeof(struct gnutella_header) + 2)
-
 /**
  * Structure produced by dq_fill_next_up, representing the nodes to which
  * we could send the query, along with routing information to be able to favor
@@ -124,9 +126,12 @@ typedef struct dquery {
 	pmsg_t *mb;				/**< The search messsage "template" */
 	query_hashvec_t *qhv;	/**< Query hash vector for QRP filtering */
 	GHashTable *queried;	/**< Contains node IDs that we queried so far */
+	const gchar *lmuid;		/**< For proxied query: the original leaf MUID */
+	guint16 query_flags;	/**< Flags from the marked query speed field */
 	guint8 ttl;				/**< Initial query TTL */
 	guint32 horizon;		/**< Theoretical horizon reached thus far */
 	guint32 up_sent;		/**< # of UPs to which we really sent our query */
+	guint32 last_status;	/**< How many UP queried last time we got status */
 	guint32 pending;		/**< Pending query messages not ACK'ed yet by mq */
 	guint32 max_results;	/**< Max results we're targetting for */
 	guint32 fin_results;	/**< # of results terminating leaf-guided query */
@@ -136,6 +141,7 @@ typedef struct dquery {
 	guint32 new_results;	/**< New we got since last query status request */
 	guint32 kept_results;	/**< Results they say they kept after filtering */
 	guint32 result_timeout;	/**< The current timeout for getting results */
+	guint32 stat_timeouts;	/**< The amount of status request timeouts we had */
 	gpointer expire_ev;		/**< Callout queue global expiration event */
 	gpointer results_ev;	/**< Callout queue results expiration event */
 	gpointer alive;			/**< Alive ping stats for computing timeouts */
@@ -151,8 +157,9 @@ typedef struct dquery {
 #define DQ_F_LINGER			0x00000002	/**< Lingering to monitor extra hits */
 #define DQ_F_LEAF_GUIDED	0x00000004	/**< Leaf-guided query */
 #define DQ_F_WAITING		0x00000008	/**< Waiting guidance reply from leaf */
-#define DQ_F_GOT_GUIDANCE	0x00000010	/**< Got unsolicited leaf guidance */
+#define DQ_F_GOT_GUIDANCE	0x00000010	/**< Got some leaf guidance */
 #define DQ_F_USR_CANCELLED	0x00000020	/**< Explicitely cancelled by user */
+#define DQ_F_ROUTING_HITS	0x00000040	/**< We'll be routing all hits */
 #define DQ_F_EXITING		0x80000000	/**< Final cleanup at exit time */
 
 /**
@@ -177,6 +184,14 @@ static GHashTable *by_node_id = NULL;
  */
 static GHashTable *by_muid = NULL;
 
+/**
+ * This table keeps track of the association between a leaf MUID and the
+ * dynamic query, so that when an unsolicited query status comes, we may
+ * account them for the relevant query (since for OOB-proxied query, the
+ * MUID we'll get is the one the leaf knows about).
+ */
+static GHashTable *by_leaf_muid = NULL;
+
 /**
  * Information about query messages sent.
  *
@@ -238,7 +253,7 @@ fill_hosts(void)
 			hosts[i][j] = hosts[i][j-1] + pow(i, j);
 
 			if (dq_debug > 19)
-				printf("horizon(degree=%d, ttl=%d) = %d\n",
+				g_message("horizon(degree=%d, ttl=%d) = %d",
 					i+1, j+1, hosts[i][j]);
 		}
 	}
@@ -292,8 +307,9 @@ dq_kept_results(dquery_t *dq)
 	 * of hits everywhere.
 	 */
 
-	return (dq->flags & DQ_F_LEAF_GUIDED) ?
-		(dq->kept_results / DQ_AVG_ULTRA_NODES) : dq->results;
+	return (dq->flags & DQ_F_GOT_GUIDANCE) ?
+		(dq->kept_results / DQ_AVG_ULTRA_NODES) + dq->new_results :
+		dq->results;
 }
 
 /**
@@ -432,7 +448,7 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		g_hash_table_remove(dq->queried, GUINT_TO_POINTER(pmi->node_id));
 
 		if (dq_debug > 19)
-			printf("DQ[%d] %snode #%d degree=%d dropped message TTL=%d\n",
+			g_message("DQ[%d] %snode #%d degree=%d dropped message TTL=%d",
 				dq->qid, dq->node_id == NODE_ID_LOCAL ? "(local) " : "",
 				pmi->node_id, pmi->degree, pmi->ttl);
 
@@ -457,11 +473,11 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		dq->up_sent++;
 
 		if (dq_debug > 19) {
-			printf("DQ[%d] %snode #%d degree=%d sent message TTL=%d\n",
+			g_message("DQ[%d] %snode #%d degree=%d sent message TTL=%d",
 				dq->qid, dq->node_id == NODE_ID_LOCAL ? "(local) " : "",
 				pmi->node_id, pmi->degree, pmi->ttl);
-			printf("DQ[%d] %s(%d secs) queried %d UP%s, "
-				"horizon=%d, results=%d\n",
+			g_message("DQ[%d] %s(%d secs) queried %d UP%s, "
+				"horizon=%d, results=%d",
 				dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
 				(gint) (tm_time() - dq->start),
 				dq->up_sent, dq->up_sent == 1 ? "" :"s",
@@ -507,8 +523,10 @@ dq_pmsg_by_ttl(dquery_t *dq, gint ttl)
 	/*
 	 * Patch the TTL in the new data buffer.
 	 */
-
-	((struct gnutella_header *) pdata_start(db))->ttl = ttl;
+	{
+		gnutella_header_t *header = cast_to_gpointer(pdata_start(db));
+		gnutella_header_set_ttl(header, ttl);
+	}
 
 	/*
 	 * Now create a message for this data buffer and save it for later perusal.
@@ -692,19 +710,18 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, gint ncount)
 static void
 dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 {
+	gconstpointer head;
 	GSList *nodes;
-	gchar *payload;
-	struct gnutella_header *head;
-
-	payload = pmsg_start(dq->mb);
-	head = (struct gnutella_header *) payload;
 
+	head = cast_to_gconstpointer(pmsg_start(dq->mb));
 	nodes = qrt_build_query_target(dq->qhv,
-		head->hops, MAX(head->ttl, 2), source);
+				gnutella_header_get_hops(head),
+				MAX(gnutella_header_get_ttl(head), 2),
+				source);
 
 	if (dq_debug > 4)
 		g_message("DQ QRP %s (%d word%s%s) forwarded to %d/%d leaves",
-			gmsg_infostr_full(payload), dq->qhv->count,
+			gmsg_infostr_full(head), dq->qhv->count,
 			dq->qhv->count == 1 ? "" : "s",
 			dq->qhv->has_urn ? " + URN" : "",
 			g_slist_length(nodes), node_leaf_count);
@@ -724,14 +741,13 @@ dq_free(dquery_t *dq)
 	gboolean found;
 	gpointer key;
 	gpointer value;
-	struct gnutella_header *head;
 
 	g_assert(dq != NULL);
 	g_assert(g_hash_table_lookup(dqueries, dq));
 
 	if (dq_debug > 19)
-		printf("DQ[%d] %s(%d secs; +%d secs) node #%d ending: "
-			"ttl=%d, queried=%d, horizon=%d, results=%d+%d\n",
+		g_message("DQ[%d] %s(%d secs; +%d secs) node #%d ending: "
+			"ttl=%d, queried=%d, horizon=%d, results=%d+%d",
 			dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
 			(gint) (tm_time() - dq->start),
 			(dq->flags & DQ_F_LINGER) ? (gint) (tm_time() - dq->stop) : 0,
@@ -744,7 +760,21 @@ dq_free(dquery_t *dq)
 	if (dq->expire_ev)
 		cq_cancel(callout_queue, dq->expire_ev);
 
-	if (dq->results >= dq->max_results)
+	/*
+	 * Update statistics.
+	 *
+	 * If a query is terminated by the user or because the node was removed,
+	 * it is counted as having been fully completed: there's nothing more
+	 * we can do about it.
+	 */
+
+	if (
+		dq->results >= dq->max_results ||
+		(dq->flags & (DQ_F_USR_CANCELLED | DQ_F_ID_CLEANING)) ||
+		dq->kept_results /
+			(dq->node_id == NODE_ID_LOCAL ? 1 : DQ_AVG_ULTRA_NODES)
+			>= dq->max_results
+	)
 		gnet_stats_count_general(GNR_DYN_QUERIES_COMPLETED_FULL, 1);
 	else if (dq->results > 0)
 		gnet_stats_count_general(GNR_DYN_QUERIES_COMPLETED_PARTIAL, 1);
@@ -812,8 +842,8 @@ dq_free(dquery_t *dq)
 	 * Remove query's MUID.
 	 */
 
-	head = (struct gnutella_header *) pmsg_start(dq->mb);
-	found = g_hash_table_lookup_extended(by_muid, head->muid, &key, &value);
+	found = g_hash_table_lookup_extended(by_muid,
+				gnutella_header_get_muid(pmsg_start(dq->mb)), &key, &value);
 
 	if (found) {			/* Could be missing if a MUID conflict occurred */
 		if (value == dq) {	/* Make sure it's for us in case of conflicts */
@@ -822,6 +852,18 @@ dq_free(dquery_t *dq)
 		}
 	}
 
+	/*
+	 * Remove the leaf-known MUID mapping.
+	 */
+
+	if (dq->lmuid != NULL) {
+		found = g_hash_table_lookup_extended(
+			by_leaf_muid, dq->lmuid, &key, &value);
+		if (found && value == dq)
+			g_hash_table_remove(by_leaf_muid, key);
+		atom_guid_free(dq->lmuid);
+	}
+
 	pmsg_free(dq->mb);			/* Now that we used the MUID */
 
 	wfree(dq, sizeof(*dq));
@@ -838,7 +880,7 @@ dq_expired(cqueue_t *unused_cq, gpointer obj)
 	(void) unused_cq;
 
 	if (dq_debug > 19)
-		printf("DQ[%d] expired\n", dq->qid);
+		g_message("DQ[%d] expired", dq->qid);
 
 	dq->expire_ev = NULL;	/* Indicates callback fired */
 
@@ -872,10 +914,10 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 {
 	dquery_t *dq = (dquery_t *) obj;
 	gnutella_node_t *n;
-	struct gnutella_header *head;
 	gint timeout;
 	guint32 avg;
 	guint32 last;
+	gboolean was_waiting = FALSE;
 
 	(void) unused_cq;
 	g_assert(!(dq->flags & DQ_F_LINGER));
@@ -884,14 +926,51 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 
 	/*
 	 * If we were waiting for a status reply from the queryier, well, we
-	 * just timed-out.  Cancel this query.
+	 * just timed-out.
+	 *
+	 * We used to cancel this query, on timeouts, but that seems harsh.
+	 * Simply turn off the leaf-guidance indication and continue.
+	 * Note that the leaf may still send us unsolicited guidance if it wants.
+	 *		--RAM, 2006-08-16
 	 */
 
 	if (dq->flags & DQ_F_WAITING) {
+		was_waiting = TRUE;
+		dq->stat_timeouts++;
+
 		if (dq_debug > 19)
-			printf("DQ[%d] (%d secs) timeout waiting for status results\n",
-				dq->qid, (gint) (tm_time() - dq->start));
+			g_message("DQ[%d] (%d secs) timeout #%u waiting for status results",
+				dq->qid, (gint) (tm_time() - dq->start), dq->stat_timeouts);
 		dq->flags &= ~DQ_F_WAITING;
+
+		if (
+			!(dq->flags & DQ_F_GOT_GUIDANCE) &&	/* No guidance already? */
+			dq->stat_timeouts >= DQ_MAX_STAT_TIMEOUT
+		) {
+			dq->flags &= ~DQ_F_LEAF_GUIDED;		/* Probably not supported */
+			node_set_leaf_guidance(dq->node_id, FALSE);
+
+			if (dq_debug > 19)
+				g_message(
+					"DQ[%d] (%d secs) turned off leaf-guidance for node #%u",
+					dq->qid, (gint) (tm_time() - dq->start), dq->node_id);
+		}
+
+		/* FALL THROUGH */
+	}
+
+	/*
+	 * If we're not routing the query hits and the query is no longer
+	 * leaf-guided (because for instance the remote host is not answering
+	 * our status requests), we have no way of performing the dynamic
+	 * query and we must abort.
+	 */
+
+	if (!(dq->flags & (DQ_F_LEAF_GUIDED|DQ_F_ROUTING_HITS))) {
+		if (dq_debug)
+			g_message(
+				"DQ[%d] terminating unguided & unrouted (queried %u UP%s)",
+				dq->qid, dq->up_sent, dq->up_sent == 1 ? "" : "s");
 		dq_terminate(dq);
 		return;
 	}
@@ -903,9 +982,22 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 	 *
 	 * For local queries, DQ_F_LEAF_GUIDED is not set, so we'll continue
 	 * anyway.
+	 *
+	 * If we ever got unsolicited guidance, then there's no need to ask
+	 * for it explicitly: we can safely assume the leaf will inform us
+	 * whenever it gets more results.
+	 *		--RAM, 2006-08-16
 	 */
 
-	if (!(dq->flags & DQ_F_LEAF_GUIDED) || (dq->flags & DQ_F_GOT_GUIDANCE)) {
+	if (
+		was_waiting ||
+		!(dq->flags & DQ_F_LEAF_GUIDED) ||
+		dq->up_sent - dq->last_status < DQ_STAT_THRESHOLD ||
+		(
+			(dq->flags & DQ_F_ROUTING_HITS) &&
+			dq->new_results < DQ_MIN_FOR_GUIDANCE
+		)
+	) {
 		dq_send_next(dq);
 		return;
 	}
@@ -921,21 +1013,26 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 
 	if (n == NULL) {
 		if (dq_debug > 19)
-			printf("DQ[%d] (%d secs) node #%d appears to be dead\n",
+			g_message("DQ[%d] (%d secs) node #%d appears to be dead",
 				dq->qid, (gint) (tm_time() - dq->start), dq->node_id);
 		dq_free(dq);
 		return;
 	}
 
 	if (dq_debug > 19)
-		printf("DQ[%d] (%d secs) requesting node #%d for status (kept=%u)\n",
+		g_message("DQ[%d] (%d secs) requesting node #%d for status (kept=%u)",
 			dq->qid, (gint) (tm_time() - dq->start), dq->node_id,
 			dq->kept_results);
 
 	dq->flags |= DQ_F_WAITING;
-	head = (struct gnutella_header *) pmsg_start(dq->mb);
 
-	vmsg_send_qstat_req(n, head->muid);
+	/*
+	 * Use the original MUID sent by the leaf, it doesn't know
+	 * the other one.
+	 */
+
+	vmsg_send_qstat_req(n,
+		dq->lmuid ? dq->lmuid : gnutella_header_get_muid(pmsg_start(dq->mb)));
 
 	/*
 	 * Compute the timout using the available ping-pong round-trip
@@ -947,7 +1044,8 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 	timeout = MAX(timeout, DQ_STATUS_TIMEOUT);
 
 	if (dq_debug > 19)
-		printf("DQ[%d] status reply timeout set to %d s\n", dq->qid, timeout);
+		g_message("DQ[%d] status reply timeout set to %d s", dq->qid,
+			timeout / 1000);
 
 	dq->results_ev = cq_insert(callout_queue, timeout,
 		dq_results_expired, dq);
@@ -959,27 +1057,33 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 static void
 dq_terminate(dquery_t *dq)
 {
+	gint delay;
+
 	g_assert(!(dq->flags & DQ_F_LINGER));
 	g_assert(dq->results_ev == NULL);
 
 	/*
 	 * Put the query in lingering mode, so we can continue to monitor
 	 * results for some time after we stopped the dynamic querying.
+	 *
+	 * Even when the query has been user-cancelled, we put it in the
+	 * callout queue to not have the query freed on the same calling stack.
 	 */
 
+	delay = (dq->flags & DQ_F_USR_CANCELLED) ? 1 : DQ_LINGER_TIMEOUT;
+
 	if (dq->expire_ev != NULL)
-		cq_resched(callout_queue, dq->expire_ev, DQ_LINGER_TIMEOUT);
+		cq_resched(callout_queue, dq->expire_ev, delay);
 	else
-		dq->expire_ev = cq_insert(callout_queue, DQ_LINGER_TIMEOUT,
-			dq_expired, dq);
+		dq->expire_ev = cq_insert(callout_queue, delay, dq_expired, dq);
 
 	dq->flags &= ~DQ_F_WAITING;
 	dq->flags |= DQ_F_LINGER;
 	dq->stop = tm_time();
 
 	if (dq_debug > 19)
-		printf("DQ[%d] (%d secs) node #%d lingering: "
-			"ttl=%d, queried=%d, horizon=%d, results=%d\n",
+		g_message("DQ[%d] (%d secs) node #%d lingering: "
+			"ttl=%d, queried=%d, horizon=%d, results=%d",
 			dq->qid, (gint) (tm_time() - dq->start), dq->node_id,
 			dq->ttl, dq->up_sent, dq->horizon, dq->results);
 }
@@ -1089,7 +1193,7 @@ dq_send_query(dquery_t *dq, gnutella_node_t *n, gint ttl)
 	mb = pmsg_clone_extend(mb, dq_pmsg_free, pmi);
 
 	if (dq_debug > 19)
-		printf("DQ[%d] (%d secs) queuing ttl=%d to #%d %s <%s> Q=%d bytes\n",
+		g_message("DQ[%d] (%d secs) queuing ttl=%d to #%d %s <%s> Q=%d bytes",
 			dq->qid, (gint) delta_time(tm_time(), dq->start),
 			pmi->ttl, n->id, node_addr(n), node_vendor(n),
 			(gint) NODE_MQUEUE_PENDING(n));
@@ -1124,13 +1228,11 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	if (current_peermode != NODE_P_ULTRA) {
-		if (dq_debug > 19)
-			printf("DQ[%d] terminating (no longer an ultra node)\n", dq->qid);
+		if (dq_debug)
+			g_message("DQ[%d] terminating (no longer an ultra node)", dq->qid);
 		goto terminate;
 	}
 
-	dq->flags &= ~DQ_F_GOT_GUIDANCE;	/* Clear flag */
-
 	/*
 	 * Terminate query if we reached the amount of results we wanted or
 	 * if we reached the maximum theoretical horizon.
@@ -1139,9 +1241,12 @@ dq_send_next(dquery_t *dq)
 	results = dq_kept_results(dq);
 
 	if (dq->horizon >= DQ_MAX_HORIZON || results >= dq->max_results) {
-		if (dq_debug > 19)
-			printf("DQ[%d] terminating (horizon=%u >= %d, results=%u >= %u)\n",
-				dq->qid, dq->horizon, DQ_MAX_HORIZON, results, dq->max_results);
+		if (dq_debug)
+			g_message("DQ[%d] terminating "
+				"(UPs=%u, horizon=%u >= %d, %s results=%u >= %u)",
+				dq->qid, dq->up_sent, dq->horizon, DQ_MAX_HORIZON,
+				(dq->flags & DQ_F_GOT_GUIDANCE) ? "guided" : "unguided",
+				results, dq->max_results);
 		goto terminate;
 	}
 
@@ -1154,9 +1259,12 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	if (dq->results + dq->oob_results > dq->fin_results) {
-		if (dq_debug > 19)
-			printf("DQ[%d] terminating (seen=%u + OOB=%u >= %u -- kept=%u)\n",
-				dq->qid, dq->results, dq->oob_results, dq->fin_results,
+		if (dq_debug)
+			g_message("DQ[%d] terminating "
+				"(UPs=%u, seen=%u + OOB=%u >= %u -- %s kept=%u)",
+				dq->qid, dq->up_sent,
+				dq->results, dq->oob_results, dq->fin_results,
+				(dq->flags & DQ_F_GOT_GUIDANCE) ? "guided" : "unguided",
 				results);
 		goto terminate;
 	}
@@ -1167,8 +1275,8 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	if (dq->up_sent >= max_connections - normal_connections) {
-		if (dq_debug > 19)
-			printf("DQ[%d] terminating (queried UPs=%u >= %u)\n",
+		if (dq_debug)
+			g_message("DQ[%d] terminating (queried UPs=%u >= %u)",
 				dq->qid, dq->up_sent, max_connections - normal_connections);
 		goto terminate;
 	}
@@ -1182,7 +1290,7 @@ dq_send_next(dquery_t *dq)
 
 	if (dq->pending >= DQ_MAX_PENDING) {
 		if (dq_debug > 19)
-			printf("DQ[%d] waiting for %u ms (pending=%u)\n",
+			g_message("DQ[%d] waiting for %u ms (pending=%u)",
 				dq->qid, dq->result_timeout, dq->pending);
 		dq->results_ev = cq_insert(callout_queue,
 			dq->result_timeout, dq_results_expired, dq);
@@ -1195,9 +1303,9 @@ dq_send_next(dquery_t *dq)
 	g_assert(dq->nv == nv);		/* Saved for next time */
 
 	if (dq_debug > 19)
-		printf("DQ[%d] still %d UP%s to query (results %sso far: %u)\n",
+		g_message("DQ[%d] still %d UP%s to query (results %sso far: %u)",
 			dq->qid, found, found == 1 ? "" : "s",
-			(dq->flags & DQ_F_LEAF_GUIDED) ? "reported kept " : "", results);
+			(dq->flags & DQ_F_GOT_GUIDANCE) ? "reported kept " : "", results);
 
 	if (found == 0) {
 		dq_terminate(dq);	/* Terminate query: no more UP to send it to */
@@ -1228,7 +1336,7 @@ dq_send_next(dquery_t *dq)
 			!qrp_node_can_route(node, dq->qhv)
 		) {
 			if (dq_debug > 19)
-				printf("DQ[%d] TTL=1, skipping node #%d: can't route query!\n",
+				g_message("DQ[%d] TTL=1, skipping node #%d: can't route query!",
 					dq->qid, node->id);
 
 			continue;
@@ -1267,7 +1375,7 @@ dq_send_next(dquery_t *dq)
 		timeout += (dq->pending - 1) * DQ_PENDING_TIMEOUT;
 
 	if (dq_debug > 1)
-		printf("DQ[%d] (%d secs) timeout set to %d ms (pending=%d)\n",
+		g_message("DQ[%d] (%d secs) timeout set to %d ms (pending=%d)",
 			dq->qid, (gint) (tm_time() - dq->start), timeout, dq->pending);
 
 	dq->results_ev = cq_insert(callout_queue, timeout, dq_results_expired, dq);
@@ -1297,7 +1405,7 @@ dq_send_probe(dquery_t *dq)
 	found = dq_fill_probe_up(dq, nv, ncount);
 
 	if (dq_debug > 19)
-		printf("DQ[%d] found %d UP%s to probe\n",
+		g_message("DQ[%d] found %d UP%s to probe",
 			dq->qid, found, found == 1 ? "" : "s");
 
 	/*
@@ -1360,7 +1468,7 @@ cleanup:
 static void
 dq_common_init(dquery_t *dq)
 {
-	struct gnutella_header *head;
+	gconstpointer head;
 
 	dq->qid = dyn_query_id++;
 	dq->queried = g_hash_table_new(NULL, NULL);
@@ -1409,22 +1517,56 @@ dq_common_init(dquery_t *dq)
 	 * Record the MUID of this query, warning if a conflict occurs.
 	 */
 
-	head = (struct gnutella_header *) pmsg_start(dq->mb);
+	head = cast_to_gconstpointer(pmsg_start(dq->mb));
 
-	if (g_hash_table_lookup(by_muid, head->muid))
+	if (g_hash_table_lookup(by_muid, gnutella_header_get_muid(head)))
 		g_warning("conflicting MUID \"%s\" for dynamic query, ignoring.",
-			guid_hex_str(head->muid));
+			guid_hex_str(gnutella_header_get_muid(head)));
 	else {
-		gchar *muid = atom_guid_get(head->muid);
+		gchar *muid = atom_guid_get(gnutella_header_get_muid(head));
 		g_hash_table_insert(by_muid, muid, dq);
 	}
 
-	if (dq_debug > 19)
-		printf("DQ[%d] created for node #%d: TTL=%d max_results=%d "
-			"guidance=%s MUID=%s q=\"%s\"\n",
+	/*
+	 * Record the leaf-known MUID of this query, warning if a conflict occurs.
+	 * Note that dq->lmuid is already an atom, so it can be inserted as-is
+	 * in the hash table as key.
+	 */
+
+	if (dq->lmuid != NULL) {
+		if (g_hash_table_lookup(by_leaf_muid, dq->lmuid))
+			g_warning("ignoring conflicting leaf MUID \"%s\" for dynamic query",
+				guid_hex_str(dq->lmuid));
+		else
+			g_hash_table_insert(by_leaf_muid,
+				deconstify_gpointer(dq->lmuid), dq);
+	}
+
+	if (dq_debug) {
+		gchar *start = pmsg_start(dq->mb);
+		guint16 req_speed;
+
+		READ_GUINT16_LE(start + GTA_HEADER_SIZE, req_speed);
+
+		g_message("DQ[%d] created for node #%d: TTL=%d max_results=%d "
+			"guidance=%s routing=%s "
+			"MUID=%s%s%s q=\"%s\" speed=0x%x (%s%s%s%s%s%s%s)",
 			dq->qid, dq->node_id, dq->ttl, dq->max_results,
 			(dq->flags & DQ_F_LEAF_GUIDED) ? "yes" : "no",
-			guid_hex_str(head->muid), QUERY_TEXT(pmsg_start(dq->mb)));
+			(dq->flags & DQ_F_ROUTING_HITS) ? "yes" : "no",
+			guid_hex_str(gnutella_header_get_muid(head)),
+			dq->lmuid ? " leaf-MUID=" : "",
+			dq->lmuid ? data_hex_str(dq->lmuid, GUID_RAW_SIZE): "",
+			gnutella_msg_search_get_text(start), req_speed,
+			(req_speed & QUERY_SPEED_MARK) ? "MARKED" : "",
+			(req_speed & QUERY_SPEED_FIREWALLED) ? " FW" : "",
+			(req_speed & QUERY_SPEED_XML) ? " XML" : "",
+			(req_speed & QUERY_SPEED_LEAF_GUIDED) ? " GUIDED" : "",
+			(req_speed & QUERY_SPEED_GGEP_H) ? " GGEP_H" : "",
+			(req_speed & QUERY_SPEED_OOB_REPLY) ? " OOB" : "",
+			(req_speed & QUERY_SPEED_FW_TO_FW) ? " FW2FW" : ""
+		);
+	}
 }
 
 /**
@@ -1435,54 +1577,122 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv)
 {
 	dquery_t *dq;
 	guint16 req_speed;
-	gboolean tagged_speed = FALSE;
+	gboolean tagged_speed;
+	const gchar *leaf_muid;
 
+	/* Query from leaf node */
 	g_assert(NODE_IS_LEAF(n));
+	g_assert(gnutella_header_get_hops(&n->header) == 1);
 
 	dq = walloc0(sizeof(*dq));
 
+	READ_GUINT16_LE(n->data, req_speed);
+	tagged_speed = (req_speed & QUERY_SPEED_MARK) ? TRUE : FALSE;
+
+	/*
+	 * Determine whether this query will be leaf-guided.
+	 *
+	 * A leaf-guided query must be marked as such in the query flags.
+	 * However, if the node has not been responding to our query status
+	 * enquiries, then we marked it explicitly as being non-guiding and
+	 * we will ignore any tagging in the query.
+	 *
+	 * LimeWire has a bug in that it does not mark the queries it sends
+	 * as supporting leaf-guidance.  However, we can derive support from
+	 * its advertising the proper vendor messages.
+	 */
+
+	if (
+		(tagged_speed && (req_speed & QUERY_SPEED_LEAF_GUIDED)) ||
+		NODE_LEAF_GUIDE(n)
+	)
+		dq->flags |= DQ_F_LEAF_GUIDED;
+
+	/*
+	 * If the query is not leaf-guided and not OOB proxied already, then we
+	 * need to ensure results are routed back to us.
+	 * We won't know how much they filter out however, but they just have
+	 * to implement proper leaf-guidance for better results as leaves...
+	 *		--RAM, 2006-08-16
+	 */
+
+	if (
+		!(dq->flags & DQ_F_LEAF_GUIDED) &&
+		NULL == oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header))
+	) {
+		if (
+			udp_active() && proxy_oob_queries && !is_udp_firewalled &&
+			host_is_valid(listen_addr(), socket_listen_port())
+		) {
+			/*
+			 * Running with UDP support.
+			 * OOB-proxy the query so that we can control how much results
+			 * they get by routing the results ourselves to the leaf.
+			 */
+
+			if (dq_debug > 19)
+				g_message("DQ node #%d %s <%s> OOB-proxying query \"%s\" (%s)",
+					n->id, node_addr(n), node_vendor(n), n->data + 2,
+					(tagged_speed && (req_speed & QUERY_SPEED_LEAF_GUIDED)) ?
+						"guided" : "unguided"
+				);
+
+			oob_proxy_create(n);
+			gnet_stats_count_general(GNR_OOB_PROXIED_QUERIES, 1);
+		} else if (tagged_speed && (req_speed & QUERY_SPEED_OOB_REPLY)) {
+			/*
+			 * Running without UDP support, or UDP-firewalled...
+			 * Must remove the OOB flag so that results be routed back.
+			 */
+
+			query_strip_oob_flag(n, n->data);
+			READ_GUINT16_LE(n->data, req_speed);	/* Refresh our cache */
+
+			if (dq_debug > 19)
+				g_message(
+					"DQ node #%d %s <%s> stripped OOB on query \"%s\" (%s)",
+					n->id, node_addr(n), node_vendor(n), n->data + 2,
+					(tagged_speed && (req_speed & QUERY_SPEED_LEAF_GUIDED)) ?
+						"guided" : "unguided"
+				);
+		}
+	}
+
+	/*
+	 * See whether we'll be seeing all the hits...
+	 */
+
+	if (
+		NULL != oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header)) ||	
+		(tagged_speed && !(req_speed && QUERY_SPEED_OOB_REPLY))
+	)
+		dq->flags |= DQ_F_ROUTING_HITS;
+
 	dq->node_id = n->id;
-	dq->mb = gmsg_split_to_pmsg(
-		(guchar *) &n->header, n->data,
-		n->size + sizeof(struct gnutella_header));
+	dq->mb = gmsg_split_to_pmsg(&n->header, n->data, n->size + GTA_HEADER_SIZE);
 	dq->qhv = qhvec_clone(qhv);
 	if (qhvec_has_urn(qhv))
 		dq->max_results = DQ_LEAF_RESULTS / DQ_SHA1_DECIMATOR;
 	else
 		dq->max_results = DQ_LEAF_RESULTS;
 	dq->fin_results = dq->max_results * 100 / DQ_PERCENT_KEPT;
-	dq->ttl = MIN(n->header.ttl, DQ_MAX_TTL);
+	dq->ttl = MIN(gnutella_header_get_ttl(&n->header), DQ_MAX_TTL);
 	dq->alive = n->alive_pings;
+	if (tagged_speed)
+		dq->query_flags = req_speed;
 
-	/*
-	 * Determine whether this query will be leaf-guided.
-	 *
-	 * If the remote node performed the vendor message supported exchange,
-	 * we have already determined whether its queries would be leaf-guided
-	 * (i.e. advertised support for BEAR/11v1 is enough to assume the query
-	 * is leaf-guided).
-	 *
-	 * But there is also a flag in the query that can be set to indicate
-	 * leaf-guidance, and some vendors may choose to rely on that solely
-	 * and never advertise the vendor messages related to this feature.
-	 * So we also perform a test on the query flags in the speed field.
-	 */
-
-	if (NODE_GUIDES_QUERY(n))
-		dq->flags |= DQ_F_LEAF_GUIDED;
-
-	READ_GUINT16_LE(n->data, req_speed);
-	tagged_speed = (req_speed & QUERY_SPEED_MARK) ? TRUE : FALSE;
-
-	if (tagged_speed && (req_speed & QUERY_SPEED_LEAF_GUIDED))
-		dq->flags |= DQ_F_LEAF_GUIDED;
+	leaf_muid = oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header));
+	if (leaf_muid != NULL)
+		dq->lmuid = atom_guid_get(leaf_muid);
 
 	if (dq_debug > 19)
-		printf("DQ node #%d %s <%s> (%s leaf-guidance) %squeries \"%s\"\n",
+		g_message("DQ node #%d %s <%s> (%s leaf-guidance) %s%squeries \"%s\"",
 			n->id, node_addr(n), node_vendor(n),
 			(dq->flags & DQ_F_LEAF_GUIDED) ? "with" : "no",
 			tagged_speed && (req_speed & QUERY_SPEED_OOB_REPLY) ? "OOB-" : "",
-			QUERY_TEXT(pmsg_start(dq->mb)));
+			oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header))
+				? "proxied " : "",
+			gnutella_msg_search_get_text(pmsg_start(dq->mb)));
 
 	gnet_stats_count_general(GNR_LEAF_DYN_QUERIES, 1);
 
@@ -1509,7 +1719,7 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	if (current_peermode != NODE_P_ULTRA) {
 		if (dq_debug)
 			g_warning("ignoring dynamic query \"%s\": no longer an ultra node",
-				QUERY_TEXT(pmsg_start(mb)));
+				gnutella_msg_search_get_text(pmsg_start(mb)));
 
 		pmsg_free(mb);
 		qhvec_free(qhv);
@@ -1533,6 +1743,7 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	dq->fin_results = dq->max_results * 100 / DQ_PERCENT_KEPT;
 	dq->ttl = MIN(my_ttl, DQ_MAX_TTL);
 	dq->alive = NULL;
+	dq->flags = DQ_F_ROUTING_HITS;		/* We get our own hits! */
 
 	gnet_stats_count_general(GNR_LOCAL_DYN_QUERIES, 1);
 
@@ -1562,9 +1773,9 @@ dq_node_removed(guint32 node_id)
 	for (sl = value; sl; sl = g_slist_next(sl)) {
 		dquery_t *dq = (dquery_t *) sl->data;
 
-		if (dq_debug > 19)
-			printf("DQ[%d] terminated by node #%d removal\n",
-				dq->qid, dq->node_id);
+		if (dq_debug)
+			g_message("DQ[%d] terminated by node #%u removal (queried %u UP%s)",
+				dq->qid, dq->node_id, dq->up_sent, dq->up_sent == 1 ? "" : "s");
 
 		/* Don't remove query from the table in dq_free() */
 		dq->flags |= DQ_F_ID_CLEANING;
@@ -1584,11 +1795,13 @@ dq_node_removed(guint32 node_id)
  * @param oob if TRUE indicates that we just got notified about OOB results
  * awaiting, but which have not been claimed yet.  If FALSE, the results
  * have been validated and will be sent to the queryier.
+ * @param status	result set `status' flags gathered during parsing
  *
- * @return FALSE if the query was explicitly cancelled by the user
+ * @return FALSE if the query was explicitly cancelled by the user or if we
+ * should not forward the results anyway.
  */
 static gboolean
-dq_count_results(gchar *muid, gint count, gboolean oob)
+dq_count_results(const gchar *muid, gint count, guint16 status, gboolean oob)
 {
 	dquery_t *dq;
 
@@ -1599,6 +1812,48 @@ dq_count_results(gchar *muid, gint count, gboolean oob)
 	if (dq == NULL)
 		return TRUE;
 
+	/*
+	 * If we got actual results (not an OOB indication) and if we see that
+	 * the replying server is firewalled, the requester is also firewalled
+	 * and does not support firewalled-to-firewalled transfers, it's not
+	 * necessary to forward the results: they would be useless.
+	 *
+	 * When firewall-to-firewall is supported, both servents need to support
+	 * if for the transfer to be initiated.  We assume that subsequent
+	 * versions of the reliable UDP layer used for these transfers and the
+	 * means to set them up will remain compatible, regardless of the versions
+	 * used by both parties.
+	 *		--RAM, 2006-08-17
+	 */
+
+	if (
+		!oob &&
+		((
+			(status & ST_FIREWALL) &&
+			(dq->query_flags & (QUERY_SPEED_FIREWALLED|QUERY_SPEED_FW_TO_FW))
+				== QUERY_SPEED_FIREWALLED
+		) || (
+			(status & (ST_FIREWALL|ST_FW2FW)) == ST_FIREWALL &&
+			(dq->query_flags & QUERY_SPEED_FIREWALLED)
+		))
+	) {
+		if (dq_debug > 19) {
+			if (dq->flags & DQ_F_LINGER)
+				g_message("DQ[%d] %s(%d secs; +%d secs) +%d ignored (firewall)",
+					dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
+					(gint) (tm_time() - dq->start),
+					(gint) (tm_time() - dq->stop),
+					count);
+			else
+				g_message("DQ[%d] %s(%d secs) +%d ignored (firewall)",
+					dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
+					(gint) (tm_time() - dq->start),
+					count);
+		}
+
+		return FALSE;		/* Don't forward those results */
+	}
+
 	if (dq->flags & DQ_F_LINGER)
 		dq->linger_results += count;
 	else if (oob)
@@ -1612,16 +1867,16 @@ dq_count_results(gchar *muid, gint count, gboolean oob)
 		if (dq->node_id == NODE_ID_LOCAL)
 			dq->kept_results = search_get_kept_results_by_handle(dq->sh);
 		if (dq->flags & DQ_F_LINGER)
-			printf("DQ[%d] %s(%d secs; +%d secs) "
-				"+%d %slinger_results=%d kept=%d\n",
+			g_message("DQ[%d] %s(%d secs; +%d secs) "
+				"+%d %slinger_results=%d kept=%d",
 				dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
 				(gint) (tm_time() - dq->start),
 				(gint) (tm_time() - dq->stop),
 				count, oob ? "OOB " : "",
 				dq->linger_results, dq->kept_results);
 		else
-			printf("DQ[%d] %s(%d secs) "
-				"+%d %sresults=%d new=%d kept=%d oob=%d\n",
+			g_message("DQ[%d] %s(%d secs) "
+				"+%d %sresults=%d new=%d kept=%d oob=%d",
 				dq->qid, dq->node_id == NODE_ID_LOCAL ? "local " : "",
 				(gint) (tm_time() - dq->start),
 				count, oob ? "OOB " : "",
@@ -1637,14 +1892,18 @@ dq_count_results(gchar *muid, gint count, gboolean oob)
  * If we have a dynamic query registered for the MUID, increase the result
  * count.
  *
+ * @param muid		the query's MUID
+ * @param count		how many results we parsed
+ * @param status	result set `status' flags gathered during parsing
+ *
  * @return FALSE if the query was explicitly cancelled by the user and
  * results should be dropped, TRUE otherwise.  In other words, returns
  * whether we should forward the results.
  */
 gboolean
-dq_got_results(gchar *muid, guint count)
+dq_got_results(const gchar *muid, guint count, guint32 status)
 {
-	return dq_count_results(muid, count, FALSE);
+	return dq_count_results(muid, count, status, FALSE);
 }
 
 /**
@@ -1655,9 +1914,9 @@ dq_got_results(gchar *muid, guint count)
  * results should not be claimed.
  */
 gboolean
-dq_oob_results_ind(gchar *muid, gint count)
+dq_oob_results_ind(const gchar *muid, gint count)
 {
-	return dq_count_results(muid, count, TRUE);
+	return dq_count_results(muid, count, 0, TRUE);
 }
 
 /**
@@ -1704,12 +1963,20 @@ dq_oob_results_got(const gchar *muid, guint count)
  * The special value 0xffff is a request to stop the query immediately.
  */
 void
-dq_got_query_status(gchar *muid, guint32 node_id, guint16 kept)
+dq_got_query_status(const gchar *muid, guint32 node_id, guint16 kept)
 {
 	dquery_t *dq;
 
 	dq = g_hash_table_lookup(by_muid, muid);
 
+	/*
+	 * Could be an OOB-proxied query, but the leaf does not know the MUID
+	 * we're using, only the one it generated.
+	 */
+
+	if (dq == NULL)
+		dq = g_hash_table_lookup(by_leaf_muid, muid);
+
 	if (dq == NULL)
 		return;
 
@@ -1718,14 +1985,30 @@ dq_got_query_status(gchar *muid, guint32 node_id, guint16 kept)
 
 	dq->kept_results = kept;
 	dq->flags |= DQ_F_GOT_GUIDANCE;
+	dq->last_status = dq->up_sent;
+	dq->new_results = 0;
+
+	if (!(dq->flags & DQ_F_WAITING)) {
+		/* Got unsolicited guidance */
+
+		if (!(dq->flags & DQ_F_LEAF_GUIDED)) {
+			node_set_leaf_guidance(node_id, TRUE);
+			dq->flags |= DQ_F_LEAF_GUIDED;
+
+			if (dq_debug > 19)
+				g_message(
+					"DQ[%d] (%d secs) turned on leaf-guidance for node #%u",
+					dq->qid, (gint) (tm_time() - dq->start), dq->node_id);
+		}
+	}
 
 	if (dq_debug > 19) {
 		if (dq->flags & DQ_F_LINGER)
-			printf("DQ[%d] (%d secs; +%d secs) kept_results=%d\n",
+			g_message("DQ[%d] (%d secs; +%d secs) kept_results=%d",
 				dq->qid, (gint) (tm_time() - dq->start),
 				(gint) (tm_time() - dq->stop), dq->kept_results);
 		else
-			printf("DQ[%d] (%d secs) %ssolicited, kept_results=%d\n",
+			g_message("DQ[%d] (%d secs) %ssolicited, kept_results=%d",
 				dq->qid, (gint) (tm_time() - dq->start),
 				(dq->flags & DQ_F_WAITING) ? "" : "un", dq->kept_results);
 	}
@@ -1739,8 +2022,9 @@ dq_got_query_status(gchar *muid, guint32 node_id, guint16 kept)
 	 */
 
 	if (kept == 0xffff) {
-		if (dq_debug > 19)
-			printf("DQ[%d] terminating at user's request\n", dq->qid);
+		if (dq_debug)
+			g_message("DQ[%d] terminating at user's request (queried %u UP%s)",
+				dq->qid, dq->up_sent, dq->up_sent == 1 ? "" : "s");
 
 		dq->flags |= DQ_F_USR_CANCELLED;
 
@@ -1834,7 +2118,7 @@ dq_search_closed(gnet_search_t handle)
  * any more, in which case nothing is returned into `wanted'.
  */
 gboolean
-dq_get_results_wanted(gchar *muid, guint32 *wanted)
+dq_get_results_wanted(const gchar *muid, guint32 *wanted)
 {
 	dquery_t *dq;
 
@@ -1847,7 +2131,30 @@ dq_get_results_wanted(gchar *muid, guint32 *wanted)
 		*wanted = 0;
 	else {
 		guint32 kept = dq_kept_results(dq);
-		*wanted = (kept > dq->max_results) ? 0 : dq->max_results - kept;
+
+		/*
+		 * d->kept_results is the true amount of total results they got, which
+		 * is different from the value returned by dq_kept_results() which
+		 * performs an average over the expected amount of UPs a leaf will have.
+		 *
+		 * When we have delivered all the hits we had to, but OOB replies still
+		 * come through, we continue to claim until the reported amount of
+		 * kept entries for this search reaches the big finalizing value.
+		 * The rationale here is that results are not necessarily filtered,
+		 * and we're getting hits without much Gnutella cost because we have
+		 * already stopped querying if we already got max_results.
+		 *		--RAM, 2006-08-16
+		 */
+
+		if (kept < dq->max_results)
+			*wanted = dq->max_results - kept;
+		else if (
+			(dq->flags & DQ_F_GOT_GUIDANCE) &&
+			dq->kept_results < dq->fin_results
+		)
+			*wanted = 1;		/* Could be discarded later by the DH layer */
+		else
+			*wanted = 0;
 	}
 
 	return TRUE;
@@ -1859,9 +2166,10 @@ dq_get_results_wanted(gchar *muid, guint32 *wanted)
 void
 dq_init(void)
 {
-	dqueries = g_hash_table_new(g_direct_hash, 0);
+	dqueries = g_hash_table_new(NULL, NULL);
 	by_node_id = g_hash_table_new(NULL, NULL);
 	by_muid = g_hash_table_new(guid_hash, guid_eq);
+	by_leaf_muid = g_hash_table_new(guid_hash, guid_eq);
 	fill_hosts();
 }
 
@@ -1923,6 +2231,20 @@ free_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
 	atom_guid_free(key);
 }
 
+/**
+ * Hashtable iteration callback to free the MUIDs in the `by_leaf_muid' table.
+ * Normally, after having freed the dqueries table, there should not be
+ * anything remaining, hence warn!
+ */
+static void
+free_leaf_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
+{
+	(void) unused_value;
+	(void) unused_udata;
+	g_warning("remained un-freed leaf MUID \"%s\" in dynamic queries",
+		guid_hex_str(key));
+}
+
 /**
  * Cleanup data structures used by dynamic querying.
  */
@@ -1937,6 +2259,9 @@ dq_close(void)
 
 	g_hash_table_foreach(by_muid, free_muid, NULL);
 	g_hash_table_destroy(by_muid);
+
+	g_hash_table_foreach(by_leaf_muid, free_leaf_muid, NULL);
+	g_hash_table_destroy(by_leaf_muid);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
