@@ -1,5 +1,5 @@
 /*
- * $Id: ggep_type.h,v 1.10 2005/08/17 13:44:05 cbiere Exp $
+ * $Id: ggep_type.h 11743 2006-08-26 14:11:32Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -65,7 +65,7 @@ struct ggep_gtkgv1 {
 	guint8 patch;
 	guint8 revchar;
 	guint32 release;
-	guint32 start;
+	guint32 build;
 };
 
 ggept_status_t ggept_gtkgv1_extract(extvec_t *exv, struct ggep_gtkgv1 *info);
