@@ -1,5 +1,5 @@
 /*
- * $Id: gmsg.c,v 1.22 2005/12/17 09:25:44 daichik Exp $
+ * $Id: gmsg.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gmsg.c,v 1.22 2005/12/17 09:25:44 daichik Exp $");
+RCSID("$Id: gmsg.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "gmsg.h"
 #include "pmsg.h"
@@ -45,11 +45,14 @@ RCSID("$Id: gmsg.c,v 1.22 2005/12/17 09:25:44 daichik Exp $");
 #include "routing.h"
 #include "vmsg.h"
 #include "search.h"
+#include "gnet_stats.h"
 
 #include "if/gnet_property_priv.h"
 
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/walloc.h"
+#include "lib/zlib_util.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 static const gchar *msg_name[256];
@@ -63,11 +66,55 @@ static guint8 msg_weight[256];	/**< For gmsg_cmp() */
  * @param size	the payload plus header size of the gnutella message.
  */
 static inline void
-gmsg_header_check(const struct gnutella_header *h, guint32 size)
+gmsg_header_check(gconstpointer msg, guint32 size)
 {
-	g_assert(h->ttl > 0);
+	g_assert(gnutella_header_get_ttl(msg) > 0);
 	g_assert(size >= GTA_HEADER_SIZE);
-	g_assert(peek_le32(h->size) == size - GTA_HEADER_SIZE);
+	g_assert(gnutella_header_get_size(msg) == size - GTA_HEADER_SIZE);
+}
+
+/**
+ * Returns whether the size field is properly architected as flags and size.
+ *
+ * @param msg		the head of the message
+ * @param size		where the message size is returned if valid
+ *
+ * @return GMSG_VALID if we can process the message, GMSG_INVALID if the
+ * message should be dropped and the connection closed, GMSG_VALID_NO_PROCESS
+ * if the message is valid but cannot be interpreted locally.
+ */
+gmsg_valid_t
+gmsg_size_valid(gconstpointer msg, guint16 *size)
+{
+	guint32 raw_size = gnutella_header_get_size(msg);
+	guint16 payload_size = (guint16) (raw_size & GTA_SIZE_MASK);
+	
+	if (raw_size == payload_size)
+		goto ok;
+
+	if (raw_size & GTA_SIZE_MARKED) {
+		guint32 flags = raw_size & ~GTA_SIZE_MASK;
+		flags &= ~GTA_SIZE_MARKED;
+
+		*size = payload_size;
+
+		if (flags == 0)
+			return GMSG_VALID_MARKED;
+
+		/*
+		 * We don't know how to handle flags yet -- they are undefined.
+		 * However, the message IS valid and could be relayed possibly.
+		 * But we cannot interpret it.
+		 */
+
+		return GMSG_VALID_NO_PROCESS;
+	}
+
+	return GMSG_INVALID;
+
+ok:
+	*size = payload_size;
+	return GMSG_VALID;
 }
 
 /**
@@ -109,28 +156,29 @@ gmsg_init(void)
 {
 	gint i;
 
-	for (i = 0; i < 256; i++)
-		msg_name[i] = "unknown";
+	for (i = 0; i < 256; i++) {
+		const gchar *s;
+		guint w;
 
-	msg_name[GTA_MSG_INIT]				= "ping";
-	msg_name[GTA_MSG_INIT_RESPONSE]		= "pong";
-	msg_name[GTA_MSG_BYE]				= "bye";
-	msg_name[GTA_MSG_SEARCH]			= "query";
-	msg_name[GTA_MSG_SEARCH_RESULTS]	= "query hit";
-	msg_name[GTA_MSG_PUSH_REQUEST]		= "push";
-	msg_name[GTA_MSG_QRP]				= "QRP";
-	msg_name[GTA_MSG_VENDOR]			= "vendor";
+		s = "unknown";
+		w = 0;
 
-	for (i = 0; i < 256; i++)
-		msg_weight[i] = 0;
-
-	msg_weight[GTA_MSG_INIT]			= 1;
-	msg_weight[GTA_MSG_SEARCH]			= 2;
-	msg_weight[GTA_MSG_INIT_RESPONSE]	= 3;
-	msg_weight[GTA_MSG_SEARCH_RESULTS]	= 4;
-	msg_weight[GTA_MSG_PUSH_REQUEST]	= 5;
-	msg_weight[GTA_MSG_QRP]				= 6;
-	msg_weight[GTA_MSG_VENDOR]			= 7;	/* deemed important */
+		switch ((enum gta_msg) i) {
+		case GTA_MSG_INIT:           w = 1; s = "Ping"; break;
+		case GTA_MSG_INIT_RESPONSE:  w = 3; s = "Pong"; break;
+		case GTA_MSG_SEARCH:         w = 2; s = "Query"; break;
+		case GTA_MSG_SEARCH_RESULTS: w = 4; s = "Q-Hit"; break;
+		case GTA_MSG_PUSH_REQUEST:   w = 5; s = "Push"; break;
+		case GTA_MSG_RUDP:   		 		s = "RUDP"; break;
+		case GTA_MSG_VENDOR:         		s = "Vndor"; break;
+		case GTA_MSG_STANDARD:       		s = "Vstd"; break;
+		case GTA_MSG_QRP:            w = 6; s = "QRP"; break;
+		case GTA_MSG_HSEP_DATA:      		s = "HSEP"; break;
+		case GTA_MSG_BYE:      		 w = 7; s = "BYE"; break;
+		}
+		msg_name[i] = s;
+		msg_weight[i] = w;
+	}
 }
 
 /**
@@ -161,6 +209,108 @@ gmsg_to_pmsg(gconstpointer msg, guint32 size)
 	return mb;
 }
 
+/**
+ * Construct compressed control PDU descriptor from message, for UDP traffic.
+ * The message payload is deflated only when the resulting size is smaller
+ * than the raw uncompressed form.
+ *
+ * Message data is copied into the new data buffer, so caller may release
+ * its memory.
+ */
+pmsg_t *
+gmsg_to_deflated_pmsg(gconstpointer msg, guint32 size)
+{
+	guint32 plen = size - GTA_HEADER_SIZE;		/* Raw payload length */
+	guint32 blen = plen + (plen >> 4) + 12;		/* 1.0625 times orginal */
+	gpointer buf;								/* Compression made there */
+	gconstpointer data;							/* Raw data start */
+	guint32 deflated_length;					/* Length of deflated data */
+	zlib_deflater_t *z;
+	pmsg_t *mb;
+
+	/*
+	 * Since there is a 2-byte header added to each deflated stream, plus
+	 * a trailing 16-bit checksum, it's no use to attempt deflation if
+	 * the payload has less than 5 bytes.
+	 */
+
+	if (plen <= 5)
+		return gmsg_to_pmsg(msg, size);
+
+	/*
+	 * Compress payload into newly allocated buffer.
+	 */
+
+	buf = walloc(blen);
+	data = (const gchar *) msg + GTA_HEADER_SIZE;
+	z = zlib_deflater_make_into(data, plen, buf, blen, Z_DEFAULT_COMPRESSION);
+
+	switch (zlib_deflate(z, plen)) {
+	case -1:
+		goto send_raw;
+		break;
+	case 0:
+		break;
+	case 1:
+		g_error("did not deflate the whole input");
+		break;
+	}
+
+	g_assert(zlib_deflater_closed(z));
+
+	/*
+	 * Check whether compressed data is smaller than the original payload.
+	 */
+
+	deflated_length = zlib_deflater_outlen(z);
+
+	g_assert(deflated_length <= blen);
+	g_assert(zlib_is_valid_header(buf, deflated_length));
+
+	if (deflated_length >= plen) {
+		if (udp_debug)
+			g_message("UDP not deflating %s into %d bytes",
+				gmsg_infostr_full(msg), deflated_length);
+
+		gnet_stats_count_general(GNR_UDP_LARGER_HENCE_NOT_COMPRESSED, 1);
+		goto send_raw;
+	}
+
+	/*
+	 * OK, we gain something so we'll send this payload deflated.
+	 */
+
+	mb = gmsg_split_to_pmsg(msg, buf, deflated_length + GTA_HEADER_SIZE);
+
+	wfree(buf, blen);
+	zlib_deflater_free(z, FALSE);
+
+	if (udp_debug)
+		g_message("UDP deflated %s into %d bytes",
+			gmsg_infostr_full(msg), deflated_length);
+
+	{
+		gpointer header;
+		
+		header = pmsg_start(mb);
+		gnutella_header_set_ttl(header,
+			gnutella_header_get_ttl(header) | GTA_UDP_DEFLATED);
+		gnutella_header_set_size(header, deflated_length);
+	}
+
+	return mb;
+
+send_raw:
+	/*
+	 * Cleanup and send payload as-is (uncompressed).
+	 */
+
+	wfree(buf, blen);
+	zlib_deflater_free(z, FALSE);
+
+	return gmsg_to_pmsg(msg, size);
+}
+
 /**
  * Construct control PDU descriptor from message.
  */
@@ -178,8 +328,8 @@ gmsg_to_ctrl_pmsg(gconstpointer msg, guint32 size)
  * Construct extended control PDU (with free routine) from message.
  */
 pmsg_t *
-gmsg_to_ctrl_pmsg_extend(
-	gconstpointer msg, guint32 size, pmsg_free_t free_cb, gpointer arg)
+gmsg_to_ctrl_pmsg_extend(gconstpointer msg, guint32 size,
+	pmsg_free_t free_cb, gpointer arg)
 {
 	pmsg_t *mb;
 
@@ -205,6 +355,10 @@ write_message(pmsg_t *mb, gconstpointer head, gconstpointer data, guint32 size)
 
 /**
  * Construct PDU from header and data.
+ *
+ * @param head		pointer to the Gnutella header
+ * @param data		pointer to the Gnutella payload
+ * @param size		the total size of the message, header + payload
  */
 pmsg_t *
 gmsg_split_to_pmsg(gconstpointer head, gconstpointer data, guint32 size)
@@ -325,8 +479,7 @@ gmsg_sendto_one_ggep(struct gnutella_node *n,
 	if (NODE_CAN_GGEP(n)) {
 		mq_putq(n->outq, gmsg_to_pmsg(msg, size));
 	} else {
-		const struct gnutella_header *msg_head = msg;
-		struct gnutella_header head;
+		gnutella_header_t head;
 		pmsg_t *mb;
 
 		/*
@@ -334,9 +487,10 @@ gmsg_sendto_one_ggep(struct gnutella_node *n,
 		 * stripped and a corrected size field in the header
 		 */
 
-		head = *msg_head;
-		WRITE_GUINT32_LE(regular_size - GTA_HEADER_SIZE, head.size);
-		mb = gmsg_split_to_pmsg(&head, &msg_head[1], regular_size);
+		memcpy(head, msg, sizeof head);
+		gnutella_header_set_size(&head, regular_size - GTA_HEADER_SIZE);
+		mb = gmsg_split_to_pmsg(&head, (const gchar *) msg + GTA_HEADER_SIZE,
+				regular_size);
 		mq_putq(n->outq, pmsg_clone(mb));
 		pmsg_free(mb);
 	}
@@ -392,7 +546,7 @@ gmsg_search_sendto_one(
 	struct gnutella_node *n, gnet_search_t sh, gconstpointer msg, guint32 size)
 {
 	gmsg_header_check(msg, size);
-	g_assert(((struct gnutella_header *) msg)->hops <= hops_random_factor);
+	g_assert(gnutella_header_get_hops(msg) <= hops_random_factor);
 
 	if (!NODE_IS_WRITABLE(n))
 		return;
@@ -489,7 +643,7 @@ gmsg_search_sendto_all(
 	pmsg_t *mb = gmsg_to_pmsg(msg, size);
 
 	gmsg_header_check(msg, size);
-	g_assert(((struct gnutella_header *) msg)->hops <= hops_random_factor);
+	g_assert(gnutella_header_get_hops(msg) <= hops_random_factor);
 
 	if (gmsg_debug > 5 && gmsg_hops(msg) == 0)
 		gmsg_dump(stdout, msg, size);
@@ -530,8 +684,8 @@ gmsg_split_sendto_all_but_one(const GSList *sl, const struct gnutella_node *n,
 
 	if (
 		current_peermode == NODE_P_ULTRA &&
-		((struct gnutella_header *) head)->function == GTA_MSG_SEARCH &&
-		((struct gnutella_header *) head)->ttl == 1
+		gnutella_header_get_function(head) == GTA_MSG_SEARCH &&
+		gnutella_header_get_ttl(head) == 1
 	)
 		skip_up_with_qrp = TRUE;
 
@@ -547,6 +701,8 @@ gmsg_split_sendto_all_but_one(const GSList *sl, const struct gnutella_node *n,
 			continue;
 		if (skip_up_with_qrp && NODE_UP_QRP(dn))
 			continue;
+		if (n->header_flags && !NODE_CAN_SFLAG(dn))
+			continue;
 		mq_putq(dn->outq, pmsg_clone(mb));
 	}
 
@@ -617,8 +773,8 @@ gmsg_split_sendto_all_but_one_ggep(
 
 	if (
 		current_peermode == NODE_P_ULTRA &&
-		((struct gnutella_header *) head)->function == GTA_MSG_SEARCH &&
-		((struct gnutella_header *) head)->ttl == 1
+		gnutella_header_get_function(head) == GTA_MSG_SEARCH &&
+		gnutella_header_get_ttl(head) == 1
 	)
 		skip_up_with_qrp = TRUE;
 
@@ -632,15 +788,17 @@ gmsg_split_sendto_all_but_one_ggep(
 			continue;
 		if (skip_up_with_qrp && NODE_UP_QRP(dn))
 			continue;
+		if (n->header_flags && !NODE_CAN_SFLAG(dn))
+			continue;
 		if (NODE_CAN_GGEP(dn))
 			mq_putq(dn->outq, pmsg_clone(mb));
 		else {
 			if (mb_stripped == NULL) {
-				const struct gnutella_header *h = head;
-				struct gnutella_header nhead;
+				gnutella_header_t nhead;
 
-				nhead = *h;
-				WRITE_GUINT32_LE(regular_size - GTA_HEADER_SIZE, nhead.size);
+				memcpy(nhead, head, sizeof nhead);
+				gnutella_header_set_size(&nhead,
+					regular_size - GTA_HEADER_SIZE);
 				mb_stripped = gmsg_split_to_pmsg(&nhead, data, regular_size);
 			}
 			mq_putq(dn->outq, pmsg_clone(mb_stripped));
@@ -667,6 +825,18 @@ gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt)
 	case ROUTE_NONE:
 		return;
 	case ROUTE_ONE:
+		/*
+		 * If message has size flags and the recipoent cannot understand it,
+		 * then too bad but we have to drop that message.  We account it
+		 * as dropped because this message was meant to be routed to one
+		 * recipient only.
+		 */
+
+		if (n->header_flags && !NODE_CAN_SFLAG(rt_node)) {
+			gnet_stats_count_dropped(n, MSG_DROP_BAD_SIZE);
+			return;
+		}
+
 		gmsg_split_sendto_one(rt_node, &n->header,
 				n->data, n->size + GTA_HEADER_SIZE);
 		return;
@@ -683,6 +853,8 @@ gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt)
 	case ROUTE_MULTI:
 		for (sl = rt->ur.u_nodes; sl; sl = g_slist_next(sl)) {
 			rt_node = (struct gnutella_node *) sl->data;
+			if (n->header_flags && !NODE_CAN_SFLAG(rt_node))
+				continue;
 			gmsg_split_sendto_one(rt_node,
 				(guchar *) &n->header, n->data, n->size + GTA_HEADER_SIZE);
 		}
@@ -709,10 +881,10 @@ sendto_ggep(struct gnutella_node *n, struct gnutella_node *dn,
 		gmsg_split_sendto_one(dn, &n->header,
 				n->data, n->size + GTA_HEADER_SIZE);
 	} else {
-		WRITE_GUINT32_LE(regular_size, n->header.size);
+		gnutella_header_set_size(&n->header, regular_size);
 		gmsg_split_sendto_one(dn, &n->header,
 				n->data, regular_size + GTA_HEADER_SIZE);
-		WRITE_GUINT32_LE(n->size, n->header.size);
+		gnutella_header_set_size(&n->header, n->size);
 	}
 }
 
@@ -739,6 +911,18 @@ gmsg_sendto_route_ggep(struct gnutella_node *n, struct route_dest *rt,
 	case ROUTE_NONE:
 		return;
 	case ROUTE_ONE:
+		/*
+		 * If message has size flags and the recipoent cannot understand it,
+		 * then too bad but we have to drop that message.  We account it
+		 * as dropped because this message was meant to be routed to one
+		 * recipient only.
+		 */
+
+		if (n->header_flags && !NODE_CAN_SFLAG(rt_node)) {
+			gnet_stats_count_dropped(n, MSG_DROP_BAD_SIZE);
+			return;
+		}
+
 		sendto_ggep(n, rt_node, regular_size);
 		return;
 	case ROUTE_ALL_BUT_ONE:
@@ -754,6 +938,8 @@ gmsg_sendto_route_ggep(struct gnutella_node *n, struct route_dest *rt,
 	case ROUTE_MULTI:
 		for (sl = rt->ur.u_nodes; sl; sl = g_slist_next(sl)) {
 			rt_node = sl->data;
+			if (n->header_flags && !NODE_CAN_SFLAG(rt_node))
+				continue;
 			sendto_ggep(n, rt_node, regular_size);
 		}
 		return;
@@ -782,24 +968,23 @@ static gboolean
 gmsg_query_can_send(pmsg_t *mb, const mqueue_t *q)
 {
 	gnutella_node_t *n = mq_node(q);
-	gconstpointer start = pmsg_start(mb);
-	const struct gnutella_header *head = start;
+	gconstpointer msg = pmsg_start(mb);
 
-	g_assert(head->function == GTA_MSG_SEARCH);
+	g_assert(GTA_MSG_SEARCH == gnutella_header_get_function(msg));
 
-	if (!node_query_hops_ok(n, head->hops)) {
+	if (!node_query_hops_ok(n, gnutella_header_get_hops(msg))) {
 		if (gmsg_debug > 4)
-			gmsg_log_dropped(start, "to node %s due to hops-flow",
+			gmsg_log_dropped(msg, "to node %s due to hops-flow",
 				node_addr(n));
 		return FALSE;
 	}
 
-	if (gmsg_is_oob_query(start))
+	if (gmsg_is_oob_query(msg))
 		return TRUE;
 
-	if (!route_exists_for_reply(start, head->function)) {
+	if (!route_exists_for_reply(msg, gnutella_header_get_function(msg))) {
 		if (gmsg_debug > 4)
-			gmsg_log_dropped(start, "to node %s due to no route for hits",
+			gmsg_log_dropped(msg, "to node %s due to no route for hits",
 				node_addr(n));
 		return FALSE;
 	}
@@ -813,10 +998,9 @@ gmsg_query_can_send(pmsg_t *mb, const mqueue_t *q)
 void
 gmsg_install_presend(pmsg_t *mb)
 {
-	gconstpointer start = pmsg_start(mb);
-	const struct gnutella_header *head = start;
+	gconstpointer msg = pmsg_start(mb);
 
-	if (GTA_MSG_SEARCH == head->function) {
+	if (GTA_MSG_SEARCH == gnutella_header_get_function(msg)) {
 		pmsg_check_t old = pmsg_set_check(mb, gmsg_query_can_send);
 		g_assert(NULL == old);
 	}
@@ -832,12 +1016,10 @@ gmsg_install_presend(pmsg_t *mb)
 gboolean
 gmsg_can_drop(gconstpointer pdu, gint size)
 {
-	const struct gnutella_header *head = pdu;
-
-	if ((size_t) size < sizeof(struct gnutella_header))
+	if ((size_t) size < GTA_HEADER_SIZE)
 		return TRUE;
 
-	switch (head->function) {
+	switch (gnutella_header_get_function(pdu)) {
 	case GTA_MSG_INIT:
 	case GTA_MSG_SEARCH:
 	case GTA_MSG_INIT_RESPONSE:
@@ -855,8 +1037,11 @@ gmsg_can_drop(gconstpointer pdu, gint size)
 gint
 gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
 {
-	const struct gnutella_header *h1 = pdu1, *h2 = pdu2;
-	gint w1 = msg_weight[h1->function], w2 = msg_weight[h2->function];
+	gconstpointer h1 = pdu1, h2 = pdu2;
+	gint w1, w2;
+	
+	w1 = msg_weight[gnutella_header_get_function(h1)];
+	w2 = msg_weight[gnutella_header_get_function(h2)];
 
 	/*
 	 * The more weight a message type has, the more prioritary it is.
@@ -878,23 +1063,26 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
 	 * to its destination (lowest TTL).
 	 */
 
-	if (h1->hops == h2->hops) {
-		switch (h1->function) {
+	if (gnutella_header_get_hops(h1) == gnutella_header_get_hops(h2)) {
+		switch (gnutella_header_get_function(h1)) {
 		case GTA_MSG_PUSH_REQUEST:
 		case GTA_MSG_SEARCH_RESULTS:
-			return CMP(h2->ttl, h1->ttl);
+			return CMP(gnutella_header_get_ttl(h2),
+						gnutella_header_get_ttl(h1));
 		default:
 			return 0;
 		}
-	}
-
-	switch (h1->function) {
-	case GTA_MSG_INIT:
-	case GTA_MSG_SEARCH:
-	case GTA_MSG_QRP:
-		return h1->hops > h2->hops ? -1 : +1;
-	default:
-		return h1->hops < h2->hops ? -1 : +1;
+	} else {
+		switch (gnutella_header_get_function(h1)) {
+		case GTA_MSG_INIT:
+		case GTA_MSG_SEARCH:
+		case GTA_MSG_QRP:
+			return gnutella_header_get_hops(h1) > gnutella_header_get_hops(h2)
+					? -1 : +1;
+		default:
+			return gnutella_header_get_hops(h1) < gnutella_header_get_hops(h2)
+					? -1 : +1;
+		}
 	}
 }
 
@@ -908,31 +1096,24 @@ gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2)
  * payload of that message.
  */
 gchar *
-gmsg_infostr_full(gconstpointer message)
+gmsg_infostr_full(gconstpointer msg)
 {
-	static gchar a[160];
-	const struct gnutella_header *h = message;
-	guint32 size;
-	const gchar *data;
+	const gchar *data = (const gchar *) msg + GTA_HEADER_SIZE;
 
-	READ_GUINT32_LE(h->size, size);
-	data = (gchar *) message + sizeof(*h);
+	return gmsg_infostr_full_split(msg, data);
+}
 
-	switch (h->function) {
-	case GTA_MSG_VENDOR:
-	case GTA_MSG_STANDARD:
-		gm_snprintf(a, sizeof(a), "%s %s (%u byte%s) [hops=%d, TTL=%d]",
-			gmsg_name(h->function), vmsg_infostr(data, size),
-			size, size == 1 ? "" : "s", h->hops, h->ttl);
-		break;
-	default:
-		gm_snprintf(a, sizeof(a), "%s (%u byte%s) [hops=%d, TTL=%d]",
-			gmsg_name(h->function),
-			size, size == 1 ? "" : "s", h->hops, h->ttl);
-		break;
-	}
+static size_t
+gmsg_infostr_to_buf(gconstpointer msg, gchar *buf, size_t buf_size)
+{
+	guint16 size = gmsg_size(msg);
 
-	return a;
+	return gm_snprintf(buf, buf_size, "%s (%u byte%s) %s[hops=%d, TTL=%d]",
+		gmsg_name(gnutella_header_get_function(msg)),
+		size, size == 1 ? "" : "s",
+		gnutella_header_get_ttl(msg) & GTA_UDP_DEFLATED ? "deflated " : "",
+		gnutella_header_get_hops(msg),
+		gnutella_header_get_ttl(msg) & ~GTA_UDP_DEFLATED);
 }
 
 /**
@@ -947,23 +1128,26 @@ gchar *
 gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
 {
 	static gchar a[160];
-	const struct gnutella_header *h = (const struct gnutella_header *) head;
-	guint32 size;
 
-	READ_GUINT32_LE(h->size, size);
-
-	switch (h->function) {
+	switch (gnutella_header_get_function(head)) {
 	case GTA_MSG_VENDOR:
 	case GTA_MSG_STANDARD:
-		gm_snprintf(a, sizeof(a), "%s %s (%u byte%s) [hops=%d, TTL=%d]",
-			gmsg_name(h->function), vmsg_infostr(data, size),
-			size, size == 1 ? "" : "s", h->hops, h->ttl);
+		{
+			guint16 size = gmsg_size(head);
+			guint8 ttl = gnutella_header_get_ttl(head);
+			
+			gm_snprintf(a, sizeof(a), "%s %s (%u byte%s) %s[hops=%d, TTL=%d]",
+				gmsg_name(gnutella_header_get_function(head)),
+				vmsg_infostr(data, size),
+				size, size == 1 ? "" : "s",
+				ttl & GTA_UDP_DEFLATED ? "deflated " : 
+					ttl & GTA_UDP_CAN_INFLATE ? "can_inflate " : "",
+				gnutella_header_get_hops(head),
+				ttl & ~(GTA_UDP_DEFLATED | GTA_UDP_CAN_INFLATE));
+		}
 		break;
 	default:
-		gm_snprintf(a, sizeof(a), "%s (%u byte%s) [hops=%d, TTL=%d]",
-			gmsg_name(h->function),
-			size, size == 1 ? "" : "s", h->hops, h->ttl);
-		break;
+		gmsg_infostr_to_buf(head, a, sizeof a);
 	}
 
 	return a;
@@ -974,47 +1158,33 @@ gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
  *
  *     msg_type (payload length) [hops=x, TTL=x]
  */
-gchar *
-gmsg_infostr(gconstpointer head)
+const gchar *
+gmsg_infostr(gconstpointer msg)
 {
-	static gchar a[80];
-	const struct gnutella_header *h = head;
-	guint32 size;
-
-	READ_GUINT32_LE(h->size, size);
-
-	gm_snprintf(a, sizeof(a), "%s (%u byte%s) [hops=%d, TTL=%d]",
-		gmsg_name(h->function), size, size == 1 ? "" : "s", h->hops, h->ttl);
-
-	return a;
+	static gchar buf[80];
+	gmsg_infostr_to_buf(msg, buf, sizeof buf);
+	return buf;
 }
 
 /**
  * Same as gmsg_infostr(), but different static buffer.
  */
 static gchar *
-gmsg_infostr2(gconstpointer head)
+gmsg_infostr2(gconstpointer msg)
 {
-	static gchar a[80];
-	const struct gnutella_header *h = head;
-	guint32 size;
-
-	READ_GUINT32_LE(h->size, size);
-
-	gm_snprintf(a, sizeof(a), "%s (%u byte%s) [hops=%d, TTL=%d]",
-		gmsg_name(h->function), size, size == 1 ? "" : "s", h->hops, h->ttl);
-
-	return a;
+	static gchar buf[80];
+	gmsg_infostr_to_buf(msg, buf, sizeof buf);
+	return buf;
 }
 
 /**
  * Log dropped message, and reason.
  */
 void
-gmsg_log_dropped(gconstpointer head, const gchar *reason, ...)
+gmsg_log_dropped(gconstpointer msg, const gchar *reason, ...)
 {
 	fputs("DROP ", stdout);
-	fputs(gmsg_infostr2(head), stdout);	/* Allows gmsg_infostr() in arglist */
+	fputs(gmsg_infostr2(msg), stdout);	/* Allows gmsg_infostr() in arglist */
 
 	if (reason) {
 		va_list args;
@@ -1033,7 +1203,7 @@ gmsg_log_dropped(gconstpointer head, const gchar *reason, ...)
 void
 gmsg_log_bad(const struct gnutella_node *n, const gchar *reason, ...)
 {
-	printf("BAD <%s> ", node_vendor(n));
+	printf("BAD <%s> %s ", node_vendor(n), node_addr(n));
 
 	fputs(gmsg_infostr_full_split(&n->header, n->data), stdout);
 
@@ -1055,13 +1225,12 @@ gmsg_log_bad(const struct gnutella_node *n, const gchar *reason, ...)
 gboolean
 gmsg_is_oob_query(gconstpointer msg)
 {
-	const struct gnutella_header *h = msg;
 	gconstpointer data = (const gchar *) msg + GTA_HEADER_SIZE;
 	guint16 req_speed;
 
-	g_assert(h->function == GTA_MSG_SEARCH);
+	g_assert(GTA_MSG_SEARCH == gnutella_header_get_function(msg));
 
-	READ_GUINT16_LE(data, req_speed);
+	req_speed = peek_le16(data);
 
 	return (req_speed & (QUERY_SPEED_MARK | QUERY_SPEED_OOB_REPLY)) ==
 		(QUERY_SPEED_MARK | QUERY_SPEED_OOB_REPLY);
@@ -1074,12 +1243,11 @@ gmsg_is_oob_query(gconstpointer msg)
 gboolean
 gmsg_split_is_oob_query(gconstpointer head, gconstpointer data)
 {
-	const struct gnutella_header *h = head;
 	guint16 req_speed;
 
-	g_assert(h->function == GTA_MSG_SEARCH);
+	g_assert(GTA_MSG_SEARCH == gnutella_header_get_function(head));
 
-	READ_GUINT16_LE(data, req_speed);
+	req_speed = peek_le16(data);
 
 	return (req_speed & (QUERY_SPEED_MARK | QUERY_SPEED_OOB_REPLY)) ==
 		(QUERY_SPEED_MARK | QUERY_SPEED_OOB_REPLY);
