@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c,v 1.20 2005/05/04 17:37:22 graaff Exp $
+ * $Id: hcache.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -29,24 +29,30 @@
  *
  * Host cache management.
  *
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2002-2003
+ *
+ * @todo
  * TODO:
  *
  *	- finer grained stats:
- *		- hits/misses while adding,
- *		- hits/misses while bad checking
- *		- how many hosts were tried to connect to?
+ *		-# hits/misses while adding,
+ *		-# hits/misses while bad checking
+ *		-# how many hosts were tried to connect to?
  *	- display stats about gwcache usage:
- *		- how often
- *		- how many hosts got
+ *		-# how often
+ *		-# how many hosts got
  *	- move unstable servant code from nodes.c to hcache.c
  *	- make sure hosts we are currently connected too are also saved
  *		to disk on exit!
  *	- save more metadata if we can make use of it.
+ *
  */
 
 #include "common.h"
 
-RCSID("$Id: hcache.c,v 1.20 2005/05/04 17:37:22 graaff Exp $");
+RCSID("$Id: hcache.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "hosts.h"
 #include "hcache.h"
@@ -59,14 +65,15 @@ RCSID("$Id: hcache.c,v 1.20 2005/05/04 17:37:22 graaff Exp $");
 #include "lib/bg.h"
 #include "lib/file.h"
 #include "lib/hashlist.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/override.h"			/* Must be the last header included */
 
-#define MIN_RESERVE_SIZE	1024	/* we'd like that many pongs in reserve */
+#define MIN_RESERVE_SIZE	1024	/**< we'd like that many pongs in reserve */
 
 /**
  * An entry within the hostcache.
@@ -76,11 +83,11 @@ RCSID("$Id: hcache.c,v 1.20 2005/05/04 17:37:22 graaff Exp $");
  * the host.
  */
 typedef struct hostcache_entry {
-    hcache_type_t type;        /**< Hostcache which contains this host */
-    time_t        time_added;  /**< Time when entry was added */
+    hcache_type_t type;				/**< Hostcache which contains this host */
+    time_t        time_added;		/**< Time when entry was added */
 #if 0
-	guint32       avg_uptime;  /**< Reported average uptime (seconds) */
-	gchar *       vendor;	   /**< Latest known vendor name (atom) */
+	guint32       avg_uptime;		/**< Reported average uptime (seconds) */
+	gchar *       vendor;			/**< Latest known vendor name (atom) */
 #endif
 } hostcache_entry_t;
 
@@ -91,10 +98,10 @@ typedef struct hostcache_entry {
  * A hostcache table.
  */
 typedef struct hostcache {
-	gchar *         name;		        /**< Name of the cache */
+	const gchar		*name;		        /**< Name of the cache */
 	hcache_type_t   type;				/**< Cache type */
 
-    gboolean        ip_only;            /**< Use IP only, port always 0 */
+    gboolean        addr_only;          /**< Use IP only, port always 0 */
     gboolean        dirty;            	/**< If updated since last disk flush */
     hash_list_t *   hostlist;           /**< Host list: IP/Port  */
 
@@ -114,8 +121,9 @@ static gboolean hcache_close_running = FALSE;
 /**
  * Names of the host caches.
  *
- * @note has to be in the same order as in the hcache_type_t definition
- *       in gnet_nodes.h.
+ * @note
+ * Has to be in the same order as in the hcache_type_t definition
+ * in gnet_nodes.h.
  */
 static const gchar * const names[HCACHE_MAX] = {
     "fresh regular",
@@ -245,21 +253,22 @@ hcache_dump_info(const struct hostcache *hc, const gchar *what)
 /**
  * Get information about the host entry, both the host and the metadata.
  *
- * @param ip		the IPv4 address of the host
- * @param port		the port used by the host
+ * @param addr	the address of the host
+ * @param port	the port used by the host
  * @param h		filled with the host entry in the table
  * @param e		filled with the meta data of the host, as held in table
  *
  * @return FALSE if entry was not found in the cache.
  */
 static gboolean
-hcache_ht_get(guint32 ip, guint16 port, gnet_host_t **h, hostcache_entry_t **e)
+hcache_ht_get(const host_addr_t addr, guint16 port,
+	gnet_host_t **h, hostcache_entry_t **e)
 {
 	gnet_host_t host;
 	gpointer k, v;
 	gboolean found;
 
-	host.ip = ip;
+	host.addr = addr;
 	host.port = port;
 
 	found = g_hash_table_lookup_extended(ht_known_hosts, &host, &k, &v);
@@ -273,6 +282,7 @@ hcache_ht_get(guint32 ip, guint16 port, gnet_host_t **h, hostcache_entry_t **e)
 
 /**
  * Add host to the hash table host cache.
+ *
  * Also creates a metadata struct unless the host was added to HL_CAUGHT
  * in which case we cannot know anything about the host. Yet we cannot
  * assert that HL_CAUGHT never contains a host with metadata because when
@@ -290,7 +300,7 @@ hcache_ht_add(hcache_type_t type, gnet_host_t *host)
 
     hce = hce_alloc();
     hce->type = type;
-    hce->time_added = time((time_t *) NULL);
+    hce->time_added = tm_time();
 
 	g_hash_table_insert(ht_known_hosts, host, hce);
 
@@ -315,7 +325,7 @@ hcache_ht_remove(gnet_host_t *host)
 
 	if (!found) {
 		g_warning("hcache_ht_remove: attempt to remove unknown host: %s",
-				  ip_port_to_gchar(host->ip, host->port));
+				  host_addr_port_to_string(host->addr, host->port));
 		return;
 	}
 
@@ -341,12 +351,20 @@ hcache_get_metadata(gnet_host_t *host)
  * @return TRUE if the host is in one of the "bad hosts" caches.
  */
 gboolean
-hcache_node_is_bad(guint32 ip)
+hcache_node_is_bad(const host_addr_t addr)
 {
     hostcache_entry_t *hce;
     gnet_host_t h;
 
-    h.ip = ip;
+	/*
+	 * When we're low on pongs, we cannot afford the luxury of discarding
+	 * any IP address, or we'll end up contacting web caches for more.
+	 */
+
+	if (host_low_on_pongs)
+		return FALSE;
+
+    h.addr = addr;
     h.port = 0;
     hce = hcache_get_metadata(&h);
 
@@ -382,7 +400,7 @@ hcache_move_entries(hostcache_t *to, hostcache_t *from)
 	hash_list_free(to->hostlist);
     to->hostlist = from->hostlist;
     to->host_count = from->host_count;
-    from->hostlist = hash_list_new();
+    from->hostlist = hash_list_new(NULL, NULL);
     from->host_count = 0;
 
     /*
@@ -446,7 +464,7 @@ static void
 hcache_unlink_host(hostcache_t *hc, gnet_host_t *host)
 {
 	g_assert(hc->host_count > 0 && hc->hostlist != NULL);
-	g_assert(hash_list_contains(hc->hostlist, host));
+	g_assert(hash_list_contains(hc->hostlist, host, NULL));
 
 	hash_list_remove(hc->hostlist, host);
     hc->host_count--;
@@ -472,7 +490,7 @@ hcache_unlink_host(hostcache_t *hc, gnet_host_t *host)
  * Convert host cache type to string.
  */
 const gchar *
-hcache_type_to_gchar(hcache_type_t type)
+hcache_type_to_string(hcache_type_t type)
 {
 	g_assert((guint) type < HCACHE_MAX);
 
@@ -483,7 +501,7 @@ hcache_type_to_gchar(hcache_type_t type)
  * Convert host type to string.
  */
 const gchar *
-host_type_to_gchar(host_type_t type)
+host_type_to_string(host_type_t type)
 {
 	g_assert((guint) type < HOST_MAX);
 
@@ -494,8 +512,9 @@ host_type_to_gchar(host_type_t type)
 /**
  * @return the number of slots which can be added to the given type.
  *
- * @note Several types share common pools. Adding a host of one type
- *       may affect the number of slots left on other types.
+ * @note
+ * Several types share common pools. Adding a host of one type may
+ * affect the number of slots left on other types.
  */
 static gint32
 hcache_slots_left(hcache_type_t type)
@@ -534,7 +553,8 @@ hcache_slots_left(hcache_type_t type)
  *         was added to the cache. (See above)
  */
 gboolean
-hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
 	gnet_host_t *host;
 	hostcache_t *hc;
@@ -542,16 +562,25 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 
 	g_assert((guint) type < HCACHE_MAX && type != HCACHE_NONE);
 
-    if ((type == HCACHE_UNSTABLE) && (!node_monitor_unstable_ip)) {
-        return FALSE;
-    }
+	/*
+	 * Don't add anything to the "unstable" cache if they don't want to
+	 * monitor unstable servents or when we're low on pongs (thereby
+	 * automatically disabling this monitoring).  The aim is to avoid
+	 * the host discarding the last few IP addresses it has, forcing it
+	 * to contact the web caches...
+	 */
 
-	if (ip == listen_ip() && port == listen_port) {
+    if (type == HCACHE_UNSTABLE) {
+    	if (!node_monitor_unstable_ip || host_low_on_pongs)
+			return FALSE;
+	}
+
+	if (is_my_address(addr, port)) {
         stats[HCACHE_LOCAL_INSTANCE]++;
 		return FALSE;
     }
 
-	if (node_host_is_connected(ip, port)) {
+	if (node_host_is_connected(addr, port)) {
         stats[HCACHE_ALREADY_CONNECTED]++;
 		return FALSE;			/* Connected to that host? */
     }
@@ -559,12 +588,15 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 	hc = caches[type];
     g_assert(hc->type == type);
 
-    if (!ip_is_valid(ip) && (!hc->ip_only || !port_is_valid(port))) {
+    if (
+		!host_addr_is_routable(addr) &&
+		(!hc->addr_only || !port_is_valid(port))
+	) {
         stats[HCACHE_INVALID_HOST]++;
 		return FALSE;			/* Is host valid? */
     }
 
-    if (bogons_check(ip) || hostiles_check(ip)) {
+    if (bogons_check(addr) || hostiles_check(addr)) {
         stats[HCACHE_INVALID_HOST]++;
 		return FALSE;			/* Is host valid? */
     }
@@ -574,7 +606,7 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 	 * entry from one cache to another.
 	 */
 
-	if (hcache_ht_get(ip, port, &host, &hce)) {
+	if (hcache_ht_get(addr, port, &host, &hce)) {
         g_assert(hce != NULL);
 
         hc->hits++;
@@ -621,7 +653,7 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 		 * OK, we can move it from the `hce->type' cache to the `type' one.
 		 */
 
-		g_assert(hash_list_contains(caches[hce->type]->hostlist, host));
+		g_assert(hash_list_contains(caches[hce->type]->hostlist, host, NULL));
 
 		hash_list_remove(caches[hce->type]->hostlist, host);
 		caches[hce->type]->host_count--;
@@ -630,15 +662,15 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
 		caches[hce->type]->dirty = hc->dirty = TRUE;
 
 		hce->type = type;
-		hce->time_added = time(NULL);
+		hce->time_added = tm_time();
 
 		return TRUE;
     }
 
 	/* Okay, we got a new host */
-	host = (gnet_host_t *) walloc(sizeof(*host));
+	host = walloc(sizeof *host);
 
-	host->ip   = ip;
+	host->addr = addr;
 	host->port = port;
 
 	hcache_ht_add(type, host);
@@ -682,7 +714,8 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
     hcache_update_low_on_pongs();
 
     if (dbg > 8) {
-        printf("Added %s %s (%s)\n", what, ip_port_to_gchar(ip, port),
+        printf("Added %s %s (%s)\n",
+			what, host_addr_port_to_string(addr, port),
             ((type == HCACHE_FRESH_ANY) || (type == HCACHE_VALID_ANY)) ?
                 (host_low_on_pongs ? "LOW" : "OK") : "");
     }
@@ -694,13 +727,14 @@ hcache_add(hcache_type_t type, guint32 ip, guint16 port, const gchar *what)
  * Add a caught (fresh) host to the right list depending on the host type.
  */
 gboolean
-hcache_add_caught(host_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add_caught(host_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
     switch (type) {
     case HOST_ANY:
-    	return hcache_add(HCACHE_FRESH_ANY, ip, port, what);
+    	return hcache_add(HCACHE_FRESH_ANY, addr, port, what);
     case HOST_ULTRA:
-    	return hcache_add(HCACHE_FRESH_ULTRA, ip, port, what);
+    	return hcache_add(HCACHE_FRESH_ULTRA, addr, port, what);
     case HOST_MAX:
 		g_assert_not_reached();
     }
@@ -713,13 +747,14 @@ hcache_add_caught(host_type_t type, guint32 ip, guint16 port, const gchar *what)
  * Add a valid host to the right list depending on the host type.
  */
 gboolean
-hcache_add_valid(host_type_t type, guint32 ip, guint16 port, const gchar *what)
+hcache_add_valid(host_type_t type, const host_addr_t addr, guint16 port,
+	const gchar *what)
 {
     switch (type) {
     case HOST_ANY:
-    	return hcache_add(HCACHE_VALID_ANY, ip, port, what);
+    	return hcache_add(HCACHE_VALID_ANY, addr, port, what);
     case HOST_ULTRA:
-    	return hcache_add(HCACHE_VALID_ULTRA, ip, port, what);
+    	return hcache_add(HCACHE_VALID_ULTRA, addr, port, what);
     case HOST_MAX:
 		g_assert_not_reached();
     }
@@ -742,7 +777,7 @@ hcache_remove(gnet_host_t *h)
     hce = hcache_get_metadata(h);
     if (hce == NULL) {
 		g_warning("hcache_remove: attempt to remove unknown host: %s",
-				  ip_port_to_gchar(h->ip, h->port));
+				  host_addr_port_to_string(h->addr, h->port));
         return; /* Host is not in hashtable */
     }
 
@@ -867,9 +902,8 @@ hcache_size(host_type_t type)
  * @return total number of expired entries
  */
 static guint32
-hcache_expire_cache(hostcache_t *hc)
+hcache_expire_cache(hostcache_t *hc, time_t now)
 {
-    time_t now = time((time_t *) NULL);
     gint32 secs_to_keep = 60 * 30; /* 30 minutes */
     guint32 expire_count = 0;
 	gnet_host_t *h;
@@ -905,14 +939,14 @@ hcache_expire_cache(hostcache_t *hc)
  *
  * @return total number of expired entries
  */
-guint32
-hcache_expire_all(void)
+static guint32
+hcache_expire_all(time_t now)
 {
     guint32 expire_count = 0;
 
-    expire_count += hcache_expire_cache(caches[HCACHE_TIMEOUT]);
-    expire_count += hcache_expire_cache(caches[HCACHE_BUSY]);
-    expire_count += hcache_expire_cache(caches[HCACHE_UNSTABLE]);
+    expire_count += hcache_expire_cache(caches[HCACHE_TIMEOUT], now);
+    expire_count += hcache_expire_cache(caches[HCACHE_BUSY], now);
+    expire_count += hcache_expire_cache(caches[HCACHE_UNSTABLE], now);
 
     return expire_count;
 }
@@ -1020,7 +1054,7 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
 	for (i = 0; i < hcount; i++) {
 		gnet_host_t host;
 
-		if (!pcache_get_recent(type, &host.ip, &host.port))
+		if (!pcache_get_recent(type, &host.addr, &host.port))
 			break;
 
 		if (g_hash_table_lookup(seen_host, &host))
@@ -1068,11 +1102,11 @@ done:
  * @return TRUE if host is found
  */
 gboolean
-hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
+hcache_find_nearby(host_type_t type, host_addr_t *addr, guint16 *port)
 {
 	gnet_host_t *h;
 	static int alternate = 0;
-	guint32 first_ip;
+	host_addr_t first_addr;
 	guint16 first_port;
 	gboolean got_recent;
 	hostcache_t *hc = NULL;
@@ -1098,30 +1132,30 @@ hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
 
 	if (alternate++ & 1) {
 		/* Iterate through all recent pongs */
-		for (
-			*ip = 0,
-				got_recent = pcache_get_recent(type, &first_ip, &first_port);
-			got_recent && (*ip != first_ip || *port != first_port);
-			got_recent = pcache_get_recent(type, ip, port)
-		) {
-			if (host_is_nearby(*ip))
+
+		*addr = zero_host_addr;
+		got_recent = pcache_get_recent(type, &first_addr, &first_port);
+
+		while (got_recent) {
+			if (host_addr_equal(*addr, first_addr) && *port == first_port)
+				break;
+
+			if (host_is_nearby(*addr))
 				return TRUE;
+
+			got_recent = pcache_get_recent(type, addr, port);
 		}
 	}
 
 	/* iterate through whole list */
 
-	iter = reading ?  hash_list_iterator(hc->hostlist) :
+	iter = reading ? hash_list_iterator(hc->hostlist) :
 		hash_list_iterator_last(hc->hostlist);
 
-	for (
-		h = hash_list_follower(iter);
-		NULL != h;
-        h = hash_list_follower(iter)
-	) {
+	for (h = hash_list_follower(iter); h; h = hash_list_follower(iter)) {
 
-		if (host_is_nearby(h->ip)) {
-            *ip = h->ip;
+		if (host_is_nearby(h->addr)) {
+            *addr = h->addr;
             *port = h->port;
 
             result = TRUE;
@@ -1142,7 +1176,7 @@ hcache_find_nearby(host_type_t type, guint32 *ip, guint16 *port)
  * recent pong cache, in alternance.
  */
 void
-hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
+hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
 {
 	static guint alternate = 0;
 	hostcache_t *hc = NULL;
@@ -1178,7 +1212,7 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 
 	if (
 		use_netmasks && number_local_networks &&
-		hcache_find_nearby(type, ip, port)
+		hcache_find_nearby(type, addr, port)
 	)
 		return;
 
@@ -1186,7 +1220,7 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 	 * Try the recent pong cache when `alternate' is odd.
 	 */
 
-	if (alternate++ & 0x1 && pcache_get_recent(type, ip, port))
+	if (alternate++ & 0x1 && pcache_get_recent(type, addr, port))
 		return;
 
 	/*
@@ -1196,7 +1230,7 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
 
 	h = reading ? hash_list_first(hc->hostlist) : hash_list_last(hc->hostlist);
 
-	*ip = h->ip;
+	*addr = h->addr;
 	*port = h->port;
 
     hcache_unlink_host(hc, h);
@@ -1220,20 +1254,20 @@ hcache_get_caught(host_type_t type, guint32 *ip, guint16 *port)
  */
 static hostcache_t *
 hcache_alloc(hcache_type_t type, gnet_property_t reading,
-	gnet_property_t catcher, gchar *name)
+	gnet_property_t catcher, const gchar *name)
 {
 	struct hostcache *hc;
 
 	g_assert((guint) type < HCACHE_MAX);
 
-	hc = g_malloc0(sizeof(*hc));
+	hc = g_malloc0(sizeof *hc);
 
-	hc->hostlist = hash_list_new();
+	hc->hostlist = hash_list_new(NULL, NULL);
 	hc->name = name;
 	hc->type = type;
 	hc->reading = reading;
     hc->hosts_in_catcher = catcher;
-    hc->ip_only = FALSE;
+    hc->addr_only = FALSE;
 
 	return hc;
 }
@@ -1261,12 +1295,12 @@ hcache_free(hostcache_t *hc)
  */
 
 #define READ_MAGIC		0x3d00003d
-#define HOST_READ_CNT	20			/* Amount of hosts to read each tick */
+#define HOST_READ_CNT	20			/**< Amount of hosts to read each tick */
 
 struct read_ctx {
-	gint magic;						/* Magic number */
-	FILE *fd;						/* File descriptor to read from */
-	hostcache_t *hc;				/* Hostcache to fill */
+	gint magic;						/**< Magic number */
+	FILE *fd;						/**< File descriptor to read from */
+	hostcache_t *hc;				/**< Hostcache to fill */
 };
 
 /**
@@ -1307,7 +1341,7 @@ read_done(hostcache_t *hc)
 static bgret_t
 read_step(gpointer unused_h, gpointer u, gint ticks)
 {
-	struct read_ctx *rctx = (struct read_ctx *) u;
+	struct read_ctx *rctx = u;
 	hostcache_t *hc;
 	gint max_read;
 	gint count;
@@ -1315,7 +1349,7 @@ read_step(gpointer unused_h, gpointer u, gint ticks)
 	static gchar h_tmp[1024];
 
 	(void) unused_h;
-	g_assert(rctx->magic == READ_MAGIC);
+	g_assert(READ_MAGIC == rctx->magic);
 	g_assert(rctx->fd);
 
 	hc = rctx->hc;
@@ -1329,11 +1363,11 @@ read_step(gpointer unused_h, gpointer u, gint ticks)
 
 	for (i = 0; i < count; i++) {
 		if (fgets(h_tmp, sizeof(h_tmp) - 1, rctx->fd)) { /* NUL appended */
-			guint32 ip;
+			host_addr_t addr;
 			guint16 port;
 
-			if (gchar_to_ip_port(h_tmp, &ip, &port))
-                hcache_add(hc->type, ip, port, "on-disk cache");
+			if (string_to_host_addr_port(h_tmp, NULL, &addr, &port))
+                hcache_add(hc->type, addr, port, "on-disk cache");
 		} else
 			goto done;
 	}
@@ -1416,7 +1450,7 @@ hcache_write(FILE *f, hostcache_t *hc)
 	iter = hash_list_iterator(hc->hostlist);
 
 	for (h = hash_list_next(iter); h != NULL; h = hash_list_next(iter))
-		fprintf(f, "%s\n", ip_port_to_gchar(h->ip, h->port));
+		fprintf(f, "%s\n", host_addr_port_to_string(h->addr, h->port));
 
 	hash_list_release(iter);
 }
@@ -1453,7 +1487,7 @@ hcache_store(hcache_type_t type, const gchar *filename, hcache_type_t extra)
 /**
  * Get statistical information about the caches.
  *
- * @param stats must point to an hcache_stats_t[HCACHE_MAX] array.
+ * @param s must point to an hcache_stats_t[HCACHE_MAX] array.
  */
 void
 hcache_get_stats(hcache_stats_t *s)
@@ -1474,9 +1508,9 @@ hcache_get_stats(hcache_stats_t *s)
  * Host cache timer.
  */
 void
-hcache_timer(void)
+hcache_timer(time_t now)
 {
-    hcache_expire_all();
+    hcache_expire_all(now);
 
     if (dbg >= 15) {
         hcache_dump_info(caches[HCACHE_FRESH_ANY],   "timer");
@@ -1530,19 +1564,19 @@ hcache_init(void)
         HCACHE_TIMEOUT, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.timeout");
-    caches[HCACHE_TIMEOUT]->ip_only = TRUE;
+    caches[HCACHE_TIMEOUT]->addr_only = TRUE;
 
     caches[HCACHE_BUSY] = hcache_alloc(
         HCACHE_BUSY, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.busy");
-    caches[HCACHE_BUSY]->ip_only = TRUE;
+    caches[HCACHE_BUSY]->addr_only = TRUE;
 
     caches[HCACHE_UNSTABLE] = hcache_alloc(
         HCACHE_UNSTABLE, PROP_READING_HOSTFILE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.unstable");
-    caches[HCACHE_UNSTABLE]->ip_only = TRUE;
+    caches[HCACHE_UNSTABLE]->addr_only = TRUE;
 }
 
 /**
@@ -1564,7 +1598,7 @@ hcache_store_if_dirty(host_type_t type)
 	gnet_property_t prop;
 	gboolean reading;
 	hcache_type_t first, second;
-	gchar *file;
+	const gchar *file;
 
 	switch (type) {
     case HOST_ANY:
