@@ -1,10 +1,8 @@
 /*
+ * $Id: hsep.c 12201 2006-11-03 05:14:47Z cbiere $
+ *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
- * Horizon Size Estimation Protocol 0.2
- *
- * Protocol is defined here: http://www.menden.org/gnutella/hsep.html
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,7 +23,18 @@
  *----------------------------------------------------------------------
  */
 
-/*
+/**
+ * @ingroup core
+ * @file
+ *
+ * Horizon Size Estimation Protocol 0.2.
+ *
+ * Protocol is defined here: http://www.menden.org/gnutella/hsep.html
+ *
+ * @author Thomas Schuerger
+ * @author Jeroen Asselman
+ * @date 2004
+ *
  * General API information:
  *
  * - hsep_init() should be called once on startup of GtkG
@@ -77,7 +86,8 @@
  * to a copy of the HSEP table and the number of provided triples. You must
  * remove the listener later using hsep_remove_global_table_listener(...).
  *
- * Note: To support exchanging information about clients that don't support
+ * @note
+ * To support exchanging information about clients that don't support
  * HSEP, these clients' library sizes (from PONG messages) are taken into
  * account when HSEP messages are sent (that info is added to what we see
  * in a distance of >= 1 hop).
@@ -85,7 +95,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hsep.c,v 1.11 2005/05/14 04:06:16 cbiere Exp $");
+RCSID("$Id: hsep.c 12201 2006-11-03 05:14:47Z cbiere $")
 
 #include "gmsg.h"
 #include "routing.h"
@@ -100,10 +110,11 @@ RCSID("$Id: hsep.c,v 1.11 2005/05/14 04:06:16 cbiere Exp $");
 
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"
 
-/* global HSEP table */
+/** global HSEP table */
 static hsep_triple hsep_global_table[HSEP_N_MAX + 1];
 
 /*
@@ -159,7 +170,7 @@ hsep_check_monotony(hsep_triple *table, unsigned int triples)
 	g_assert(table);
 
 	for (i = 1; i < triples; i++) {
-		
+
 		/* if any triple is not >= the previous one, error will be TRUE */
 		for (j = 0; j < G_N_ELEMENTS(table[0]); j++)
 			error |= table[i - 1][j] > table[i][j];
@@ -211,10 +222,10 @@ hsep_sanity_check(void)
 		if (!(n->attrs & NODE_A_CAN_HSEP))
 			continue;
 
-		g_assert(0 == n->hsep->table[0][HSEP_IDX_NODES]);      /* check nodes */
-		g_assert(0 == n->hsep->table[0][HSEP_IDX_FILES]);      /* check files */
-		g_assert(0 == n->hsep->table[0][HSEP_IDX_KIB]);        /* check KiB */
-		g_assert(1 == n->hsep->table[1][HSEP_IDX_NODES]);  	/* check nodes */
+		g_assert(0 == n->hsep->table[0][HSEP_IDX_NODES]);	/* check nodes */
+		g_assert(0 == n->hsep->table[0][HSEP_IDX_FILES]);	/* check files */
+		g_assert(0 == n->hsep->table[0][HSEP_IDX_KIB]);		/* check KiB */
+		g_assert(1 == n->hsep->table[1][HSEP_IDX_NODES]);	/* check nodes */
 
 		/* check if values are monotonously increasing (skip first) */
 		g_assert(
@@ -277,7 +288,8 @@ hsep_dump_table(void)
  * If there is no such n_opt, n_opt := triples.
  * If all triples are equal, 1 is returned, which is correct.
  *
- * NOTE: this algorithm works regardless of the byte order of the triple data,
+ * @note
+ * This algorithm works regardless of the byte order of the triple data,
  * because only equality tests are used.
  */
 
@@ -297,7 +309,7 @@ hsep_triples_to_send(const hsep_triple *table, unsigned int triples)
 	 */
 
 	for (i = last; i-- > 0; triples--) {
-			
+
 		for (j = 0; j < G_N_ELEMENTS(table[0]); j++)
 	   		changed |= table[i][j] != table[last][j];
 
@@ -316,11 +328,10 @@ hsep_triples_to_send(const hsep_triple *table, unsigned int triples)
 void
 hsep_init(void)
 {
-	header_features_add(&xfeatures.connections,
+	header_features_add(FEATURES_CONNECTIONS,
 		"HSEP", HSEP_VERSION_MAJOR, HSEP_VERSION_MINOR);
-	memset(hsep_global_table, 0, sizeof(hsep_global_table));
 	hsep_global_table_changed_event = event_new("hsep_global_table_changed");
- 	hsep_fire_global_table_changed(time(NULL));
+ 	hsep_fire_global_table_changed(tm_time());
 }
 
 /**
@@ -336,7 +347,7 @@ hsep_add_global_table_listener(GCallback cb, frequency_t t, guint32 interval)
 {
 	hsep_triple table[G_N_ELEMENTS(hsep_global_table)];
 	hsep_global_listener_t func = (hsep_global_listener_t) cb;
-	
+
 
 	/* add callback to the event subscriber list */
 	event_add_subscriber(hsep_global_table_changed_event, cb, t, interval);
@@ -402,7 +413,7 @@ hsep_reset(void)
 		 * to all HSEP connections the next time it is called.
 		 */
 	}
-	hsep_fire_global_table_changed(time(NULL));
+	hsep_fire_global_table_changed(tm_time());
 }
 
 /**
@@ -413,14 +424,14 @@ void
 hsep_connection_init(struct gnutella_node *n)
 {
 	static const hsep_ctx_t zero_hsep;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 	guint i;
 
 	g_assert(n);
 
 	if (hsep_debug > 1)
 		printf("HSEP: Initializing node %s\n",
-			ip_port_to_gchar(n->ip, n->port));
+			host_addr_port_to_string(n->addr, n->port));
 
 	n->hsep = walloc(sizeof *n->hsep);
 	*n->hsep = zero_hsep; /* Initializes everything to 0 */
@@ -512,7 +523,7 @@ hsep_connection_close(struct gnutella_node *n)
 
 	if (hsep_debug > 1)
 		printf("HSEP: Deinitializing node %s\n",
-			ip_port_to_gchar(n->ip, n->port));
+			host_addr_port_to_string(n->addr, n->port));
 
 	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
 
@@ -534,7 +545,7 @@ hsep_connection_close(struct gnutella_node *n)
 	if (hsep_debug > 1)
 		hsep_dump_table();
 
-	hsep_fire_global_table_changed(time(NULL));
+	hsep_fire_global_table_changed(tm_time());
 }
 
 /**
@@ -562,7 +573,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	if (length == 0) {   /* error, at least 1 triple must be present */
 		if (hsep_debug > 1)
 			printf("HSEP: Node %s sent empty message\n",
-				ip_port_to_gchar(n->ip, n->port));
+				host_addr_port_to_string(n->addr, n->port));
 
 		return;
 	}
@@ -570,7 +581,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	if (length % 24) {   /* error, # of triples not an integer */
 		if (hsep_debug > 1)
 			printf("HSEP: Node %s sent broken message\n",
-				ip_port_to_gchar(n->ip, n->port));
+				host_addr_port_to_string(n->addr, n->port));
 
 		return;
 	}
@@ -582,7 +593,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 		if (hsep_debug > 1) {
 			printf(
 				"HSEP: Node %s is a leaf, but sent %u triples instead of 1\n",
-				ip_port_to_gchar(n->ip, n->port), msgmax);
+				host_addr_port_to_string(n->addr, n->port), msgmax);
 		}
 		return;
 	}
@@ -608,14 +619,14 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	if (messaget[0][HSEP_IDX_NODES] != 1) { /* # of nodes for 1 hop must be 1 */
 		if (hsep_debug > 1)
 			printf("HSEP: Node %s's message's #nodes for 1 hop is not 1\n",
-				ip_port_to_gchar(n->ip, n->port));
+				host_addr_port_to_string(n->addr, n->port));
 		return;
 	}
 
 	if (!hsep_check_monotony(messaget, max)) {
 		if (hsep_debug > 1)
 			printf("HSEP: Node %s's message's monotony check failed\n",
-				ip_port_to_gchar(n->ip, n->port));
+				host_addr_port_to_string(n->addr, n->port));
 
 		return;
 	}
@@ -623,7 +634,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	if (hsep_debug > 1) {
 		printf("HSEP: Received %d %s from node %s (msg #%u): ", max,
 		    max == 1 ? "triple" : "triples",
-			ip_port_to_gchar(n->ip, n->port),
+			host_addr_port_to_string(n->addr, n->port),
 			hsep->msgs_received + 1);
 	}
 
@@ -637,12 +648,12 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 			gchar buf[G_N_ELEMENTS(messaget[0])][32];
 
 			for (j = 0; j < G_N_ELEMENTS(buf); j++)
-				uint64_to_string_buf(buf[j], sizeof buf[0], messaget[k][j]);
-		
-			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));	
+				uint64_to_string_buf(messaget[k][j], buf[j], sizeof buf[0]);
+
+			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));
 			printf("(%s, %s, %s) ", buf[0], buf[1], buf[2]);
 		}
-		
+
 		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
 			hsep_global_table[i][j] += messaget[k][j] - hsep->table[i][j];
 			hsep->table[i][j] = messaget[k][j];
@@ -660,9 +671,9 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	/* Go back to last triple */
 	if (k > 0)
 		k--;
-	
+
 	for (/* NOTHING */; i < G_N_ELEMENTS(hsep_global_table); i++) {
-		
+
 		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
 			hsep_global_table[i][j] += messaget[k][j] - hsep->table[i][j];
 			hsep->table[i][j] = messaget[k][j];
@@ -696,7 +707,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 {
 	hsep_triple tmp[G_N_ELEMENTS(n->hsep->sent_table)], other;
 	unsigned int i, j, msglen, triples, opttriples;
-	struct gnutella_msg_hsep_data *m;
+	gnutella_msg_hsep_t *msg;
 	hsep_ctx_t *hsep;
 
 	g_assert(n);
@@ -718,14 +729,18 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	 * Allocate and initialize message to send.
 	 */
 
-	msglen = sizeof(struct gnutella_header) + triples * sizeof m->triple;
-	m = g_malloc(msglen);
+	msglen = GTA_HEADER_SIZE + triples * (sizeof *msg - GTA_HEADER_SIZE);
+	msg = g_malloc(msglen);
 
-	message_set_muid(&m->header, GTA_MSG_HSEP_DATA);
-
-	m->header.function = GTA_MSG_HSEP_DATA;
-	m->header.ttl = 1;
-	m->header.hops = 0;
+	{
+		gnutella_header_t *header;
+		
+		header = gnutella_msg_hsep_header(msg);
+		message_set_muid(header, GTA_MSG_HSEP_DATA);
+		gnutella_header_set_function(header, GTA_MSG_HSEP_DATA);
+		gnutella_header_set_ttl(header, 1);
+		gnutella_header_set_hops(header, 0);
+	}
 
 	/*
 	 * Collect HSEP data to send and convert the data to
@@ -752,11 +767,12 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	if (
 		0 == memcmp(tmp, hsep->sent_table, sizeof tmp)
 	) {
-		G_FREE_NULL(m);
+		G_FREE_NULL(msg);
 		goto charge_timer;
 	}
 
-	memcpy(m->triple, tmp, triples * sizeof tmp[0]);
+	memcpy(cast_to_gchar_ptr(msg) + GTA_HEADER_SIZE,
+		tmp, triples * sizeof tmp[0]);
 
 	/* store the table for later comparison */
 	memcpy(hsep->sent_table, tmp, triples * sizeof tmp[0]);
@@ -773,7 +789,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	if (hsep_debug > 1) {
 		printf("HSEP: Sending %d %s to node %s (msg #%u): ", opttriples,
 		    opttriples == 1 ? "triple" : "triples",
-			ip_port_to_gchar(n->ip, n->port),
+			host_addr_port_to_string(n->addr, n->port),
 			hsep->msgs_sent + 1);
 	}
 
@@ -785,10 +801,10 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 				guint64 v;
 
 				v = hsep_own[j] + hsep_global_table[i][j] - hsep->table[i][j];
-				uint64_to_string_buf(buf[j], sizeof buf[0], v);
+				uint64_to_string_buf(v, buf[j], sizeof buf[0]);
 			}
-		
-			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));	
+
+			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));
 			printf("(%s, %s, %s) ", buf[0], buf[1], buf[2]);
 		}
 	}
@@ -798,15 +814,15 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 
 	/* write message size */
 	msglen = opttriples * 24;
-	WRITE_GUINT32_LE(msglen, m->header.size);
+	gnutella_header_set_size(gnutella_msg_hsep_header(msg), msglen);
 
 	/* correct message length */
-	msglen += sizeof(struct gnutella_header);
+	msglen += GTA_HEADER_SIZE;
 
 	/* send message to peer node */
-	gmsg_sendto_one(n, m, msglen);
+	gmsg_sendto_one(n, msg, msglen);
 
-	G_FREE_NULL(m);
+	G_FREE_NULL(msg);
 
 	/*
 	 * Update counters.
@@ -836,7 +852,7 @@ hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
 		own_files != hsep_own[HSEP_IDX_FILES] ||
 		own_kibibytes != hsep_own[HSEP_IDX_KIB]
 	) {
-		
+
 		if (hsep_debug) {
 			printf("HSEP: Shared files changed to %s (%s KiB)\n",
 			    uint64_to_string(own_files), uint64_to_string2(own_kibibytes));
@@ -859,7 +875,7 @@ hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
  * triples in the table, it is truncated appropriately. Note that also
  * the 0'th triple is copied, which is always zero.
  *
- * The number of copied triples is returned.
+ * @return The number of copied triples.
  */
 
 unsigned int
@@ -879,11 +895,11 @@ hsep_get_global_table(hsep_triple *buffer, unsigned int maxtriples)
  * triples in the table, it is truncated appropriately. Note that also
  * the 0'th triple is copied, which is always zero.
  *
- * The number of copied triples is returned.
+ * @return The number of copied triples.
  */
 
 unsigned int
-hsep_get_connection_table(struct gnutella_node *n,
+hsep_get_connection_table(const struct gnutella_node *n,
     hsep_triple *buffer, unsigned int maxtriples)
 {
 	g_assert(n);
@@ -892,7 +908,7 @@ hsep_get_connection_table(struct gnutella_node *n,
 
 	maxtriples = MIN(maxtriples, G_N_ELEMENTS(n->hsep->table));
 	memcpy(buffer, n->hsep->table, maxtriples * sizeof buffer[0]);
-	
+
 	return maxtriples;
 }
 
@@ -969,7 +985,9 @@ hsep_get_non_hsep_triple(hsep_triple *tripledest)
 
 
 /**
- * Returns a static string of the cell contents of the given row and column.
+ * @returns a static string of the cell contents of the given row and column.
+ *
+ * @attention
  * NB: The static buffers for each column are disjunct.
  */
 const gchar *
@@ -986,30 +1004,32 @@ hsep_get_static_str(gint row, gint column)
     switch (column) {
     case HSEP_IDX_NODES:
 		{
-			static gchar buf[21];
+			static gchar buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_NODES] + other[0][HSEP_IDX_NODES];
-			ret = uint64_to_string_buf(buf, sizeof buf, v);
+			uint64_to_string_buf(v, buf, sizeof buf);
+			ret = buf;
 		}
 		break;
 
     case HSEP_IDX_FILES:
 		{
-			static gchar buf[21];
+			static gchar buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_FILES] + other[0][HSEP_IDX_FILES];
-			ret = uint64_to_string_buf(buf, sizeof buf, v);
+			uint64_to_string_buf(v, buf, sizeof buf);
+			ret = buf;
 		}
 		break;
 
 	case HSEP_IDX_KIB:
 		{
-			static gchar buf[21];
+			static gchar buf[UINT64_DEC_BUFLEN];
 
 			/* Make a copy because concurrent usage of short_kb_size()
 	 	 	 * could be hard to discover. */
 			v = hsep_table[row][HSEP_IDX_KIB] + other[0][HSEP_IDX_KIB];
-			g_strlcpy(buf, short_kb_size(v), sizeof buf);
+			g_strlcpy(buf, short_kb_size(v, display_metric_units), sizeof buf);
   			ret = buf;
 		}
 		break;
@@ -1020,8 +1040,7 @@ hsep_get_static_str(gint row, gint column)
 }
 
 /**
- * Returns the size of the global hsep table
- *
+ * @returns the size of the global hsep table
  */
 gint
 hsep_get_table_size(void)
