@@ -1,5 +1,5 @@
 /*
- * $Id: huge.c,v 1.24 2005/09/10 08:17:28 daichik Exp $
+ * $Id: huge.c 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: huge.c,v 1.24 2005/09/10 08:17:28 daichik Exp $");
+RCSID("$Id: huge.c 12058 2006-09-27 19:33:55Z cbiere $")
 
 #include "huge.h"
 #include "share.h"
@@ -47,6 +47,7 @@ RCSID("$Id: huge.c,v 1.24 2005/09/10 08:17:28 daichik Exp $");
 #include "dmesh.h"
 #include "version.h"
 #include "settings.h"
+#include "spam.h"
 
 #include "lib/atoms.h"
 #include "lib/base32.h"
@@ -113,7 +114,8 @@ static gboolean cache_dirty = FALSE;
  *
  * Copy the ASCII representation of a SHA1 digest from source to dest.
  */
-static void copy_sha1(char *dest, const char *source)
+static void
+copy_sha1(char *dest, const char *source)
 {
 	memcpy(dest, source, SHA1_RAW_SIZE);
 }
@@ -125,8 +127,6 @@ static void copy_sha1(char *dest, const char *source)
 /* In-memory cache */
 
 /**
- * update_volatile_cache
- *
  * Takes an in-memory cached entry, and update its content.
  */
 static void update_volatile_cache(
@@ -142,16 +142,15 @@ static void update_volatile_cache(
 }
 
 /**
- * add_volatile_cache_entry
- *
  * Add a new entry to the in-memory cache.
  */
 static void
 add_volatile_cache_entry(const char *filename, filesize_t size, time_t mtime,
 	const char *digest, gboolean known_to_be_shared)
 {
-	struct sha1_cache_entry *new_entry = g_malloc(sizeof *new_entry);
-
+	struct sha1_cache_entry *new_entry;
+   
+	new_entry = walloc(sizeof *new_entry);
 	new_entry->file_name = atom_str_get(filename);
 	new_entry->size = size;
 	new_entry->mtime = mtime;
@@ -442,7 +441,7 @@ static struct file_sha1 *waiting_for_library_build_complete = NULL;
  * waiting_for_library_build_complete).
  */
 static void
-push(struct file_sha1 **stack,struct file_sha1 *record)
+push(struct file_sha1 **stack, struct file_sha1 *record)
 {
 	record->next = *stack;
 	*stack = record;
@@ -451,11 +450,30 @@ push(struct file_sha1 **stack,struct file_sha1 *record)
 /**
  * Free a working cell.
  */
-static void
-free_cell(struct file_sha1 *cell)
+static struct file_sha1 *
+alloc_cell(void)
 {
-	atom_str_free(cell->file_name);
-	G_FREE_NULL(cell);
+	struct file_sha1 *p;
+
+	p = walloc(sizeof *p);
+	return p;
+}
+
+/**
+ * Free a working cell.
+ */
+static void
+free_cell(struct file_sha1 **cell_ptr)
+{
+	g_assert(cell_ptr);
+
+	if (*cell_ptr) {
+		struct file_sha1 *cell = *cell_ptr;
+
+		atom_str_free(cell->file_name);
+		wfree(cell, sizeof *cell);
+		*cell_ptr = NULL;
+	}
 }
 
 /* The context of the SHA1 computation being performed */
@@ -466,7 +484,7 @@ struct sha1_computation_context {
 	guint magic;
 	SHA1Context context;
 	struct file_sha1 *file;
-	gchar *buffer;				/**< Large buffer where data is read */
+	gpointer buffer;			/**< Large buffer where data is read */
 	gint fd;
 	time_t start;				/**< Debugging, show computation rate */
 };
@@ -474,27 +492,26 @@ struct sha1_computation_context {
 static void
 sha1_computation_context_free(gpointer u)
 {
-	struct sha1_computation_context *ctx =
-		(struct sha1_computation_context *) u;
+	struct sha1_computation_context *ctx = u;
 
 	g_assert(ctx->magic == SHA1_MAGIC);
 
-	if (ctx->fd != -1)
+	if (ctx->fd != -1) {
 		close(ctx->fd);
-
-	if (ctx->file)
-		free_cell(ctx->file);
-
-	G_FREE_NULL(ctx->buffer);
-
-	wfree(ctx, sizeof(*ctx));
+		ctx->fd = -1;
+	}
+	free_cell(&ctx->file);
+	g_free(ctx->buffer);
+	wfree(ctx, sizeof *ctx);
 }
 
 /**
  * When SHA1 is computed, and we know what struct share it's related
  * to, we call this function to update set the share SHA1 value.
+ *
+ * @return FALSE if the pointer "sf" is not valid any longer; TRUE otherwise.
  */
-static void
+static gboolean
 put_sha1_back_into_share_library(struct shared_file *sf,
 	const char *file_name, const char *digest)
 {
@@ -505,55 +522,67 @@ put_sha1_back_into_share_library(struct shared_file *sf,
 
 	if (!sf) {
 		g_warning("got SHA1 for unknown file: %s", file_name);
-		return;
+		return FALSE;
 	}
 
-	if (0 != strcmp(sf->file_path, file_name)) {
+	if (0 != strcmp(shared_file_path(sf), file_name)) {
 
 		/*
 		 * File name changed since last time
 		 * (that is, "rescan dir" was called)
 		 */
 
-		g_warning("name of file #%d changed from \"%s\" to \"%s\" (rescan?): "
-				"discarding SHA1", sf->file_index, file_name, sf->file_path);
-		return;
+		g_warning("name of file #%lu changed from \"%s\" to \"%s\" (rescan?): "
+				"discarding SHA1",
+				(gulong) shared_file_index(sf),
+				file_name, shared_file_path(sf));
+		return TRUE;
 	}
 
 	/*
 	 * Make sure the file's timestamp is still accurate.
 	 */
 
-	if (-1 == stat(sf->file_path, &buf)) {
+	if (-1 == stat(shared_file_path(sf), &buf)) {
 		g_warning("discarding SHA1 for file #%d \"%s\": can't stat(): %s",
-			sf->file_index, sf->file_path, g_strerror(errno));
-		return;
+			shared_file_index(sf), shared_file_path(sf), g_strerror(errno));
+		return TRUE;
 	}
 
-	if (buf.st_mtime != sf->mtime) {
-		g_warning("file #%d \"%s\" was modified whilst SHA1 was computed",
-			sf->file_index, sf->file_path);
-		sf->mtime = buf.st_mtime;
-		request_sha1(sf);					/* Retry! */
-		return;
+	if (buf.st_mtime != shared_file_modification_time(sf)) {
+		g_warning("file #%lu \"%s\" was modified whilst SHA1 was computed",
+			(gulong) shared_file_index(sf), shared_file_path(sf));
+		shared_file_set_modification_time(sf, buf.st_mtime);
+		return request_sha1(sf);					/* Retry! */
 	}
 
-	set_sha1(sf, digest);
+	if (spam_check(digest)) {
+		g_warning("file #%lu \"%s\" is listed as spam",
+			(gulong) shared_file_index(sf), shared_file_path(sf));
+		shared_file_remove(sf);
+		return FALSE;
+	}
+
+	shared_file_set_sha1(sf, digest);
 
 	/* Update cache */
 
-	cached_sha1 = (struct sha1_cache_entry *)
-		g_hash_table_lookup(sha1_cache, (gconstpointer) sf->file_path);
+	cached_sha1 = g_hash_table_lookup(sha1_cache,
+					cast_to_gconstpointer(shared_file_path(sf)));
 
 	if (cached_sha1) {
-		update_volatile_cache(cached_sha1, sf->file_size, sf->mtime, digest);
+		update_volatile_cache(cached_sha1, shared_file_size(sf),
+			shared_file_modification_time(sf), digest);
 		cache_dirty = TRUE;
 	} else {
-		add_volatile_cache_entry(sf->file_path,
-			sf->file_size, sf->mtime, digest, TRUE);
-		add_persistent_cache_entry(sf->file_path,
-			sf->file_size, sf->mtime, digest);
+		add_volatile_cache_entry(shared_file_path(sf),
+			shared_file_size(sf), shared_file_modification_time(sf),
+			digest, TRUE);
+		add_persistent_cache_entry(shared_file_path(sf),
+			shared_file_size(sf), shared_file_modification_time(sf), digest);
 	}
+
+	return TRUE;
 }
 
 /**
@@ -591,7 +620,7 @@ try_to_put_sha1_back_into_share_library(void)
 		waiting_for_library_build_complete = f->next;
 		put_sha1_back_into_share_library(sf2, f->file_name, f->sha1_digest);
 
-		free_cell(f);
+		free_cell(&f);
 	}
 }
 
@@ -602,10 +631,7 @@ try_to_put_sha1_back_into_share_library(void)
 static void
 close_current_file(struct sha1_computation_context *ctx)
 {
-	if (ctx->file) {
-		free_cell(ctx->file);
-		ctx->file = NULL;
-	}
+	free_cell(&ctx->file);
 
 	if (ctx->fd != -1) {
 		if (dbg > 1) {
@@ -666,7 +692,7 @@ get_next_file_from_list(void)
 			if (-1 == stat(l->file_name, &buf)) {
 				g_warning("ignoring SHA1 recomputation request for \"%s\": %s",
 					l->file_name, g_strerror(errno));
-				free_cell(l);
+				free_cell(&l);
 				continue;
 			}
 
@@ -677,7 +703,7 @@ get_next_file_from_list(void)
 				if (dbg > 1)
 					printf("ignoring duplicate SHA1 work for \"%s\"\n",
 						l->file_name);
-				free_cell(l);
+				free_cell(&l);
 				continue;
 			}
 		}
@@ -814,8 +840,7 @@ static bgret_t
 sha1_step_compute(gpointer h, gpointer u, gint ticks)
 {
 	gboolean call_again;
-	struct sha1_computation_context *ctx =
-		(struct sha1_computation_context *) u;
+	struct sha1_computation_context *ctx = u;
 	gint credit = ticks;
 
 	g_assert(ctx->magic == SHA1_MAGIC);
@@ -902,8 +927,9 @@ static void
 queue_shared_file_for_sha1_computation(guint32 file_index,
 	const char *file_name)
 {
-	struct file_sha1 *new_cell = g_malloc(sizeof(struct file_sha1));
-
+	struct file_sha1 *new_cell;
+   
+	new_cell = alloc_cell();
 	new_cell->file_name = atom_str_get(file_name);
 	new_cell->file_index = file_index;
 	push(&waiting_for_sha1_computation, new_cell);
@@ -915,7 +941,7 @@ queue_shared_file_for_sha1_computation(guint32 file_index,
 			sha1_step_dump,
 		};
 
-		ctx = walloc0(sizeof(*ctx));
+		ctx = walloc0(sizeof *ctx);
 		ctx->magic = SHA1_MAGIC;
 		ctx->fd = -1;
 		ctx->buffer = g_malloc(HASH_BUF_SIZE);
@@ -936,18 +962,19 @@ static gboolean
 cached_entry_up_to_date(const struct sha1_cache_entry *cache_entry,
 	const struct shared_file *sf)
 {
-	return cache_entry->size == sf->file_size
-		&& cache_entry->mtime == sf->mtime;
+	return cache_entry->size == shared_file_size(sf)
+		&& cache_entry->mtime == shared_file_modification_time(sf);
 }
 
 /**
  * External interface to check whether the sha1 for shared_file is known.
  */
-gboolean sha1_is_cached(const struct shared_file *sf)
+gboolean
+sha1_is_cached(const struct shared_file *sf)
 {
 	const struct sha1_cache_entry *cached_sha1;
 
-	cached_sha1 = g_hash_table_lookup(sha1_cache, sf->file_path);
+	cached_sha1 = g_hash_table_lookup(sha1_cache, shared_file_path(sf));
 
 	return cached_sha1 && cached_entry_up_to_date(cached_sha1, sf);
 }
@@ -955,30 +982,45 @@ gboolean sha1_is_cached(const struct shared_file *sf)
 
 /**
  * External interface to call for getting the hash for a shared_file.
+ *
+ * @return if shared_file_remove() was called, FALSE is returned and
+ *         "sf" is no longer valid. Otherwise TRUE is returned.
  */
-void
+gboolean
 request_sha1(struct shared_file *sf)
 {
 	struct sha1_cache_entry *cached_sha1;
 
-	cached_sha1 = g_hash_table_lookup(sha1_cache, sf->file_path);
+	cached_sha1 = g_hash_table_lookup(sha1_cache, shared_file_path(sf));
 
 	if (cached_sha1 && cached_entry_up_to_date(cached_sha1, sf)) {
-		set_sha1(sf, cached_sha1->digest);
+		if (spam_check(cached_sha1->digest)) {
+			g_warning("file #%lu \"%s\" is listed as spam",
+				(gulong) shared_file_index(sf), shared_file_path(sf));
+			shared_file_remove(sf);
+			return FALSE;
+		}
+
+		shared_file_set_sha1(sf, cached_sha1->digest);
 		cached_sha1->shared = TRUE;
-		return;
+		return TRUE;
 	}
 
 	if (dbg > 4) {
 		if (cached_sha1)
 			g_message(
-				"Cached SHA1 entry for \"%s\" outdated: had mtime %d, now %d",
-				sf->file_path, (gint) cached_sha1->mtime, (gint) sf->mtime);
+				"Cached SHA1 entry for \"%s\" outdated: had mtime %lu, now %lu",
+				shared_file_path(sf),
+				(gulong) cached_sha1->mtime,
+				(gulong) shared_file_modification_time(sf));
 		else
-			g_message("Queuing \"%s\" for SHA1 computation", sf->file_path);
+			g_message("Queuing \"%s\" for SHA1 computation",
+				shared_file_path(sf));
 	}
 
-	queue_shared_file_for_sha1_computation(sf->file_index, sf->file_path);
+	queue_shared_file_for_sha1_computation(shared_file_index(sf),
+		shared_file_path(sf));
+	return TRUE;
 }
 
 /**
@@ -1001,13 +1043,13 @@ huge_init(void)
 static gboolean
 cache_free_entry(gpointer unused_key, gpointer v, gpointer unused_udata)
 {
-	struct sha1_cache_entry *e = (struct sha1_cache_entry *) v;
+	struct sha1_cache_entry *e = v;
 
 	(void) unused_key;
 	(void) unused_udata;
 
 	atom_str_free(e->file_name);
-	G_FREE_NULL(e);
+	wfree(e, sizeof *e);
 
 	return TRUE;
 }
@@ -1024,8 +1066,7 @@ huge_close(void)
 	if (cache_dirty)
 		dump_cache();
 
-	if (persistent_cache_file_name)
-		G_FREE_NULL(persistent_cache_file_name);
+	G_FREE_NULL(persistent_cache_file_name);
 
 	g_hash_table_foreach_remove(sha1_cache, cache_free_entry, NULL);
 	g_hash_table_destroy(sha1_cache);
@@ -1034,7 +1075,7 @@ huge_close(void)
 		struct file_sha1 *l = waiting_for_sha1_computation;
 
 		waiting_for_sha1_computation = waiting_for_sha1_computation->next;
-		free_cell(l);
+		free_cell(&l);
 	}
 }
 
@@ -1070,8 +1111,6 @@ huge_improbable_sha1(const gchar *buf, size_t len)
 }
 
 /**
- * huge_sha1_extract32
- *
  * Validate `len' bytes starting from `buf' as a proper base32 encoding
  * of a SHA1 hash, and write decoded value in `retval'.
  * Also make sure that the SHA1 is not an improbable value.
@@ -1151,9 +1190,15 @@ bad:
  * about other sources for this file.
  */
 void
-huge_collect_locations(gchar *sha1, header_t *header)
+huge_collect_locations(const gchar *sha1, header_t *header)
 {
-	gchar *alt = header_get(header, "X-Gnutella-Alternate-Location");
+	gchar *alt;
+   
+
+	g_return_if_fail(sha1);
+	g_return_if_fail(header);
+
+	alt = header_get(header, "X-Gnutella-Alternate-Location");
 
 	/*
 	 * Unfortunately, clueless people broke the HUGE specs and made up their
