@@ -1,5 +1,5 @@
 /*
- * $Id: ignore.c,v 1.15 2005/12/27 16:34:49 cbiere Exp $
+ * $Id: ignore.c 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,13 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: ignore.c,v 1.15 2005/12/27 16:34:49 cbiere Exp $");
+RCSID("$Id: ignore.c 12058 2006-09-27 19:33:55Z cbiere $")
 
 #include "ignore.h"
 #include "huge.h"
 #include "share.h"
 #include "settings.h"
 #include "namesize.h"
+#include "spam.h"
 
 #include "lib/atoms.h"
 #include "lib/base32.h"
@@ -316,36 +317,54 @@ ignore_sha1_filename(const gchar *sha1)
 	return g_hash_table_lookup(by_sha1, sha1);
 }
 
+
+const gchar *
+ignore_reason_to_string(enum ignore_val reason)
+{
+	switch (reason) {
+	case IGNORE_OURSELVES:	return "Points to ourselves";
+	case IGNORE_HOSTILE:	return "Hostile IP";
+	case IGNORE_SHA1:		return "SHA1";
+	case IGNORE_SPAM:		return "Known Spam";
+	case IGNORE_LIBRARY:	return "Already Owned";
+	case IGNORE_NAMESIZE:	return "Name & Size";
+	case IGNORE_FALSE:		return "NOT ignored";
+	}
+	return NULL;
+}
+
 /**
- * Is ignoring requested for `file' of size `size' and SHA1 `sha1'?
- * Priority is given to the SHA1, if supplied.
+ * Is ignoring requested for `filename' of size `size' and SHA1 `sha1'?
+ * `filename' and `size' are only used if `sha1' is NULL.
  *
- * @param file Must be a basename, without any directory separator
+ * @param filename Must be a basename, without any directory separator
  * @param size the filesize
  * @param sha1 must point to a SHA1 (binary) or NULL
  */
 enum ignore_val
-ignore_is_requested(const gchar *file, filesize_t size, gchar *sha1)
+ignore_is_requested(const gchar *filename, filesize_t size, const gchar *sha1)
 {
-	namesize_t ns;
-
-	g_assert(file != NULL);
+	g_assert(filename != NULL);
 
 	if (sha1) {
 		const struct shared_file *sf;
 		if (g_hash_table_lookup(by_sha1, sha1))
 			return IGNORE_SHA1;
+		if (spam_check(sha1))
+			return IGNORE_SPAM;
 		sf = shared_file_by_sha1(sha1);
-		if (sf && sf != SHARE_REBUILDING && sf->fi == NULL)
+		if (sf && sf != SHARE_REBUILDING && !shared_file_is_partial(sf))
 			return IGNORE_LIBRARY;
+	} else {
+		namesize_t ns;
+
+		ns.name = deconstify_gchar(filename);
+		ns.size = size;
+
+		if (g_hash_table_lookup(by_namesize, &ns))
+			return IGNORE_NAMESIZE;
 	}
 
-	ns.name = deconstify_gchar(file);
-	ns.size = size;
-
-	if (g_hash_table_lookup(by_namesize, &ns))
-		return IGNORE_NAMESIZE;
-
 	return IGNORE_FALSE;
 }
 
