@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h,v 1.28 2005/06/25 01:37:40 daichik Exp $
+ * $Id: nodes.h 12185 2006-11-02 14:28:28Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -38,15 +38,12 @@
 
 #include "if/core/wrap.h"			/* For wrap_io_t */
 #include "if/core/hsep.h"
+#include "if/core/guid.h"
+#include "if/core/hcache.h"
 #include "if/core/nodes.h"
 
 #include "lib/header.h"
 
-typedef enum node_protocol_types {
-	PROTOCOL_TYPE_GNUTELLA = 0,
-	PROTOCOL_TYPE_G2
-} node_protocol_types_t;
-
 typedef enum {
 	NODE_MAGIC = 0x67f8e02f
 } node_magic_t;
@@ -104,22 +101,22 @@ typedef struct gnutella_node {
 	struct gnutella_socket *socket;		/**< Socket of the node */
 	guint8 proto_major;			/**< Handshaking protocol major number */
 	guint8 proto_minor;			/**< Handshaking protocol minor number */
-	node_protocol_types_t protocol_type;
 
 	guint8 qrp_major;			/**< Query routing protocol major number */
 	guint8 qrp_minor;			/**< Query routing protocol minor number */
 	guint8 uqrp_major;			/**< UP Query routing protocol major number */
 	guint8 uqrp_minor;			/**< UP Query routing protocol minor number */
-	gchar *vendor;				/**< Vendor information */
+	gchar *vendor;				/**< Vendor information (always UTF-8) */
 	gint country;				/**< Country of origin -- encoded ISO3166 */
-	guchar vcode[4];			/**< Vendor code (vcode[0] == NUL when unknown) */
+	union vendor_code vcode;	/**< Vendor code (vcode.u32 == 0 if unknown) */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 
-	struct gnutella_header header;	/**< Header of the current message */
+	gnutella_header_t header;		/**< Header of the current message */
 	extvec_t extvec[MAX_EXTVEC];	/**< GGEP extensions in "fat" messages */
 	gint extcount;					/**< Amount of extensions held */
 
-	guint32 size;	/**< How many bytes we need to read for the current message */
+	guint16 size; /**< How many bytes we need to read for the current message */
+	guint16 header_flags;		/**< Header flags (new message architecture) */
 
 	gchar *data;				/**< data of the current message */
 	guint32 pos;				/**< write position in data */
@@ -146,6 +143,8 @@ typedef struct gnutella_node {
 	guint16 n_hard_ttl;			/**< Number of hard_ttl exceeded (bad) */
 	guint32 n_weird;			/**< Number of weird messages from that node */
 	guint32 n_hostile;			/**< Number of messages from hostile IP */
+	guint32 n_spam;				/**< Number of messages rated as spam */
+	guint32 n_evil;				/**< Number of messages with evil filenames */
 
 	guint32 allocated;			/**< Size of allocated buffer data, 0 for none */
 	gboolean have_header;		/**< TRUE if we have got a full message header */
@@ -159,15 +158,15 @@ typedef struct gnutella_node {
 	time_t shutdown_date;		/**< When we entered in shutdown mode */
 	time_t up_date;				/**< When remote server started (0 if unknown) */
 	time_t leaf_flowc_start;	/**< Time when leaf flow-controlled queries */
-	guint32 shutdown_delay;		/**< How long we can stay in shutdown mode */
+	time_delta_t shutdown_delay; /**< How long we can stay in shutdown mode */
 
 	const gchar *remove_msg;	/**< Reason of removing */
 
-	guint32 ip;					/**< ip of the node */
+	host_addr_t addr;			/**< ip of the node */
 	guint16 port;				/**< port of the node */
 
 	gchar *guid;				/**< GUID of node (atom) for push-proxying */
-	guint32 proxy_ip;			/**< ip of the node for push proxyfication */
+	host_addr_t proxy_addr;		/**< ip of the node for push proxyfication */
 	guint16 proxy_port;			/**< port of the node for push proxyfication */
 
 	mqueue_t *outq;				/**< TX Output queue */
@@ -183,7 +182,7 @@ typedef struct gnutella_node {
 
 	gpointer alive_pings;		/**< Opaque info, for alive ping checks */
 	time_t last_alive_ping;		/**< Last time we sent an alive ping */
-	guint alive_period;			/**< Period for sending alive pings (secs) */
+	time_delta_t alive_period;	/**< Period for sending alive pings (secs) */
 
 	wrap_buf_t hello;			/**< Spill buffer for GNUTELLA HELLO */
 
@@ -206,24 +205,24 @@ typedef struct gnutella_node {
 	guint ping_throttle;		/**< Period for accepting new pings (secs) */
 	time_t ping_accept;			/**< Time after which we accept new pings */
 	time_t next_ping;			/**< When to send a ping, for "OLD" clients */
-	gchar ping_guid[16];		/**< The GUID of the last accepted ping */
+	gchar ping_guid[GUID_RAW_SIZE];	/**< The GUID of the last accepted ping */
 	guchar pong_needed[MAX_CACHE_HOPS+1];	/**< Pongs needed, by hop value */
-	guchar pong_missing;		/**< Sum(pong_needed[i]), i = 0..MAX_CACHE_HOPS */
+	guchar pong_missing;	/**< Sum(pong_needed[i]), i = 0..MAX_CACHE_HOPS */
 
-	guint32 gnet_ip;			/**< When != 0, we know the remote IP/port */
+	host_addr_t gnet_addr;		/**< When != 0, we know the remote IP/port */
 	guint16 gnet_port;			/**< (listening port, that is ) */
 	guint32 gnet_files_count;	/**< Used to answer "Crawling" pings */
 	guint32 gnet_kbytes_count;	/**< Used to answer "Crawling" pings */
-	guint32 gnet_pong_ip;		/**< When != 0, last IP we got in pong */
-	guint32 gnet_qhit_ip;		/**< When != 0, last IP we got in query hit */
+	host_addr_t gnet_pong_addr;	/**< When != 0, last IP we got in pong */
+	host_addr_t gnet_qhit_addr;	/**< When != 0, last IP we got in query hit */
 	gchar *gnet_guid;			/**< GUID of node (atom) seen on the network */
 
-	guint32 n_ping_throttle;	/**< Number of pings we throttled */
-	guint32 n_ping_accepted;	/**< Number of pings we accepted */
-	guint32 n_ping_special;		/**< Number of special pings we received */
-	guint32 n_ping_sent;		/**< Number of pings we sent to this node */
-	guint32 n_pong_received;	/**< Number of pongs we received from this node */
-	guint32 n_pong_sent;		/**< Number of pongs we sent to this node */
+	guint32 n_ping_throttle;  /**< Number of pings we throttled */
+	guint32 n_ping_accepted;  /**< Number of pings we accepted */
+	guint32 n_ping_special;	  /**< Number of special pings we received */
+	guint32 n_ping_sent;	  /**< Number of pings we sent to this node */
+	guint32 n_pong_received;  /**< Number of pongs we received from this node */
+	guint32 n_pong_sent;	  /**< Number of pongs we sent to this node */
 
 	/*
 	 * Traffic statistics -- RAM, 13/05/2002.
@@ -255,7 +254,7 @@ typedef struct gnutella_node {
 	guint32 rx_qhits;			/**< Total amount of hits received */
 	guint32 tx_qhits;			/**< Total amount of hits sent */
 
-	hsep_ctx_t *hsep;	/**< Horizon size estimation (HSEP) -- TSC, 11/02/2004. */
+	hsep_ctx_t *hsep;	/**< Horizon size estimation (HSEP) -- TSC, 11/02/2004 */
 
 } gnutella_node_t;
 
@@ -263,56 +262,60 @@ typedef struct gnutella_node {
  * Node flags.
  */
 
-#define NODE_F_HDSK_PING	0x00000001	/**< Expecting handshake ping */
-#define NODE_F_STALE_QRP	0x00000002	/**< Is sending a stale QRP patch */
-#define NODE_F_INCOMING		0x00000004	/**< Incoming (permanent) connection */
-#define NODE_F_ESTABLISHED	0x00000008	/**< Gnutella connection established */
-#define NODE_F_VALID		0x00000010	/**< We handshaked with a Gnutella node */
-#define NODE_F_ALIEN_IP		0x00000020	/**< Pong-IP does not match TCP/IP addr */
-#define NODE_F_WRITABLE		0x00000040	/**< Node is writable */
-#define NODE_F_READABLE		0x00000080	/**< Node is readable, process queries */
-#define NODE_F_BYE_SENT		0x00000100	/**< Bye message was queued */
-#define NODE_F_NODELAY		0x00000200	/**< TCP_NODELAY was activated */
-#define NODE_F_NOREAD		0x00000400	/**< Prevent further reading from node */
-#define NODE_F_EOF_WAIT		0x00000800	/**< During final shutdown, waiting EOF */
-#define NODE_F_CLOSING		0x00001000	/**< Initiated bye or shutdown */
-#define NODE_F_ULTRA		0x00002000	/**< Is one of our ultra nodes */
-#define NODE_F_LEAF			0x00004000	/**< Is one of our leaves */
-#define NODE_F_CRAWLER		0x00008000	/**< Is a Gnutella Crawler */
-#define NODE_F_FAKE_NAME	0x00010000	/**< Was unable to validate GTKG name */
-#define NODE_F_PROXY		0x00020000	/**< We sent a push-proxy request to it */
-#define NODE_F_PROXIED		0x00040000	/**< We are the push-proxy of that node */
-#define NODE_F_QRP_SENT		0x00080000	/**< Undergone one complete QRP sending */
-#define NODE_F_TLS			0x00100000	/**< TLS-tunneled */
-#define NODE_F_TSYNC_WAIT	0x00200000	/**< Time sync pending via TCP */
-#define NODE_F_TSYNC_TCP	0x00400000	/**< No replies via UDP, use TCP */
-#define NODE_F_GTKG			0x00800000	/**< Node is another gtk-gnutella */
+#define NODE_F_HDSK_PING	0x00000001 /**< Expecting handshake ping */
+#define NODE_F_STALE_QRP	0x00000002 /**< Is sending a stale QRP patch */
+#define NODE_F_INCOMING		0x00000004 /**< Incoming (permanent) connection */
+#define NODE_F_ESTABLISHED	0x00000008 /**< Gnutella connection established */
+#define NODE_F_VALID		0x00000010 /**< Handshaked with a Gnutella node */
+#define NODE_F_ALIEN_IP		0x00000020 /**< Pong-IP did not match TCP/IP addr */
+#define NODE_F_WRITABLE		0x00000040 /**< Node is writable */
+#define NODE_F_READABLE		0x00000080 /**< Node is readable, process queries */
+#define NODE_F_BYE_SENT		0x00000100 /**< Bye message was queued */
+#define NODE_F_NODELAY		0x00000200 /**< TCP_NODELAY was activated */
+#define NODE_F_NOREAD		0x00000400 /**< Prevent further reading from node */
+#define NODE_F_EOF_WAIT		0x00000800 /**< At final shutdown, waiting EOF */
+#define NODE_F_CLOSING		0x00001000 /**< Initiated bye or shutdown */
+#define NODE_F_ULTRA		0x00002000 /**< Is one of our ultra nodes */
+#define NODE_F_LEAF			0x00004000 /**< Is one of our leaves */
+#define NODE_F_CRAWLER		0x00008000 /**< Is a Gnutella Crawler */
+#define NODE_F_FAKE_NAME	0x00010000 /**< Was unable to validate GTKG name */
+#define NODE_F_PROXY		0x00020000 /**< Sent a push-proxy request to it */
+#define NODE_F_PROXIED		0x00040000 /**< We are push-proxy for that node */
+#define NODE_F_QRP_SENT		0x00080000 /**< Undergone 1 complete QRP sending */
+#define NODE_F_TLS			0x00100000 /**< TLS-tunneled */
+#define NODE_F_TSYNC_WAIT	0x00200000 /**< Time sync pending via TCP */
+#define NODE_F_TSYNC_TCP	0x00400000 /**< No replies via UDP, use TCP */
+#define NODE_F_GTKG			0x00800000 /**< Node is another gtk-gnutella */
+#define NODE_F_FORCE		0x01000000 /**< Connection is forced */
 
 /*
  * Node attributes.
  */
 
-#define NODE_A_BYE_PACKET	0x00000001	/**< Supports Bye-Packet */
-#define NODE_A_PONG_CACHING	0x00000002	/**< Supports Pong-Caching */
-#define NODE_A_PONG_ALIEN	0x00000004	/**< Alien Pong-Caching scheme */
-#define NODE_A_QHD_NO_VTAG	0x00000008	/**< Servent has no vendor tag in QHD */
-#define NODE_A_RX_INFLATE	0x00000010	/**< Reading compressed data */
-#define NODE_A_TX_DEFLATE	0x00000020	/**< Sending compressed data */
-#define NODE_A_ULTRA		0x00000040	/**< Node wants to be an Ultrapeer */
-#define NODE_A_NO_ULTRA		0x00000080	/**< Node is NOT ultra capable */
-#define NODE_A_UP_QRP		0x00000100	/**< Supports intra-UP QRP */
-#define NODE_A_LEAF_GUIDE	0x00000200	/**< Supports leaf-guided dyn queries */
-#define NODE_A_TIME_SYNC	0x00000400	/**< Supports time sync */
-#define NODE_A_CRAWLABLE	0x00000800	/**< Node can be UDP-crawled */
-#define NODE_A_DYN_QUERY	0x00001000	/**< Node can perform dynamic queries */
-
-#define NODE_A_CAN_HSEP		0x04000000	/**< Node supports HSEP */
-#define NODE_A_CAN_QRP		0x08000000	/**< Node supports query routing */
-#define NODE_A_CAN_VENDOR	0x10000000	/**< Node supports vendor messages */
-#define NODE_A_CAN_GGEP		0x20000000	/**< Node supports big pongs, etc.. */
-#define NODE_A_CAN_ULTRA	0x40000000	/**< Node is ultra capable */
-#define NODE_A_CAN_INFLATE	0x80000000	/**< Node capable of inflating */
-
+#define NODE_A_BYE_PACKET	0x00000001 /**< Supports Bye-Packet */
+#define NODE_A_PONG_CACHING	0x00000002 /**< Supports Pong-Caching */
+#define NODE_A_PONG_ALIEN	0x00000004 /**< Alien Pong-Caching scheme */
+#define NODE_A_QHD_NO_VTAG	0x00000008 /**< Servent has no vendor tag in QHD */
+#define NODE_A_RX_INFLATE	0x00000010 /**< Reading compressed data */
+#define NODE_A_TX_DEFLATE	0x00000020 /**< Sending compressed data */
+#define NODE_A_ULTRA		0x00000040 /**< Node wants to be an Ultrapeer */
+#define NODE_A_NO_ULTRA		0x00000080 /**< Node is NOT ultra capable */
+#define NODE_A_UP_QRP		0x00000100 /**< Supports intra-UP QRP */
+#define NODE_A_GUIDANCE		0x00000200 /**< Can leaf-guide dyn queries */
+#define NODE_A_TIME_SYNC	0x00000400 /**< Supports time sync */
+#define NODE_A_CRAWLABLE	0x00000800 /**< Node can be UDP-crawled */
+#define NODE_A_DYN_QUERY	0x00001000 /**< Node can perform dynamic queries */
+#define NODE_A_CAN_SFLAG	0x00002000 /**< Node supports flags in headers */
+ 
+#define NODE_A_NO_KEPT_ZERO	0x01000000 /**< For GTKG < 2006-08-15: no kept=0! */
+#define NODE_A_NO_DUPS		0x02000000 /**< For broken old GTKG: no dups! */
+#define NODE_A_CAN_HSEP		0x04000000 /**< Node supports HSEP */
+#define NODE_A_CAN_QRP		0x08000000 /**< Node supports query routing */
+#define NODE_A_CAN_VENDOR	0x10000000 /**< Node supports vendor messages */
+#define NODE_A_CAN_GGEP		0x20000000 /**< Node supports big pongs, etc.. */
+#define NODE_A_CAN_ULTRA	0x40000000 /**< Node is ultra capable */
+#define NODE_A_CAN_INFLATE	0x80000000 /**< Node capable of inflating */
+ 
 /*
  * UDP crawling "feature" flags.
  */
@@ -398,9 +401,11 @@ typedef struct gnutella_node {
 
 #define NODE_ID(n)				((n)->id)
 
+#define NODE_CAN_SFLAG(n)		((n)->attrs & NODE_A_CAN_SFLAG)
 #define NODE_CAN_GGEP(n)		((n)->attrs & NODE_A_CAN_GGEP)
 #define NODE_UP_QRP(n)			((n)->attrs & NODE_A_UP_QRP)
-#define NODE_GUIDES_QUERY(n)	((n)->attrs & NODE_A_LEAF_GUIDE)
+#define NODE_LEAF_GUIDE(n)		((n)->attrs & NODE_A_GUIDANCE)
+#define NODE_CAN_INFLATE(n)		((n)->attrs & NODE_A_CAN_INFLATE)
 
 /*
  * Peer inspection macros
@@ -416,17 +421,14 @@ typedef struct gnutella_node {
  */
 
 #define node_vendor(n)		((n)->vendor != NULL ? (n)->vendor : "????")
+#define node_type(n)		\
+	(NODE_IS_LEAF(n) ? "leaf" : NODE_IS_ULTRA(n) ? "ultra" : "legacy")
 
 #define node_inc_sent(n)            node_add_sent(n, 1)
 #define node_inc_txdrop(n)          node_add_txdrop(n, 1)
 #define node_inc_rxdrop(n)          node_add_rxdrop(n, 1)
 
 #define node_add_tx_given(n,x)		do { (n)->tx_given += (x); } while (0)
-#define node_add_tx_written(n,x)	do { (n)->tx_written += (x); } while (0)
-#define node_add_tx_deflated(n,x)	do { (n)->tx_deflated += (x); } while (0)
-
-#define node_add_rx_given(n,x)		do { (n)->rx_given += (x); } while (0)
-#define node_add_rx_inflated(n,x)	do { (n)->rx_inflated += (x); } while (0)
 #define node_add_rx_read(n,x)		do { (n)->rx_read += (x); } while (0)
 
 #define node_inc_tx_query(n)		do { (n)->tx_queries++; } while (0)
@@ -471,7 +473,7 @@ typedef struct gnutella_node {
  */
 #define node_flowc_swift_grace(n)	(NODE_IS_LEAF(n) ? 210 : 30)
 #define node_flowc_swift_period(n)	(NODE_IS_LEAF(n) ? 140 : 20)
-	
+
 /*
  * Global Data
  */
@@ -500,9 +502,11 @@ guint node_keep_missing(void);
 guint node_missing(void);
 guint node_leaves_missing(void);
 guint node_outdegree(void);
-gboolean node_is_connected(guint32 ip, guint16 port, gboolean incoming);
-gboolean node_host_is_connected(guint32 ip, guint16 port);
-void node_add_socket(struct gnutella_socket *s, guint32 ip, guint16 port);
+gboolean node_is_connected(const host_addr_t addr, guint16 port,
+		gboolean incoming);
+gboolean node_host_is_connected(const host_addr_t addr, guint16 port);
+void node_add_socket(struct gnutella_socket *s, const host_addr_t addr,
+		guint16 port, guint32 flags);
 void node_remove(struct gnutella_node *,
 	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
 void node_bye(gnutella_node_t *, gint code,
@@ -548,8 +552,9 @@ void node_set_vendor(gnutella_node_t *n, const gchar *vendor);
 void node_set_hops_flow(gnutella_node_t *n, guint8 hops);
 void node_set_online_mode(gboolean on);
 void node_current_peermode_changed(node_peer_t mode);
-gchar *node_ip(const gnutella_node_t *n);
-gchar *node_gnet_ip(const gnutella_node_t *n);
+const gchar *node_addr(const gnutella_node_t *n);
+const gchar *node_addr2(const gnutella_node_t *n);
+const gchar *node_gnet_addr(const gnutella_node_t *n);
 
 void node_connect_back(const gnutella_node_t *n, guint16 port);
 void node_connected_back(struct gnutella_socket *s);
@@ -557,30 +562,41 @@ void node_connected_back(struct gnutella_socket *s);
 void node_mark_bad_vendor(struct gnutella_node *n);
 
 void node_proxying_remove(gnutella_node_t *n, gboolean discard);
-gboolean node_proxying_add(gnutella_node_t *n, gchar *guid);
-void node_proxy_add(gnutella_node_t *n, guint32 ip, guint16 port);
+gboolean node_proxying_add(gnutella_node_t *n, const gchar *guid);
+void node_proxy_add(gnutella_node_t *n, const host_addr_t addr, guint16 port);
 void node_proxy_cancel_all(void);
 void node_http_proxies_add(
 	gchar *buf, gint *retval, gpointer arg, guint32 flags);
 GSList *node_push_proxies(void);
 const GSList *node_all_nodes(void);
+const GSList *node_all_but_broken_gtkg(void);
 gnutella_node_t *node_active_by_id(guint32 id);
+void node_set_leaf_guidance(guint32 id, gboolean supported);
 
 void node_became_firewalled(void);
 void node_became_udp_firewalled(void);
 void node_set_socket_rx_size(gint rx_size);
 
-mqueue_t *node_udp_get_outq(void);
+mqueue_t *node_udp_get_outq(enum net_type net);
 void node_udp_enable(void);
 void node_udp_disable(void);
 void node_udp_process(struct gnutella_socket *s);
-gnutella_node_t *node_udp_get_ip_port(guint32 ip, guint16 port);
+gnutella_node_t *node_udp_get_addr_port(const host_addr_t addr, guint16 port);
 
 void node_can_tsync(gnutella_node_t *n);
 void node_crawl(gnutella_node_t *n, gint ucnt, gint lcnt, guint8 features);
 
 void node_update_udp_socket(void);
-void node_check_remote_ip_header(guint32 peer, header_t *head);
+void node_check_remote_ip_header(const host_addr_t peer, header_t *head);
+
+guint feed_host_cache_from_headers(header_t *headers,
+	host_type_t sender, gboolean gnet, const host_addr_t peer);
+
+gnutella_node_t *node_browse_prepare(
+	gnet_host_t *host, const gchar *vendor, gnutella_header_t *header,
+	gchar *data, guint32 size);
+void node_browse_cleanup(gnutella_node_t *n);
+void node_kill_hostiles(void);
 
 #endif /* _core_nodes_h_ */
 
