@@ -1,5 +1,5 @@
 /*
- * $Id: oob_proxy.c,v 1.15 2005/06/29 14:24:24 daichik Exp $
+ * $Id: oob_proxy.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,13 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: oob_proxy.c,v 1.15 2005/06/29 14:24:24 daichik Exp $");
+RCSID("$Id: oob_proxy.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "oob_proxy.h"
 #include "share.h"
 #include "nodes.h"
 #include "routing.h"
 #include "settings.h"
+#include "sockets.h"		/* For socket_listen_addr() */
 #include "dq.h"
 #include "dh.h"
 #include "gnet_stats.h"
@@ -55,7 +56,12 @@ RCSID("$Id: oob_proxy.c,v 1.15 2005/06/29 14:24:24 daichik Exp $");
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-#define PROXY_EXPIRE_MS		(10*60*1000)	/**< 10 minutes at most */
+/*
+ * The following should be larger than the dynamic query maximum lifetime
+ * in case we OOB-proxy a query from a leaf because it does not send us
+ * meaningful result indications.
+ */
+#define PROXY_EXPIRE_MS		(11*60*1000)	/**< 11 minutes at most */
 
 /**
  * Record keeping track of the MUID remappings happening for the proxied
@@ -146,8 +152,10 @@ oob_proxy_rec_destroy(cqueue_t *unused_cq, gpointer obj)
 
 	(void) unused_cq;
 
-	if (query_debug)
-		printf("OOB proxied query %s expired\n", guid_hex_str(opr->leaf_muid));
+	if (query_debug || oob_proxy_debug)
+		g_message("OOB proxied query leaf-MUID=%s proxied-MUID=%s expired",
+			guid_hex_str(opr->leaf_muid),
+			data_hex_str(opr->proxied_muid, GUID_RAW_SIZE));
 
 	opr->expire_ev = NULL;		/* The timer which just triggered */
 	oob_proxy_rec_free_remove(opr);
@@ -159,26 +167,30 @@ oob_proxy_rec_destroy(cqueue_t *unused_cq, gpointer obj)
 void
 oob_proxy_create(gnutella_node_t *n)
 {
-	gchar proxied_muid[16];
+	gchar proxied_muid[GUID_RAW_SIZE];
 	struct oob_proxy_rec *opr;
+	guint32 ip;
 
-	g_assert(n->header.function == GTA_MSG_SEARCH);
+	g_assert(gnutella_header_get_function(&n->header) == GTA_MSG_SEARCH);
 	g_assert(NODE_IS_LEAF(n));
-	g_assert(n->header.hops == 1);
+	/* Hops can be 0 when called from DQ layer */
+	g_assert(gnutella_header_get_hops(&n->header) <= 1);
 
 	/*
 	 * Mangle the MUID of the query to insert our own IP:port.
 	 */
 
-	memcpy(proxied_muid, n->header.muid, 16);
-	WRITE_GUINT32_BE(listen_ip(), &proxied_muid[0]);
-	WRITE_GUINT16_LE(listen_port, &proxied_muid[13]);
+	ip = host_addr_ipv4(listen_addr()); /* @todo TODO: IPv6 */
+	memcpy(proxied_muid, gnutella_header_get_muid(&n->header), 16);
+	poke_be32(&proxied_muid[0], ip);
+	poke_le16(&proxied_muid[13], socket_listen_port());
 
 	/*
 	 * Record the mapping, and make sure it expires in PROXY_EXPIRE_MS.
 	 */
 
-	opr = oob_proxy_rec_make(n->header.muid, proxied_muid, n->id);
+	opr = oob_proxy_rec_make(gnutella_header_get_muid(&n->header),
+			proxied_muid, n->id);
 	g_hash_table_insert(proxied_queries, (gchar *) opr->proxied_muid, opr);
 
 	opr->expire_ev = cq_insert(callout_queue, PROXY_EXPIRE_MS,
@@ -189,15 +201,15 @@ oob_proxy_create(gnutella_node_t *n)
 	 */
 
 	query_set_oob_flag(n, n->data);
-	memcpy(n->header.muid, proxied_muid, 16);
+	gnutella_header_set_muid(&n->header, proxied_muid);
 
-	message_add(n->header.muid, GTA_MSG_SEARCH, NULL);
+	message_add(gnutella_header_get_muid(&n->header), GTA_MSG_SEARCH, NULL);
 
-	if (query_debug > 5) {
-		gchar *orig = g_strdup(guid_hex_str(opr->leaf_muid));
-		printf("QUERY OOB-proxying query %s from %s <%s> as %s\n",
-			orig, node_ip(n), node_vendor(n), guid_hex_str(opr->proxied_muid));
-		g_free(orig);
+	if (query_debug > 5 || oob_proxy_debug) {
+		g_message("QUERY OOB-proxying query %s from %s <%s> as %s",
+			data_hex_str(opr->leaf_muid, GUID_RAW_SIZE),
+			node_addr(n), node_vendor(n),
+			guid_hex_str(opr->proxied_muid));
 	}
 }
 
@@ -217,12 +229,13 @@ oob_proxy_create(gnutella_node_t *n)
  */
 gboolean
 oob_proxy_pending_results(
-	gnutella_node_t *n, gchar *muid, gint hits, gboolean uu_udp_firewalled)
+	gnutella_node_t *n, const gchar *muid,
+	gint hits, gboolean uu_udp_firewalled)
 {
 	struct oob_proxy_rec *opr;
 	struct gnutella_node *leaf;
 	guint32 wanted;
-	gchar *msg = NULL;
+	const gchar *msg = NULL;
 
 	(void) uu_udp_firewalled;
 
@@ -250,6 +263,35 @@ oob_proxy_pending_results(
 		goto ignore;		/* Leaf gone, drop the message */
 	}
 
+	/*
+	 * Before letting the dynamic query know about those OOB results, see
+	 * whether we're going to claim then.  There's no need to account for
+	 * a flood of results we will not propagate.
+	 *
+	 * This may cause the dynamic query to continue sending the query to
+	 * other UPs, but if we account for the results and they are not sent
+	 * back to the leaf, it will have a poor search experience, so there is
+	 * a balance to keep between the amount of query flooding we do and the
+	 * amount of results people will get.
+	 *		--RAM, 2006-08-16
+	 */
+
+	if (NODE_IN_TX_FLOW_CONTROL(leaf)) {
+		msg = "leaf in TX flow-control";
+		goto ignore;
+	}
+
+	/*
+	 * If we would not route the hits should we get them, there's no
+	 * need to claim them at all.
+	 */
+
+	if (!dh_would_route(opr->leaf_muid, leaf)) {
+		msg = "would not route hits to leaf";
+		goto ignore;
+	}
+
+
 	/*
 	 * Let the dynamic query know about pending hits, to help it
 	 * measure its popularity.  This also enables us to see whether
@@ -263,7 +305,7 @@ oob_proxy_pending_results(
 
 	/*
 	 * Lookup the dynamic query, to see whether it has not already
-	 * received the maximum amout of results, or whether the search
+	 * received the maximum amount of results, or whether the search
 	 * was not cancelled by the leaf.
 	 */
 
@@ -281,40 +323,28 @@ oob_proxy_pending_results(
 		goto ignore;
 	}
 
-	if (NODE_IN_TX_FLOW_CONTROL(leaf)) {
-		msg = "leaf in TX flow-control";
-		goto ignore;
-	}
-
-
-	/*
-	 * If we would not route the hits should we get them, there's no
-	 * need to claim them at all.
-	 */
-
-	if (!dh_would_route(opr->leaf_muid, leaf)) {
-		msg = "would not route hits to leaf";
-		goto ignore;
-	}
-
 	/*
 	 * Claim the results (all of it).
 	 */
 
-	vmsg_send_oob_reply_ack(n, muid, MIN(hits, 254));
+	if (query_debug > 5 || oob_proxy_debug > 1)
+		g_message("QUERY OOB-proxied %s notified of %d hits at %s %s"
+			" for leaf #%u %s, wants %u",
+			guid_hex_str(muid), hits,
+			NODE_IS_UDP(n) ? "UDP" : "TCP", node_addr(n), opr->node_id,
+			leaf == NULL ? "???" : node_gnet_addr(leaf), wanted);
 
-	if (query_debug > 5)
-		printf("QUERY OOB-proxied %s notified of %d hits at %s, wants %u\n",
-			guid_hex_str(muid), hits, node_ip(n), wanted);
+	vmsg_send_oob_reply_ack(n, muid, MIN(hits, 254));
 
 	return TRUE;
 
 ignore:
-	if (query_debug > 5)
-		printf("QUERY OOB-proxied %s "
-			"notified of %d hits at %s for %s, ignored (%s)\n",
-			guid_hex_str(muid), hits, node_ip(n),
-			leaf == NULL ? "???" : ip_to_gchar(leaf->ip), msg);
+	if (query_debug > 5 || oob_proxy_debug > 1)
+		g_message("QUERY OOB-proxied %s "
+			"notified of %d hits at %s %s for leaf #%u %s, ignored (%s)",
+			guid_hex_str(muid), hits,
+			NODE_IS_UDP(n) ? "UDP" : "TCP", node_addr(n), opr->node_id,
+			leaf == NULL ? "???" : node_gnet_addr(leaf), msg);
 
 	return TRUE;
 }
@@ -337,10 +367,11 @@ oob_proxy_got_results(gnutella_node_t *n, guint results)
 	struct oob_proxy_rec *opr;
 	struct gnutella_node *leaf;
 
-	g_assert(n->header.function == GTA_MSG_SEARCH_RESULTS);
+	g_assert(gnutella_header_get_function(&n->header) == GTA_MSG_SEARCH_RESULTS);
 	g_assert(results > 0 && results <= INT_MAX);
 
-	opr = g_hash_table_lookup(proxied_queries, n->header.muid);
+	opr = g_hash_table_lookup(proxied_queries,
+				gnutella_header_get_muid(&n->header));
 	if (opr == NULL)
 		return FALSE;
 
@@ -358,6 +389,14 @@ oob_proxy_got_results(gnutella_node_t *n, guint results)
 	leaf = node_active_by_id(opr->node_id);
 	if (leaf == NULL) {
 		gnet_stats_count_dropped(n, MSG_DROP_ROUTE_LOST);
+
+		if (query_debug > 5 || oob_proxy_debug > 1)
+			g_message(
+				"QUERY OOB-proxied %s dropping %d hit%s from %s: no leaf #%u",
+				guid_hex_str(opr->proxied_muid),
+				results, results == 1 ? "" : "s",
+				node_addr(n), opr->node_id);
+
 		return TRUE;		/* Leaf gone, drop the message */
 	}
 
@@ -386,33 +425,39 @@ oob_proxy_got_results(gnutella_node_t *n, guint results)
 	 * the leaf sent us.
 	 */
 
-	memcpy(n->header.muid, opr->leaf_muid, 16);
-	if (n->header.ttl == 0)
-		n->header.ttl++;
+	gnutella_header_set_muid(&n->header, opr->leaf_muid);
+	if (gnutella_header_get_ttl(&n->header) == 0)
+		gnutella_header_set_ttl(&n->header, 1);
 
 	/*
 	 * Route message to leaf node.
 	 */
 
-	g_assert(n->header.hops > 0);	/* Went through route_message() already */
+	/* Went through route_message() already */
+	g_assert(gnutella_header_get_hops(&n->header) > 0);
 
 	dh_route(n, leaf, results);
 
-	if (query_debug > 5)
-		printf("QUERY OOB-proxied %s routed %d hit%s to %s <%s> from %s\n",
+	if (query_debug > 5 || oob_proxy_debug > 1)
+		g_message("QUERY OOB-proxied %s routed %d hit%s to %s <%s> from %s %s",
 			guid_hex_str(opr->proxied_muid), results, results == 1 ? "" : "s",
-			node_ip(leaf), node_vendor(leaf), NODE_IS_UDP(n) ? "UDP" : "TCP");
+			node_addr(leaf), node_vendor(leaf),
+			NODE_IS_UDP(n) ? "UDP" : "TCP", node_addr2(n));
 
 	return TRUE;			/* We routed the message */
 }
 
 /**
  * Check whether MUID is for an OOB-proxied query.
+ * @return NULL if the MUID is unknown, otherwise the original leaf MUID.
  */
-gboolean
-oob_proxy_muid_proxied(gchar *muid)
+const gchar *
+oob_proxy_muid_proxied(const gchar *muid)
 {
-	return g_hash_table_lookup(proxied_queries, muid) ? TRUE : FALSE;
+	const struct oob_proxy_rec *opr;
+	
+	opr = g_hash_table_lookup(proxied_queries, muid);
+	return opr ? opr->leaf_muid : NULL;
 }
 
 /**
