@@ -1,5 +1,5 @@
 /*
- * $Id: pmsg.c,v 1.9 2005/06/25 01:37:40 daichik Exp $
+ * $Id: pmsg.c 11989 2006-09-16 17:33:15Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pmsg.c,v 1.9 2005/06/25 01:37:40 daichik Exp $");
+RCSID("$Id: pmsg.c 11989 2006-09-16 17:33:15Z rmanfredi $")
 
 #include "pmsg.h"
 
@@ -91,7 +91,7 @@ pmsg_close(void)
  * Compute message's size.
  */
 int
-pmsg_size(pmsg_t *mb)
+pmsg_size(const pmsg_t *mb)
 {
 	int msize = 0;
 
@@ -376,6 +376,23 @@ pmsg_free(pmsg_t *mb)
 	} while (0);
 }
 
+/**
+ * @return amount of data that can be written at the end of the message.
+ */
+gint
+pmsg_writable_length(const pmsg_t *mb)
+{
+	pdata_t *arena = mb->m_data;
+	gint available = arena->d_end - mb->m_wptr;
+
+	/*
+	 * If buffer is not writable (shared among several readers), it is
+	 * forbidden to write any new data to it.
+	 */
+
+	return pmsg_is_writable(mb) ? available : 0;
+}
+
 /**
  * Write data at the end of the message.
  * The message must be the only reference to the underlying data.
@@ -421,6 +438,38 @@ pmsg_read(pmsg_t *mb, gpointer data, gint len)
 	return readable;
 }
 
+/**
+ * Discard data from the message, returning the amount of bytes discarded.
+ */
+gint
+pmsg_discard(pmsg_t *mb, gint len)
+{
+	gint available = mb->m_wptr - mb->m_rptr;
+	gint n = len >= available ? available : len;
+
+	g_assert(available >= 0);		/* Data cannot go beyond end of arena */
+
+	mb->m_rptr += n;
+	return n;
+}
+
+
+/**
+ * Discard trailing data from the message, returning the amount of
+ * bytes discarded.
+ */
+gint
+pmsg_discard_trailing(pmsg_t *mb, gint len)
+{
+	gint available = mb->m_wptr - mb->m_rptr;
+	gint n = len >= available ? available : len;
+
+	g_assert(available >= 0);		/* Data cannot go beyond end of arena */
+
+	mb->m_wptr -= n;
+	return n;
+}
+
 /**
  * Allocate a new data block of given size.
  * The block header is at the start of the allocated block.
@@ -484,7 +533,7 @@ pdata_allocb_ext(void *buf, gint len, pdata_free_t freecb, gpointer freearg)
 	g_assert(valid_ptr(buf));
 	g_assert(implies(freecb, valid_ptr(freecb)));
 
-	db = g_malloc(sizeof(*db));
+	db = walloc(sizeof(*db));
 
 	db->d_arena = buf;
 	db->d_end = (gchar *) buf + len;
@@ -493,6 +542,7 @@ pdata_allocb_ext(void *buf, gint len, pdata_free_t freecb, gpointer freearg)
 	db->d_arg = freearg;
 
 	g_assert((size_t) len == pdata_len(db));
+	g_assert(db->d_arena != db->d_embedded);
 
 	return db;
 }
@@ -526,11 +576,14 @@ pdata_free(pdata_t *db)
 		gpointer p = is_embedded ? (gpointer) db : (gpointer) db->d_arena;
 		(*db->d_free)(p, db->d_arg);
 		if (!is_embedded)
-			G_FREE_NULL(db);
+			wfree(db, sizeof(*db));
 	} else {
-		if (!is_embedded)
+		if (!is_embedded) {
 			G_FREE_NULL(db->d_arena);
-		G_FREE_NULL(db);
+			wfree(db, sizeof(*db));
+		} else {
+			G_FREE_NULL(db);
+		}
 	}
 }
 
