@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.h,v 1.4 2005/06/25 01:37:40 daichik Exp $
+ * $Id: qhit.h 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -50,11 +50,14 @@ void qhit_init(void);
 void qhit_close(void);
 
 void qhit_send_results(
-	struct gnutella_node *n, GSList *files, gint count, gboolean use_ggep_h);
+	struct gnutella_node *n, GSList *files, gint count,
+	const gchar *muid, gboolean use_ggep_h);
 void qhit_build_results(
+	const GSList *files, gint count,
+	size_t max_msgsize,
 	qhit_process_t cb, gpointer udata,
-	gchar *muid, GSList *files, gint count, gboolean use_ggep_h);
+	const gchar *muid, gboolean use_ggep_h);
 
 #endif /* _core_qhit_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
