@@ -1,5 +1,5 @@
 /*
- * $Id: routing.c,v 1.37 2005/10/20 06:39:19 cbiere Exp $
+ * $Id: routing.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: routing.c,v 1.37 2005/10/20 06:39:19 cbiere Exp $");
+RCSID("$Id: routing.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "search.h"			/* For search_passive. */
 #include "routing.h"
@@ -244,6 +244,9 @@ routing_log_set_new(struct route_log *log)
 	log->new = TRUE;
 }
 
+static void routing_log_extra(struct route_log *log,
+	const gchar *fmt, ...) G_GNUC_PRINTF(2, 3);
+
 /**
  * Record extra logging information, appending to existing information.
  */
@@ -351,7 +354,8 @@ routing_log_flush(struct route_log *log)
  *
  * @return copy of mangled MUID as pointer to static data.
  */
-static const gchar *route_mangled_oob_muid(const gchar *muid)
+static const gchar *
+route_mangled_oob_muid(const gchar *muid)
 {
 	static gchar mangled[GUID_RAW_SIZE];
 
@@ -366,10 +370,10 @@ static const gchar *route_mangled_oob_muid(const gchar *muid)
 /**
  * Used to ensure type safety when accessing the routing_data field
  */
-static struct route_data *
+static inline struct route_data *
 get_routing_data(struct gnutella_node *n)
 {
-	return (struct route_data *)(n->routing_data);
+	return n->routing_data;
 }
 
 /**
@@ -388,7 +392,7 @@ init_routing_data(struct gnutella_node *node)
 	 * Allocate and link some routing data to it
 	 */
 
-	route = (struct route_data *) walloc(sizeof(struct route_data));
+	route = walloc(sizeof *route);
 
 	route->node = node;
 	route->saved_messages = 0;
@@ -454,6 +458,8 @@ get_next_slot(void)
 	guint chunk_idx;
 	struct message **chunk;
 	struct message **slot = NULL;
+	time_t now = tm_time();
+	time_delta_t elapsed = delta_time(now, routing.last_rotation);
 
 	idx = routing.next_idx;
 	chunk_idx = CHUNK_INDEX(idx);
@@ -463,7 +469,6 @@ get_next_slot(void)
 	chunk = routing.chunks[chunk_idx];
 
 	if (chunk == NULL) {
-		time_t now = tm_time();
 
 		g_assert((gint) idx >= routing.capacity);
 
@@ -472,15 +477,10 @@ get_next_slot(void)
 		 * it or recycle the table by going back to the start.
 		 */
 
-		if (
-			idx > 0 &&
-			delta_time(now, routing.last_rotation) > TABLE_MIN_CYCLE
-		) {
-			gint elapsed = now - routing.last_rotation;
-
+		if (idx > 0 && elapsed > TABLE_MIN_CYCLE) {
 			if (routing_debug)
 				printf("RT cycling over table, elapsed=%d, holds %d / %d\n",
-					elapsed, routing.count, routing.capacity);
+					(gint) elapsed, routing.count, routing.capacity);
 
 			chunk_idx = 0;
 			idx = routing.next_idx = 0;
@@ -494,7 +494,7 @@ get_next_slot(void)
 			g_assert(idx == 0 || chunk_idx > 0);
 
 			routing.capacity += CHUNK_MESSAGES;
-			routing.chunks[chunk_idx] = (struct message **)
+			routing.chunks[chunk_idx] =
 				g_malloc0(CHUNK_MESSAGES * sizeof(struct message *));
 
 			if (routing_debug)
@@ -513,12 +513,9 @@ get_next_slot(void)
 		 */
 
 		if (idx == 0) {
-			time_t now = tm_time();
-			gint elapsed = now - routing.last_rotation;
-
 			if (routing_debug)
 				printf("RT cycling over FORCED, elapsed=%d, holds %d / %d\n",
-					elapsed, routing.count, routing.capacity);
+					(gint) elapsed, routing.count, routing.capacity);
 
 			routing.last_rotation = now;
 		}
@@ -611,8 +608,8 @@ revitalize_entry(struct message *entry, gboolean force)
 static gboolean
 node_sent_message(struct gnutella_node *n, struct message *m)
 {
-	GSList *l;
-	struct route_data * route;
+	struct route_data *route;
+	GSList *sl;
 
 	if (n == fake_node)
 		route = &fake_route;
@@ -627,8 +624,8 @@ node_sent_message(struct gnutella_node *n, struct message *m)
 	if (route == NULL)
 		return FALSE;
 
-	for (l = m->routes; l; l = l->next) {
-		if (route == ((struct route_data *) l->data))
+	for (sl = m->routes; sl; sl = g_slist_next(sl)) {
+		if (route == sl->data)
 			return TRUE;
 	}
 
@@ -648,7 +645,7 @@ node_ttl_higher(struct gnutella_node *n, struct message *m, guint8 ttl)
 {
 	GSList *l;
 	gint i;
-	struct route_data * route;
+	struct route_data *route;
 
 	g_assert(n != fake_node);
 	g_assert(
@@ -660,7 +657,7 @@ node_ttl_higher(struct gnutella_node *n, struct message *m, guint8 ttl)
 	g_assert(route != NULL);
 
 	for (l = m->routes, i = 0; l; l = g_slist_next(l), i++) {
-		if (route == ((struct route_data *) l->data)) {
+		if (route == l->data) {
 			GSList *t = g_slist_nth(m->ttls, i);
 			guint8 old_ttl;
 
@@ -682,13 +679,11 @@ node_ttl_higher(struct gnutella_node *n, struct message *m, guint8 ttl)
  * compares two message structures
  */
 static gint
-message_compare_func(gconstpointer a, gconstpointer b)
+message_compare_func(gconstpointer p, gconstpointer q)
 {
-	return
-		0 == memcmp(((const struct message *) a)->muid,
-			((const struct message *) b)->muid, 16)
-		&& ((const struct message *) a)->function ==
-			((const struct message *) b)->function;
+	const struct message *a = p, *b = q;
+
+	return a->function == b->function && 0 == memcmp(a->muid, b->muid, 16);
 }
 
 /**
@@ -697,21 +692,13 @@ message_compare_func(gconstpointer a, gconstpointer b)
 static guint
 message_hash_func(gconstpointer key)
 {
-	int count;
-	guint hash = 0;
+	const struct message *msg = key;
+	guint hash, i;
 
-	for (count = 0; count <= 12; count += 4) {
-		guint hashadd =
-			( (const struct message *) key)->muid[count]            |
-			(((const struct message *) key)->muid[count + 1] << 8)  |
-			(((const struct message *) key)->muid[count + 2] << 16) |
-			(((const struct message *) key)->muid[count + 3] << 24);
-
-		hash ^= hashadd;
+	hash = msg->function;
+	for (i = 0; i < 4; i++) {
+		hash ^= peek_le32(&msg->muid[i * 4]);
 	}
-
-	hash ^= (guint) ((const struct message *) key)->function;
-
 	return hash;
 }
 
@@ -789,18 +776,25 @@ retry:
 	 * Initialize message type array for routing logs.
 	 */
 
-	for (i = 0; i < 256; i++)
-		debug_msg[i] = "UNKN ";
+	for (i = 0; i < 256; i++) {
+		const gchar *s;
 
-	debug_msg[GTA_MSG_INIT]           = "Ping ";
-	debug_msg[GTA_MSG_INIT_RESPONSE]  = "Pong ";
-	debug_msg[GTA_MSG_SEARCH]         = "Query";
-	debug_msg[GTA_MSG_SEARCH_RESULTS] = "Q-Hit";
-	debug_msg[GTA_MSG_PUSH_REQUEST]   = "Push ";
-	debug_msg[GTA_MSG_VENDOR]         = "Vndor";
-	debug_msg[GTA_MSG_STANDARD]       = "Vstd ";
-	debug_msg[GTA_MSG_QRP]            = "QRP  ";
-	debug_msg[GTA_MSG_HSEP_DATA]      = "HSEP ";
+		s = "UNKN ";
+		switch ((enum gta_msg) i) {
+		case GTA_MSG_INIT:           s = "Ping "; break;
+		case GTA_MSG_INIT_RESPONSE:  s = "Pong "; break;
+		case GTA_MSG_SEARCH:         s = "Query"; break;
+		case GTA_MSG_SEARCH_RESULTS: s = "Q-Hit"; break;
+		case GTA_MSG_PUSH_REQUEST:   s = "Push "; break;
+		case GTA_MSG_RUDP:   		 s = "RUDP "; break;
+		case GTA_MSG_VENDOR:         s = "Vndor"; break;
+		case GTA_MSG_STANDARD:       s = "Vstd "; break;
+		case GTA_MSG_QRP:            s = "QRP  "; break;
+		case GTA_MSG_HSEP_DATA:      s = "HSEP "; break;
+		case GTA_MSG_BYE:      		 s = "Bye  "; break;
+		}
+		debug_msg[i] = s;
+	}
 
 	/*
 	 * Should be around for life of program, so should *never*
@@ -824,7 +818,7 @@ retry:
  * Generate a new muid and put it in a message header.
  */
 void
-message_set_muid(struct gnutella_header *header, guint8 function)
+message_set_muid(gnutella_header_t *header, guint8 function)
 {
 	switch (function) {
 	case GTA_MSG_PUSH_REQUEST:
@@ -833,14 +827,15 @@ message_set_muid(struct gnutella_header *header, guint8 function)
 	case GTA_MSG_HSEP_DATA:
 	case GTA_MSG_STANDARD:
 	case GTA_MSG_VENDOR:		/* When a non-blank random GUID is needed */
-		guid_random_muid(header->muid);
-		break;
+		guid_random_muid(gnutella_header_muid(header));
+		return;
 	case GTA_MSG_INIT:
-		guid_ping_muid(header->muid);
-		break;
-	default:
-		g_error("unexpected message type %d", function);
+		guid_ping_muid(gnutella_header_muid(header));
+		return;
+	case GTA_MSG_RUDP:
+		g_assert_not_reached();
 	}
+	g_error("unexpected message type %d", function);
 }
 
 /**
@@ -877,12 +872,12 @@ remove_one_message_reference(struct route_data *rd)
 static void
 free_route_list(struct message *m)
 {
-	GSList *l;
+	GSList *sl;
 
 	g_assert(m);
 
-	for (l = m->routes; l; l = l->next)
-		remove_one_message_reference((struct route_data *) l->data);
+	for (sl = m->routes; sl; sl = g_slist_next(sl))
+		remove_one_message_reference(sl->data);
 
 	g_slist_free(m->routes);
 	m->routes = NULL;
@@ -1015,7 +1010,8 @@ message_add(const gchar *muid, guint8 function, struct gnutella_node *node)
 		 *		--RAM, 2005-10-02
 		 */
 
-		ttl = node == fake_node ? my_ttl : node->header.ttl;
+		ttl = node == fake_node
+				? my_ttl : gnutella_header_get_ttl(&node->header);
 
 		switch (function) {
 		case GTA_MSG_PUSH_REQUEST:
@@ -1033,7 +1029,7 @@ message_add(const gchar *muid, guint8 function, struct gnutella_node *node)
 	 */
 
 	if (node != fake_node)
-		entry->ttl = node->header.ttl;
+		entry->ttl = gnutella_header_get_ttl(&node->header);
 
 	/* insert the new message into the hash table */
 	g_hash_table_insert(routing.messages_hashed, entry, entry);
@@ -1046,18 +1042,18 @@ message_add(const gchar *muid, guint8 function, struct gnutella_node *node)
 static void
 purge_dangling_references(struct message *m)
 {
-	GSList *l;
+	GSList *sl;
 	GSList *t;
 
-	for (l = m->routes, t = m->ttls; l; /* empty */) {
-		struct route_data *rd = (struct route_data *) l->data;
+	for (sl = m->routes, t = m->ttls; sl; /* empty */) {
+		struct route_data *rd = sl->data;
 
 		if (rd->node == NULL) {
-			GSList *next = g_slist_next(l);
-			m->routes = g_slist_remove_link(m->routes, l);
+			GSList *next = g_slist_next(sl);
+			m->routes = g_slist_remove_link(m->routes, sl);
 			remove_one_message_reference(rd);
-			g_slist_free_1(l);
-			l = next;
+			g_slist_free_1(sl);
+			sl = next;
 
 			if (t) {
 				next = g_slist_next(t);
@@ -1066,7 +1062,7 @@ purge_dangling_references(struct message *m)
 				t = next;
 			}
 		} else {
-			l = g_slist_next(l);
+			sl = g_slist_next(sl);
 			if (t)
 				t = g_slist_next(t);
 		}
@@ -1093,8 +1089,7 @@ find_message(const gchar *muid, guint8 function, struct message **m)
 	memcpy(dummyMessage.muid, muid, 16);
 	dummyMessage.function = function;
 
-	found_message = (struct message *)
-		g_hash_table_lookup(routing.messages_hashed, &dummyMessage);
+	found_message = g_hash_table_lookup(routing.messages_hashed, &dummyMessage);
 
 	if (!found_message) {
 		*m = NULL;
@@ -1124,7 +1119,7 @@ check_hops_ttl(struct route_log *log, struct gnutella_node *sender)
 	 *		--RAM, 04/07/2002
 	 */
 
-	if (sender->header.hops == 255) {
+	if (gnutella_header_get_hops(&sender->header) == 255) {
 		routing_log_extra(log, "max hop count reached");
 		gnet_stats_count_dropped(sender, MSG_DROP_MAX_HOP_COUNT);
 		sender->n_bad++;
@@ -1133,7 +1128,7 @@ check_hops_ttl(struct route_log *log, struct gnutella_node *sender)
 		return FALSE;	/* Don't route, something is wrong */
 	}
 
-	if (sender->header.ttl == 0) {
+	if (gnutella_header_get_ttl(&sender->header) == 0) {
 		routing_log_extra(log, "TTL was 0");
 		node_sent_ttl0(sender);
 		return FALSE;	/* Don't route */
@@ -1173,7 +1168,10 @@ forward_message(
 	routing_log_set_new(log);
 
 	/* Drop messages that would travel way too many nodes --RAM */
-	if ((guint32) sender->header.ttl + sender->header.hops > hard_ttl_limit) {
+	if (
+		(guint32) gnutella_header_get_ttl(&sender->header) +
+			gnutella_header_get_hops(&sender->header) > hard_ttl_limit
+	) {
 		routing_log_extra(log, "hard TTL limit reached");
 
 		/*
@@ -1192,8 +1190,8 @@ forward_message(
         gnet_stats_count_dropped(sender, MSG_DROP_HARD_TTL_LIMIT);
 
 		if (
-			sender->header.function == GTA_MSG_SEARCH &&
-			sender->header.hops <= max_high_ttl_radius &&
+			gnutella_header_get_function(&sender->header) == GTA_MSG_SEARCH &&
+			gnutella_header_get_hops(&sender->header) <= max_high_ttl_radius &&
 			sender->n_hard_ttl > max_high_ttl_msg &&
 			!NODE_IS_UDP(sender)
 		) {
@@ -1209,7 +1207,7 @@ forward_message(
 	if (!check_hops_ttl(log, sender))
 		return TRUE;
 
-	if (sender->header.ttl == 1) {
+	if (gnutella_header_get_ttl(&sender->header) == 1) {
 		/* TTL expired, message stops here */
 		routing_log_extra(log, "TTL expired");
 		gnet_stats_count_expired(sender);
@@ -1224,7 +1222,8 @@ forward_message(
 			GSList *nodes = NULL;
 			gint count;
 
-			g_assert(sender->header.function == GTA_MSG_PUSH_REQUEST);
+			g_assert(gnutella_header_get_function(&sender->header)
+					== GTA_MSG_PUSH_REQUEST);
 
 			for (l = routes, count = 0; l; l = g_slist_next(l), count++) {
 				struct route_data *rd = (struct route_data *) l->data;
@@ -1270,18 +1269,24 @@ forward_message(
 			 * NB: Account for the fact that we haven't decremented it yet.
 			 */
 
-			if (sender->header.hops + sender->header.ttl > (gint) max_ttl) {
-				gint ttl_max = max_ttl - sender->header.hops;
-				sender->header.ttl = MAX(ttl_max, 1);	/* Trim down */
+			if (
+				(guint) gnutella_header_get_hops(&sender->header) +
+					gnutella_header_get_ttl(&sender->header) > max_ttl
+			) {
+				gint ttl_max;
+			   
+				ttl_max = max_ttl - gnutella_header_get_hops(&sender->header);
+				/* Trim down */
+				gnutella_header_set_ttl(&sender->header, MAX(ttl_max, 1));
 
-				if (sender->header.ttl == 1) {
+				if (gnutella_header_get_ttl(&sender->header) == 1) {
 					/* TTL expired, message stops here */
 					routing_log_extra(log, "TTL forcefully expired");
 					gnet_stats_count_expired(sender);
 					return TRUE;
 				} else
-					routing_log_extra(log,
-						"TTL trimmed down to %d ", sender->header.ttl);
+					routing_log_extra(log, "TTL trimmed down to %d ",
+						gnutella_header_get_ttl(&sender->header));
 			}
 
 			dest->type = duplicate ? ROUTE_NO_DUPS_BUT_ONE : ROUTE_ALL_BUT_ONE;
@@ -1311,7 +1316,10 @@ check_duplicate(struct route_log *log,
 {
 	struct gnutella_node *sender = *node;
 
-	if (find_message(sender->header.muid, sender->header.function, mp)) {
+	if (
+		find_message(gnutella_header_get_muid(&sender->header),
+			gnutella_header_get_function(&sender->header), mp)
+	) {
 		struct message *m = *mp;
 		gboolean forward = FALSE;
 
@@ -1327,12 +1335,13 @@ check_duplicate(struct route_log *log,
 		 *		--RAM, 2004-08-28
 		 */
 
-		if (sender->header.ttl > m->ttl) {
+		if (gnutella_header_get_ttl(&sender->header) > m->ttl) {
 			routing_log_extra(log, "dup message with higher ttl");
 
 			gnet_stats_count_general(GNR_DUPS_WITH_HIGHER_TTL, 1);
 
-			m->ttl = sender->header.ttl;	/* Remember highest TTL */
+			/* Remember highest TTL */
+			m->ttl = gnutella_header_get_ttl(&sender->header);
 
 			forward = TRUE;			/* Forward but don't handle */
 		}
@@ -1356,7 +1365,8 @@ check_duplicate(struct route_log *log,
 			 * the highest TTL seen along this route.
 			 */
 
-			higher_ttl = node_ttl_higher(sender, m, sender->header.ttl);
+			higher_ttl = node_ttl_higher(sender, m,
+							gnutella_header_get_ttl(&sender->header));
 
 			if (higher_ttl)
 				routing_log_extra(log, "dup message (same node, higher TTL)");
@@ -1393,7 +1403,7 @@ check_duplicate(struct route_log *log,
 			} else {
 				if (routing_debug > 2)
 					gmsg_log_bad(sender, "dup message ID %s from same node",
-						guid_hex_str(sender->header.muid));
+					   guid_hex_str(gnutella_header_get_muid(&sender->header)));
 			}
 		} else {
 			if (m->routes == NULL)
@@ -1425,19 +1435,24 @@ check_duplicate(struct route_log *log,
 	 * XXX factorize this code.
 	 */
 
-	if (mangled && find_message(mangled, sender->header.function, mp)) {
+	if (
+		mangled &&
+		find_message(mangled,
+			gnutella_header_get_function(&sender->header), mp)
+	) {
 		struct message *m = *mp;
 
 		g_assert(m != NULL);		/* find_message() succeeded */
 
 		gnet_stats_count_general(GNR_QUERY_OOB_PROXIED_DUPS, 1);
 
-		if (sender->header.ttl > m->ttl) {
+		if (gnutella_header_get_ttl(&sender->header) > m->ttl) {
 			routing_log_extra(log, "dup OOB query with higher ttl");
 
 			gnet_stats_count_general(GNR_DUPS_WITH_HIGHER_TTL, 1);
 
-			m->ttl = sender->header.ttl;	/* Remember highest TTL */
+			/* Remember highest TTL */
+			m->ttl = gnutella_header_get_ttl(&sender->header);
 
 			return TRUE;			/* Forward but don't handle */
 		}
@@ -1514,7 +1529,7 @@ route_push(struct route_log *log,
 	 * If IP address is among the hostile set, drop.
 	 */
 
-	addr = host_addr_set_ipv4(peek_be32(&sender->data[20]));
+	addr = host_addr_get_ipv4(peek_be32(&sender->data[20]));
 
 	if (hostiles_check(addr)) {
 		gnet_stats_count_dropped(sender, MSG_DROP_HOSTILE_IP);
@@ -1628,12 +1643,16 @@ route_query(struct route_log *log,
 
 	if (
 		!(sender->flags & NODE_A_DYN_QUERY) &&
-		sender->header.ttl + sender->header.hops >= (gint) my_ttl
+		(guint) gnutella_header_get_ttl(&sender->header) +
+			gnutella_header_get_hops(&sender->header) >= my_ttl
 	) {
-		gint ttl_max = my_ttl - sender->header.hops - 1;
-		sender->header.ttl = MAX(ttl_max, 1);	/* Trim down */
+		gint ttl_max = my_ttl - gnutella_header_get_hops(&sender->header) - 1;
+	
+		/* Trim down */
+		gnutella_header_set_ttl(&sender->header, MAX(ttl_max, 1));
+
 		routing_log_extra(log, "No dyn. query, TTL forced to %d ",
-			sender->header.ttl);
+			gnutella_header_get_ttl(&sender->header));
 	}
 
 	/* Broadcast */
@@ -1649,7 +1668,6 @@ static gboolean
 route_query_hit(struct route_log *log,
 	struct gnutella_node **node, struct route_dest *dest)
 {
-	GSList *l;
 	struct gnutella_node *sender = *node;
 	struct message *m;
 	gboolean node_is_target = FALSE;
@@ -1717,12 +1735,16 @@ route_query_hit(struct route_log *log,
 	 * the initial query, regardless of the hops/TTL value of the hit.
 	 */
 
-	is_oob_proxied = oob_proxy_muid_proxied(sender->header.muid);
+	is_oob_proxied = NULL != oob_proxy_muid_proxied(
+								gnutella_header_get_muid(&sender->header));
 
 	if (!is_oob_proxied && !check_hops_ttl(log, sender))
 		goto handle;				/* We will handle the hit nonetheless */
 
-	if (!find_message(sender->header.muid, GTA_MSG_SEARCH, &m)) {
+	if (
+		!find_message(gnutella_header_get_muid(&sender->header),
+						GTA_MSG_SEARCH, &m)
+	) {
 		/* We have never seen any request matching this reply ! */
 
 		routing_log_extra(log, "no request matching the reply!");
@@ -1732,7 +1754,7 @@ route_query_hit(struct route_log *log,
 
 		if (routing_debug)
 			gmsg_log_bad(sender, "got reply without matching request %s%s",
-				guid_hex_str(sender->header.muid),
+				guid_hex_str(gnutella_header_get_muid(&sender->header)),
 				is_oob_proxied ? " (OOB-proxied)" : "");
 
 		goto handle;
@@ -1776,7 +1798,7 @@ route_query_hit(struct route_log *log,
 
 	if (is_oob_proxied) {
 		g_warning("BUG: forgot we sent OOB-proxied query %s in routing table!",
-			guid_hex_str(sender->header.muid));
+			guid_hex_str(gnutella_header_get_muid(&sender->header)));
 		node_is_target = TRUE;		/* We are the target of the reply */
 		goto handle;
 	}
@@ -1787,12 +1809,16 @@ route_query_hit(struct route_log *log,
 	 * XXX should remember hops from queries and choose the lowest hop
 	 * XXX route for relaying. --RAM, 2004-08-29
 	 */
+	{
+		GSList *sl;
 
-	for (found = NULL, l = m->routes; l; l = g_slist_next(l)) {
-		struct route_data *route = (struct route_data *) l->data;
-		if (route->node != sender) {
-			found = route->node;
-			break;
+		found = NULL;
+		for (sl = m->routes; sl; sl = g_slist_next(sl)) {
+			struct route_data *route = sl->data;
+			if (route->node != sender) {
+				found = route->node;
+				break;
+			}
 		}
 	}
 
@@ -1805,11 +1831,11 @@ route_query_hit(struct route_log *log,
 	 * are a leaf node, in which case we won't route the message!.
 	 */
 
-	if (sender->header.ttl == 1) {
+	if (gnutella_header_get_ttl(&sender->header) == 1) {
 		if (NODE_IS_LEAF(found)) {
 			/* TTL expired, but target is a leaf node */
 			routing_log_extra(log, "expired TTL bumped");
-			sender->header.ttl++;
+			gnutella_header_set_ttl(&sender->header, 2);
 		} else {
 			/* TTL expired, message stops here in any case */
 			if (current_peermode != NODE_P_LEAF) {
@@ -1852,9 +1878,10 @@ final:
 	 *				--RAM, 15/09/2001
 	 */
 
-	if (sender->header.ttl > hard_ttl_limit + 1) {	/* TTL too large */
+	if (gnutella_header_get_ttl(&sender->header) > hard_ttl_limit + 1) {
+		/* TTL too large */
 		routing_log_extra(log, "TTL adjusted");
-		sender->header.ttl = hard_ttl_limit + 1;
+		gnutella_header_set_ttl(&sender->header, hard_ttl_limit + 1);
 	}
 
 	return TRUE;
@@ -1885,7 +1912,8 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	const gchar *mangled = NULL;
 
 	/* Ensure we never get something bearing our special GUID route marker */
-	g_assert(sender->header.function != QUERY_HIT_ROUTE_SAVE);
+	g_assert(
+		gnutella_header_get_function(&sender->header) != QUERY_HIT_ROUTE_SAVE);
 
 	dest->type = ROUTE_NONE;
 
@@ -1893,8 +1921,11 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	if (sender->routing_data == NULL)
 		init_routing_data(sender);
 
-	routing_log_init(&log, sender, sender->header.muid,
-		sender->header.function, sender->header.hops, sender->header.ttl);
+	routing_log_init(&log, sender,
+		gnutella_header_get_muid(&sender->header),
+		gnutella_header_get_function(&sender->header),
+		gnutella_header_get_hops(&sender->header),
+		gnutella_header_get_ttl(&sender->header));
 
 	/*
 	 * For OOB queries, we have to mangle the MUID to detect duplicates
@@ -1902,10 +1933,11 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	 */
 
 	if (
-		sender->header.function == GTA_MSG_SEARCH &&
+		gnutella_header_get_function(&sender->header) == GTA_MSG_SEARCH &&
 		gmsg_split_is_oob_query(&sender->header, sender->data)
 	) {
-		mangled = route_mangled_oob_muid(sender->header.muid);
+		mangled = route_mangled_oob_muid(
+						gnutella_header_get_muid(&sender->header));
 		gnet_stats_count_general(GNR_OOB_QUERIES, 1);
 	}
 
@@ -1914,7 +1946,7 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	 * whether the message should be handled locally.
 	 */
 
-	switch (sender->header.function) {
+	switch (gnutella_header_get_function(&sender->header)) {
 	case GTA_MSG_PUSH_REQUEST:
 	case GTA_MSG_SEARCH:
 		route_it = check_duplicate(&log, node, mangled, &m);
@@ -1928,7 +1960,8 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 		 * the original one fail due to a node disconnection.
 		 */
 
-		message_add(sender->header.muid, sender->header.function, sender);
+		message_add(gnutella_header_get_muid(&sender->header),
+			gnutella_header_get_function(&sender->header), sender);
 
 		/*
 		 * Unfortunately, to be able to detect duplicate OOB-proxied queries
@@ -1939,7 +1972,8 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 		 */
 
 		if (mangled)
-			message_add(mangled, sender->header.function, sender);
+			message_add(mangled,
+				gnutella_header_get_function(&sender->header), sender);
 
 		if (!route_it)
 			goto done;
@@ -1952,7 +1986,7 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 	 * Compute the route, determine if we should handle the message.
 	 */
 
-	switch (sender->header.function) {
+	switch (gnutella_header_get_function(&sender->header)) {
 	case GTA_MSG_PUSH_REQUEST:
 		handle_it = route_push(&log, node, dest, duplicate);
 		break;
@@ -1963,8 +1997,12 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 		handle_it = route_query_hit(&log, node, dest);
 		break;
 	default:
-		g_error("messages 0x%x \"%s\" should not be routed",
-			sender->header.function, debug_msg[sender->header.function]);
+		/*
+		 * Any other message that gets passed to route_message() must
+		 * neither be routed nor handled.
+		 */
+		handle_it = FALSE;
+		break;
 	}
 
 	routing_log_set_route(&log, dest, handle_it);
@@ -1972,9 +2010,15 @@ route_message(struct gnutella_node **node, struct route_dest *dest)
 done:
 	routing_log_flush(&log);
 
-	sender->header.hops++;				/* Mark passage through our node */
-	if (sender->header.ttl)
-		sender->header.ttl--;
+	/* Paranoid: avoid hop overflow */
+	if (gnutella_header_get_hops(&sender->header) < 255) {
+		/* Mark passage through our node */
+		gnutella_header_set_hops(&sender->header,
+			gnutella_header_get_hops(&sender->header) + 1);
+	}
+	if (gnutella_header_get_ttl(&sender->header) > 0)
+		gnutella_header_set_ttl(&sender->header,
+			gnutella_header_get_ttl(&sender->header) - 1);
 
 	return !duplicate && handle_it;		/* Don't handle duplicates */
 }
@@ -2008,7 +2052,7 @@ route_towards_guid(const gchar *guid)
 {
 	struct message *m;
 	GSList *nodes = NULL;
-	GSList *l;
+	GSList *sl;
 
 	if (g_hash_table_lookup(ht_banned_push, guid))	/* Banned for PUSH */
 		return NULL;
@@ -2018,8 +2062,8 @@ route_towards_guid(const gchar *guid)
 
 	revitalize_entry(m, TRUE);
 
-	for (l = m->routes; l; l = g_slist_next(l)) {
-		struct route_data *rd = (struct route_data *) l->data;
+	for (sl = m->routes; sl; sl = g_slist_next(sl)) {
+		struct route_data *rd = sl->data;
 		nodes = g_slist_prepend(nodes, rd->node);
 	}
 
@@ -2068,7 +2112,7 @@ route_proxy_add(gchar *guid, struct gnutella_node *n)
 struct gnutella_node *
 route_proxy_find(gchar *guid)
 {
-	return (struct gnutella_node *) g_hash_table_lookup(ht_proxyfied, guid);
+	return g_hash_table_lookup(ht_proxyfied, guid);
 }
 
 /**
@@ -2079,7 +2123,7 @@ free_banned_push(gpointer key, gpointer unused_value, gpointer unused_udata)
 {
 	(void) unused_value;
 	(void) unused_udata;
-	atom_guid_free((gchar *) key);
+	atom_guid_free(key);
 }
 
 /**
@@ -2123,4 +2167,4 @@ routing_close(void)
 	ht_proxyfied = NULL;
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
