@@ -1,5 +1,5 @@
 /*
- * $Id: rx_inflate.c,v 1.14 2005/12/17 09:25:44 daichik Exp $
+ * $Id: rx_inflate.c 11957 2006-09-13 23:27:04Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: rx_inflate.c,v 1.14 2005/12/17 09:25:44 daichik Exp $");
+RCSID("$Id: rx_inflate.c 11957 2006-09-13 23:27:04Z cbiere $")
 
 #include <zlib.h>
 
@@ -126,7 +126,7 @@ inflate_data(rxdrv_t *rx, pmsg_t *mb)
 	return pmsg_alloc(PMSG_P_DATA, db, 0, inflated);
 
 cleanup:
-	rxbuf_free(db, NULL);
+	rxbuf_free(db);
 	return NULL;
 }
 
@@ -150,8 +150,8 @@ rx_inflate_init(rxdrv_t *rx, gconstpointer args)
 
 	inz = walloc(sizeof(*inz));
 
-	inz->zalloc = NULL;
-	inz->zfree = NULL;
+	inz->zalloc = zlib_alloc_func;
+	inz->zfree = zlib_free_func;
 	inz->opaque = NULL;
 
 	ret = inflateInit(inz);
@@ -197,10 +197,11 @@ rx_inflate_destroy(rxdrv_t *rx)
 /**
  * Got data from lower layer.
  */
-static void
+static gboolean 
 rx_inflate_recv(rxdrv_t *rx, pmsg_t *mb)
 {
 	struct attr *attr = rx->opaque;
+	gboolean error = FALSE;
 	pmsg_t *imb;		/**< Inflated message */
 
 	g_assert(rx);
@@ -212,10 +213,14 @@ rx_inflate_recv(rxdrv_t *rx, pmsg_t *mb)
 	 * disabled, in which case we must stop.
 	 */
 
-	while ((attr->flags & IF_ENABLED) && (imb = inflate_data(rx, mb)))
-		(*rx->data_ind)(rx, imb);
+	while ((attr->flags & IF_ENABLED) && (imb = inflate_data(rx, mb))) {
+		error = !(*rx->data_ind)(rx, imb);
+		if (error)
+			break;
+	}
 
 	pmsg_free(mb);
+	return !error;
 }
 
 /**
