@@ -1,5 +1,5 @@
 /*
- * $Id: tsync.c,v 1.14 2005/06/25 01:37:41 daichik Exp $
+ * $Id: tsync.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tsync.c,v 1.14 2005/06/25 01:37:41 daichik Exp $");
+RCSID("$Id: tsync.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "nodes.h"
 #include "tsync.h"
@@ -80,7 +80,7 @@ tsync_free(struct tsync *ts)
 {
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
-	
+
 	if (ts->expire_ev)
 		cq_cancel(callout_queue, ts->expire_ev);
 
@@ -101,7 +101,7 @@ tsync_expire(cqueue_t *unused_cq, gpointer obj)
 
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
-	
+
 	if (dbg > 1)
 		printf("TSYNC expiring time %d.%d\n",
 			(gint) ts->sent.tv_sec, (gint) ts->sent.tv_usec);
@@ -140,7 +140,7 @@ tsync_send(struct gnutella_node *n, guint32 node_id)
 
 	ts = walloc(sizeof(*ts));
 	ts->magic = TSYNC_MAGIC;
-	tm_now(&ts->sent);
+	tm_now_exact(&ts->sent);
 	ts->sent.tv_sec = clock_loc2gmt(ts->sent.tv_sec);
 	ts->node_id = node_id;
 	ts->udp = NODE_IS_UDP(n);
@@ -190,7 +190,7 @@ tsync_send_timestamp(tm_t *orig, tm_t *final)
 
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
-	
+
 	g_hash_table_remove(tsync_by_time, orig);
 	ts->sent = *final;
 	g_hash_table_insert(tsync_by_time, &ts->sent, ts);
@@ -245,7 +245,7 @@ tsync_got_reply(struct gnutella_node *n,
 	if (dbg > 1)
 		printf("TSYNC RTT for %d.%d with %s via %s is: %.6f secs\n",
 			(gint) sent->tv_sec, (gint) sent->tv_usec,
-			node_ip(n), NODE_IS_UDP(n) ? "UDP" : "TCP", (double) rtt);
+			node_addr(n), NODE_IS_UDP(n) ? "UDP" : "TCP", (double) rtt);
 
 	/*
 	 * If request is too ancient, we'll just use it to measure the
@@ -265,7 +265,7 @@ tsync_got_reply(struct gnutella_node *n,
 
 		g_assert(ts);
 		g_assert(ts->magic == TSYNC_MAGIC);
-	
+
 		/*
 		 * Compute the clock offset: ((received - sent) + (replied - got)) / 2
 		 */
@@ -279,7 +279,7 @@ tsync_got_reply(struct gnutella_node *n,
 
 		if (dbg > 1)
 			printf("TSYNC offset between %s clock at %s and ours: %.6f secs\n",
-				ntp ? "NTP" : "regular", node_ip(n), (double) clock_offset);
+				ntp ? "NTP" : "regular", node_addr(n), (double) clock_offset);
 
 		/*
 		 * If the node is still connected (which we can't know easily if
@@ -307,7 +307,7 @@ tsync_got_reply(struct gnutella_node *n,
 		precision = ntp ? (NODE_IS_UDP(n) ? 0 : 1) : 2;
 		precision += (gint) (rtt * 0.5);
 
-		clock_update(got->tv_sec + (gint) clock_offset, precision,  n->ip);
+		clock_update(got->tv_sec + (gint) clock_offset, precision,  n->addr);
 
 		g_hash_table_remove(tsync_by_time, &ts->sent);
 		tsync_free(ts);
@@ -333,10 +333,10 @@ free_tsync_kv(gpointer unused_key, gpointer value, gpointer unused_udata)
 
 	(void) unused_key;
 	(void) unused_udata;
-	
+
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
-	
+
 	tsync_free(ts);
 }
 
