@@ -1,5 +1,5 @@
 /*
- * $Id: tx_deflate.c,v 1.31 2005/11/15 21:30:07 rmanfredi Exp $
+ * $Id: tx_deflate.c 11957 2006-09-13 23:27:04Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_deflate.c,v 1.31 2005/11/15 21:30:07 rmanfredi Exp $");
+RCSID("$Id: tx_deflate.c 11957 2006-09-13 23:27:04Z cbiere $")
 
 #include "sockets.h"
 #include "hosts.h"
@@ -693,8 +693,8 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 
 	outz = walloc(sizeof *outz);
 
-	outz->zalloc = NULL;
-	outz->zfree = NULL;
+	outz->zalloc = zlib_alloc_func;
+	outz->zfree = zlib_free_func;
 	outz->opaque = NULL;
 
 	ret = targs->gzip
@@ -726,7 +726,7 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	for (i = 0; i < BUFFER_COUNT; i++) {
 		struct buffer *b = &attr->buf[i];
 
-		b->arena = b->wptr = b->rptr = g_malloc(attr->buffer_size);
+		b->arena = b->wptr = b->rptr = walloc(attr->buffer_size);
 		b->end = &b->arena[attr->buffer_size];
 	}
 
@@ -779,7 +779,7 @@ tx_deflate_destroy(txdrv_t *tx)
 
 	for (i = 0; i < BUFFER_COUNT; i++) {
 		struct buffer *b = &attr->buf[i];
-		g_free(b->arena);
+		wfree(b->arena, attr->buffer_size);
 	}
 
 	/*
@@ -806,7 +806,7 @@ tx_deflate_destroy(txdrv_t *tx)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_deflate_write(txdrv_t *tx, gpointer data, size_t len)
+tx_deflate_write(txdrv_t *tx, gconstpointer data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 
@@ -979,24 +979,27 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 
 	tx_deflate_flush(tx);
 
+	if (attr->gzip.enabled && 0 == tx_deflate_pending(tx)) {
+		/* See RFC 1952 - GZIP file format specification version 4.3 */
+		struct buffer *b;
+		guint32 trailer[2]; /* 0: CRC32, 1: SIZE % (1 << 32) */
+
+		/* We don't want to send the trailer more than once */
+		attr->gzip.enabled = FALSE;
+
+		attr->send_idx = 0;
+		b = &attr->buf[attr->send_idx];
+		poke_le32(&trailer[0], (guint32) attr->gzip.crc);
+		poke_le32(&trailer[1], attr->gzip.size);
+
+		g_assert(sizeof trailer <= (size_t) (b->end - b->wptr));
+		memcpy(b->wptr, trailer, sizeof trailer);
+		b->wptr += sizeof trailer;
+
+		deflate_send(tx);
+	}
+
 	if (0 == tx_deflate_pending(tx)) {
-		if (attr->gzip.enabled) {
-			/* See RFC 1952 - GZIP file format specification version 4.3 */
-			struct buffer *b;
-			guint32 trailer[2]; /* 0: CRC32, 1: SIZE % (1 << 32) */
-
-			attr->send_idx = 0;
-			b = &attr->buf[attr->send_idx];
-			poke_le32(&trailer[0], (guint32) attr->gzip.crc);
-			poke_le32(&trailer[1], attr->gzip.size);
-
-			g_assert(sizeof trailer <= (size_t) (b->end - b->wptr));
-			memcpy(b->wptr, trailer, sizeof trailer);
-			b->wptr += sizeof trailer;
-
-			deflate_send(tx);
-		}
-
 		if (dbg > 9)
 			printf("tx_deflate_close: flushed everything immediately\n");
 
