@@ -1,5 +1,5 @@
 /*
- * $Id: uhc.c,v 1.41 2005/12/27 16:37:55 cbiere Exp $
+ * $Id: uhc.c 12171 2006-10-31 22:13:56Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: uhc.c,v 1.41 2005/12/27 16:37:55 cbiere Exp $");
+RCSID("$Id: uhc.c 12171 2006-10-31 22:13:56Z cbiere $")
 
 #include "gnutella.h"
 #include "uhc.h"
@@ -56,6 +56,7 @@ RCSID("$Id: uhc.c,v 1.41 2005/12/27 16:37:55 cbiere Exp $");
 
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
+#include "if/core/settings.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -92,22 +93,19 @@ struct used_uhc {
 static const struct {
 	const gchar *uhc;
 } boot_hosts[] = {
-	{ "cache.kicks-ass.net:8000" },
-	{ "g6.dns6.org:1337" },
-	{ "gwc.mine.nu:9999" },
-	{ "gwc2.mine.nu:9999" },
-	{ "kisama.ath.cx:8080" },
-	{ "krill.shacknet.nu:20095" },
+	{ "g6.6dns.org:1337" },
+	{ "guruz.udp-host-cache.com:6666" },
 	{ "plankton.merseine.nu:20098" },
-	{ "pokerface.bishopston.net:3558" },
 	{ "secondary.udp-host-cache.com:9999" },
 	{ "uhc.udp-host-cache.com:9999" },
 	{ "uhc2.limewire.com:20181" },
+	{ "zomfg.gtkg.org:40000" },
 };
 
 static gboolean uhc_connecting = FALSE;
 
-static void uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata);
+static void uhc_host_resolved(const host_addr_t *addr, size_t n,
+				gpointer uu_udata);
 
 /**
  * Parse hostname:port and return the hostname and port parts.
@@ -289,7 +287,8 @@ uhc_try_random(void)
 	 * we're protected by the `attempts' counter.
 	 */
 
-	(void) adns_resolve(uhc_ctx.host, uhc_host_resolved, NULL);
+	(void) adns_resolve(uhc_ctx.host, settings_dns_net(),
+				uhc_host_resolved, NULL);
 }
 
 /**
@@ -316,17 +315,19 @@ uhc_ping_timeout(cqueue_t *unused_cq, gpointer unused_obj)
 static void
 uhc_send_ping(const host_addr_t addr, guint16 port)
 {
-	struct gnutella_msg_init *m;
-	struct gnutella_node *n = node_udp_get_addr_port(addr, port);
-	guint32 size;
 	gchar *muid;
-	gchar msg[256];
 
 	g_assert(uhc_connecting);
 
-	m = build_ping_msg(NULL, 1, TRUE, &size);
-	muid = atom_guid_get(m->header.muid);
-	udp_send_msg(n, m, size);
+	{
+		struct gnutella_node *n = node_udp_get_addr_port(addr, port);
+		gnutella_msg_init_t *m;
+		guint32 size;
+		
+		m = build_ping_msg(NULL, 1, TRUE, &size);
+		muid = atom_guid_get(gnutella_header_get_muid(m));
+		udp_send_msg(n, m, size);
+	}
 
 	/*
 	 * Save the GUID of the ping we sent, to be able to determine when
@@ -345,10 +346,13 @@ uhc_send_ping(const host_addr_t addr, guint16 port)
 	/*
 	 * Give GUI feedback.
 	 */
+	{
+		gchar msg[256];
 
-	gm_snprintf(msg, sizeof msg, _("Sent ping to UDP host cache %s:%u"),
-		uhc_ctx.host, uhc_ctx.port);
-	gcu_statusbar_message(msg);
+		gm_snprintf(msg, sizeof msg, _("Sent ping to UDP host cache %s:%u"),
+			uhc_ctx.host, uhc_ctx.port);
+		gcu_statusbar_message(msg);
+	}
 
 	/*
 	 * Arm a timer to see whether we should not try to ping another
@@ -365,15 +369,16 @@ uhc_send_ping(const host_addr_t addr, guint16 port)
  * Callback for adns_resolve(), invoked when the resolution is complete.
  */
 static void
-uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata)
+uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
 {
 	(void) uu_udata;
+	g_assert(addrs);
 
 	/*
 	 * If resolution failed, try again if possible.
 	 */
 
-	if (!addr || !host_is_valid(*addr, uhc_ctx.port)) {
+	if (0 == n) {
 		if (gwc_debug)
 			g_warning("could not resolve UDP host cache \"%s\"",
 				uhc_ctx.host);
@@ -382,11 +387,12 @@ uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata)
 		return;
 	}
 
+	uhc_ctx.addr = addrs[random_raw() % n];
+	
 	if (gwc_debug || bootstrap_debug)
 		g_message("BOOT UDP host cache \"%s\" resolved to %s",
-			uhc_ctx.host, host_addr_to_string(*addr));
+			uhc_ctx.host, host_addr_to_string(uhc_ctx.addr));
 
-	uhc_ctx.addr = *addr;
 
 	/*
 	 * Now send the ping.
@@ -457,14 +463,14 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 	if (gwc_debug || bootstrap_debug)
 		g_message("extracting %d host%s in UDP IPP pong %s from %s (%s)",
 			cnt, cnt == 1 ? "" : "s",
-			guid_hex_str(n->header.muid), node_addr(n),
+			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n),
 			uhc_connecting ? "expected" : "unsollicited");
 
 	for (i = 0, p = payload; i < cnt; i++) {
 		host_addr_t ha;
 		guint16 port;
 
-		ha = host_addr_set_ipv4(peek_be32(p));
+		ha = host_addr_get_ipv4(peek_be32(p));
 		p += 4;
 		port = peek_le16(p);
 		p += 2;
@@ -484,7 +490,11 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 	 * check whether we're still in a probing cycle.
 	 */
 
-	if (uhc_connecting && g_hash_table_lookup(uhc_ctx.guids, n->header.muid)) {
+	if (
+		uhc_connecting &&
+		g_hash_table_lookup(uhc_ctx.guids,
+			gnutella_header_get_muid(&n->header))
+	) {
 		g_assert(uhc_ctx.timeout_ev != NULL);
 
 		if (gwc_debug || bootstrap_debug)
