@@ -1,7 +1,8 @@
 /*
- * $Id: upload_stats.c,v 1.8 2005/05/04 17:37:24 graaff Exp $
+ * $Id: upload_stats.c 12240 2006-11-06 18:08:15Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
+ * Copyright (c) 2002, Michael Tesch
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -29,59 +30,109 @@
  *
  * Keep track of which files we send away, and how often.
  *
- *		Statistics are kept by _FILENAME_ and file size,
- *		not by actual path, so two files with the same
- *		name and size will be counted in the same bin.
- *		I dont see this as a limitation because the
- *		user wouldn't be able to differentiate the files anyway.
- *		This could be extended to keep the entire path to
- *		each file and optionally show the entire path, but..
+ * Statistics are kept by _FILENAME_ and file size, not by actual path,
+ * so two files with the same name and size will be counted in the same
+ * bin. I don't see this as a limitation because the user wouldn't be able
+ * to differentiate the files anyway. This could be extended to keep the
+ * entire path to each file and optionally show the entire path, but..
  *
- *		the 'upload_history' file has the following format:
- *		<url-escaped filename> <file size> <attempts> <completions>
+ * The 'upload_history' file has the following format:
  *
- *		TODO: add a check to make sure that all of the files still exist(?)
- *			grey them out if they dont, optionally remove them from the
- *			stats list (when 'Clear Non-existent Files' is clicked)
+ *	- "<url-escaped filename> <file size> <attempts> <completions>"
+ *
+ * @todo
+ * TODO: Add a check to make sure that all of the files still exist(?)
+ *       grey them out if they dont, optionally remove them from the
+ *       stats list (when 'Clear Non-existent Files' is clicked).
  *
  * @author Michael Tesch
- * @version 1.6
  * @date 2002
- *
- * Released by Michael Tesch with gtk-gnutella & its license
+ * @author Raphael Manfredi
+ * @date 2001-2003
+ * @version 1.6
  */
 
 #include "common.h"
 
-RCSID("$Id: upload_stats.c,v 1.8 2005/05/04 17:37:24 graaff Exp $");
+RCSID("$Id: upload_stats.c 12240 2006-11-06 18:08:15Z cbiere $")
 
 #include "upload_stats.h"
 
 #include "if/bridge/c2ui.h"
 
+#include "lib/atoms.h"
 #include "lib/file.h"
+#include "lib/hashlist.h"
+#include "lib/tm.h"
 #include "lib/url.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 static gboolean dirty = FALSE;
-static gchar *stats_file = NULL;
-static GList *upload_stats_list = NULL;
+static gchar *stats_file;
+static hash_list_t *upload_stats_list;
+
+static gboolean 
+ul_stats_eq(gconstpointer p, gconstpointer q)
+{
+	const struct ul_stats *a = p, *b = q;
+  
+	/* filename is an atom */
+	return a->filename == b->filename && b->size == b->size;
+}
+
+static guint
+ul_stats_hash(gconstpointer p)
+{
+	const struct ul_stats *s = p;
+  
+	return ((size_t) s->filename >> 3) ^ s->size;
+}
+
+static struct ul_stats *
+upload_stats_find(const gchar *name, guint64 size)
+{
+	struct ul_stats *s = NULL;
+
+	if (upload_stats_list) {
+		static const struct ul_stats zero_stats;
+		struct ul_stats key;
+		gpointer orig_key;
+
+		key = zero_stats;
+		key.filename = atom_str_get(name);
+		key.size = size;
+
+		if (hash_list_contains(upload_stats_list, &key, &orig_key)) {
+			s = orig_key;
+		}
+		atom_str_free_null(&key.filename);
+	}
+	return s;
+}
 
 static void
 upload_stats_add(const gchar *filename,
 	filesize_t size, guint32 attempts, guint32 complete, guint64 ul_bytes)
 {
-	struct ul_stats *stat;
+	static const struct ul_stats zero_stats;
+	struct ul_stats *s;
 
-	stat = g_malloc0(sizeof(struct ul_stats));
-	stat->filename = g_strdup(filename);
-	stat->size = size;
-	stat->attempts = attempts;
-	stat->complete = complete;
-	stat->norm = size > 0 ? (gfloat) ul_bytes / (gfloat) size : 0;
-	stat->bytes_sent = ul_bytes;
-	upload_stats_list = g_list_append(upload_stats_list, stat);
-	gcu_upload_stats_gui_add(stat);
+	s = walloc(sizeof *s);
+	*s = zero_stats;
+	s->filename = atom_str_get(filename);
+	s->size = size;
+	s->attempts = attempts;
+	s->complete = complete;
+	s->norm = size > 0 ? (gfloat) ul_bytes / (gfloat) size : 0;
+	s->bytes_sent = ul_bytes;
+
+	if (!upload_stats_list) {
+		upload_stats_list = hash_list_new(ul_stats_hash, ul_stats_eq);
+	}
+	hash_list_append(upload_stats_list, s);
+	gcu_upload_stats_gui_add(s);
 }
 
 void
@@ -89,7 +140,7 @@ upload_stats_load_history(const gchar *ul_history_file_name)
 {
 	FILE *upload_stats_file;
 	gchar line[FILENAME_MAX + 64];
-	gint lineno = 0;
+	guint lineno = 0;
 
 	stats_file = g_strdup(ul_history_file_name);
 
@@ -149,7 +200,12 @@ upload_stats_load_history(const gchar *ul_history_file_name)
 		if (!url_unescape(line, TRUE))
 			goto corrupted;
 
-		upload_stats_add(line, size, attempt, complete, ulbytes);
+		if (upload_stats_find(line, size)) {
+			g_warning("upload_stats_load_history():"
+				" Ignoring line %u due to duplicate file.", lineno);
+		} else {
+			upload_stats_add(line, size, attempt, complete, ulbytes);
+		}
 
 		continue;
 
@@ -164,6 +220,26 @@ done:
 	return;
 }
 
+static void
+upload_stats_dump_item(gpointer p, gpointer user_data)
+{
+	FILE *out = user_data;
+	struct ul_stats *s = p;
+	gchar *escaped;
+
+	g_assert(NULL != s);
+
+	escaped = url_escape_cntrl(s->filename);
+	fprintf(out, "%s\t%s\t%u\t%u\t%lu\t%lu\n", escaped,
+		uint64_to_string(s->size), s->attempts, s->complete,
+			(gulong) (s->bytes_sent >> 32),
+			(gulong) (s->bytes_sent & 0xffffffff));
+
+	if (escaped != s->filename) {		/* File had escaped chars */
+		G_FREE_NULL(escaped);
+	}
+}
+
 /**
  * Save upload statistics to file.
  */
@@ -171,14 +247,15 @@ static void
 upload_stats_dump_history(const gchar *ul_history_file_name)
 {
 	FILE *out;
-	time_t now = time((time_t *) NULL);
-	GList *l;
+	time_t now = tm_time();
+
+	g_return_if_fail(ul_history_file_name);
 
 	/* open file for writing */
 	out = file_fopen(ul_history_file_name, "w");
-
-	if (NULL == out)
+	if (NULL == out) {
 		return;
+	}
 
 	fprintf(out,
 		"# THIS FILE IS AUTOMATICALLY GENERATED -- DO NOT EDIT\n"
@@ -194,21 +271,13 @@ upload_stats_dump_history(const gchar *ul_history_file_name)
 		"\n",
 		ctime(&now));
 
-	/* for each element in uploads_stats_list, write out to hist file */
-	for (l = g_list_first(upload_stats_list); NULL != l; l = g_list_next(l)) {
-		gchar *escaped;
-		struct ul_stats *stat;
-
-		stat = l->data;
-		g_assert(NULL != stat);
-		escaped = url_escape_cntrl(stat->filename);
-		fprintf(out, "%s\t%s\t%u\t%u\t%u\t%u\n", escaped,
-			uint64_to_string(stat->size), stat->attempts, stat->complete,
-				(guint32) (stat->bytes_sent >> 32),
-				(guint32) stat->bytes_sent);
-
-		if (escaped != stat->filename)		/* File had escaped chars */
-			G_FREE_NULL(escaped);
+	/*
+	 * Don't check this sooner so that the file is cleared, if the user
+	 * cleared the history.
+	 */
+	if (upload_stats_list) {
+		/* for each element in uploads_stats_list, write out to hist file */
+		hash_list_foreach(upload_stats_list, upload_stats_dump_item, out);
 	}
 
 	/* close file */
@@ -233,38 +302,22 @@ upload_stats_flush_if_dirty(void)
 		g_warning("can't save upload statistics: no file name recorded");
 }
 
-static struct ul_stats *
-upload_stats_find(const gchar *name, guint64 size)
-{
-    GList *l;
-
-	for (l = g_list_first(upload_stats_list); NULL != l; l = g_list_next(l)) {
-		struct ul_stats *s;
-
-		s = l->data;
-		if (size == s->size && 0 == strcmp(name, s->filename))
-			return s;
-	}
-
-	return NULL;
-}
-
 /**
- * Called when an upload starts
+ * Called when an upload starts.
  */
 void
 upload_stats_file_begin(const struct upload *u)
 {
-	struct ul_stats *stat;
+	struct ul_stats *s;
 
 	/* find this file in the ul_stats_clist */
-	stat = upload_stats_find(u->name, u->file_size);
+	s = upload_stats_find(u->name, u->file_size);
 
 	/* increment the attempted counter */
-	if (NULL == stat)
+	if (NULL == s)
 		upload_stats_add(u->name, u->file_size, 1, 0, 0);
 	else {
-		stat->attempts++;
+		s->attempts++;
 		gcu_upload_stats_gui_update(u->name, u->file_size);
 	}
 
@@ -282,21 +335,21 @@ static void
 upload_stats_file_add(const gchar *name, filesize_t size,
 	gint comp, guint64 sent)
 {
-	struct ul_stats *stat;
+	struct ul_stats *s;
 
 	g_assert(comp >= 0);
 
 	/* find this file in the ul_stats_clist */
-	stat = upload_stats_find(name, size);
+	s = upload_stats_find(name, size);
 
 	/* increment the completed counter */
-	if (NULL == stat) {
+	if (NULL == s) {
 		/* uh oh, row has since been deleted, add it: 1 attempt */
 		upload_stats_add(name, size, 1, comp, sent);
 	} else {
-		stat->bytes_sent += sent;
-		stat->norm = (gfloat) stat->bytes_sent / (gfloat) stat->size;
-		stat->complete += comp;
+		s->bytes_sent += sent;
+		s->norm = (gfloat) s->bytes_sent / (gfloat) s->size;
+		s->complete += comp;
 		gcu_upload_stats_gui_update(name, size);
 	}
 
@@ -316,7 +369,7 @@ upload_stats_file_aborted(const struct upload *u)
 }
 
 /**
- * Called when an upload completes
+ * Called when an upload completes.
  */
 void
 upload_stats_file_complete(const struct upload *u)
@@ -333,29 +386,36 @@ upload_stats_prune_nonexistent(void)
 }
 
 /**
- * Clear all the upload stats data structure
+ * Clear all the upload stats data structure.
  */
-static void upload_stats_free_all(void)
+static void
+upload_stats_free_all(void)
 {
-    GList *l;
+	if (upload_stats_list) {
+		struct ul_stats *s;
 
-	for (l = g_list_first(upload_stats_list); NULL != l; l = g_list_next(l)) {
-		G_FREE_NULL(((struct ul_stats *)(l->data))->filename);
-		G_FREE_NULL(l->data);
+		while (NULL != (s = hash_list_first(upload_stats_list))) {
+			hash_list_remove(upload_stats_list, s);
+			atom_str_free_null(&s->filename);
+			wfree(s, sizeof *s);
+		}
+		hash_list_free(upload_stats_list);
+		upload_stats_list = NULL;
 	}
-
-	g_list_free(upload_stats_list);
-	upload_stats_list = NULL;
 	dirty = TRUE;
 }
 
 /**
  * Like upload_stats_free_all() but also clears the GUI.
  */
-void upload_stats_clear_all(void)
+void
+upload_stats_clear_all(void)
 {
 	gcu_upload_stats_gui_clear_all();
 	upload_stats_free_all();
+	if (stats_file) {
+		upload_stats_dump_history(stats_file);
+	}
 }
 
 /**
