@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h,v 1.16 2005/12/01 20:23:23 rmanfredi Exp $
+ * $Id: uploads.h 12210 2006-11-04 01:28:01Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -57,7 +57,7 @@ typedef struct upload {
 	gpointer io_opaque;				/**< Opaque I/O callback information */
 	gpointer parq_opaque;			/**< Opaque parq information */
 
-	gint file_desc;
+	struct file_object *file;		/**< uploaded file */
 	bio_source_t *bio;				/**< Bandwidth-limited source */
 	sendfile_ctx_t sendfile_ctx;
 	struct special_ctx *special;	/**< For special ops like browsing */
@@ -67,9 +67,9 @@ typedef struct upload {
 	gint bsize;
 	gint buf_size;
 
-	guint index;
 	gchar *name;
 	filesize_t file_size;
+	guint file_index;
 
 	time_t start_date;
 	time_t last_update;
