@@ -1,5 +1,5 @@
 /*
- * $Id: whitelist.c,v 1.14 2005/09/10 08:17:29 daichik Exp $
+ * $Id: whitelist.c 12054 2006-09-26 19:54:47Z cbiere $
  *
  * Copyright (c) 2002, Vidar Madsen
  *
@@ -38,10 +38,11 @@
 
 #include "common.h"
 
-RCSID("$Id: whitelist.c,v 1.14 2005/09/10 08:17:29 daichik Exp $");
+RCSID("$Id: whitelist.c 12054 2006-09-26 19:54:47Z cbiere $")
 
 #include "whitelist.h"
 #include "settings.h"
+#include "tls_cache.h"
 #include "nodes.h"
 
 #include "lib/file.h"
@@ -56,6 +57,23 @@ static const gchar whitelist_file[] = "whitelist";
 static time_t whitelist_mtime, whitelist_checked;
 static gchar *whitelist_path = NULL;
 
+static guint
+addr_default_mask(const host_addr_t addr)
+{
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:
+		return 32;
+	case NET_TYPE_IPV6:
+		return 128;
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+	g_assert_not_reached();
+	return 0;
+}
+
+
 /**
  * Loads the whitelist into memory.
  */
@@ -83,11 +101,14 @@ whitelist_retrieve(void)
     whitelist_mtime = st.st_mtime;
 
     while (fgets(line, sizeof line, f)) {
-		const gchar *endptr;
+		GSList *sl_addr, *sl;
+		const gchar *endptr, *start;
 		host_addr_t addr;
     	guint16 port;
 		guint8 bits;
 		gchar *p;
+		gboolean item_ok;
+		gboolean use_tls;
 
         linenum++;
         if ('#' == line[0]) continue;
@@ -103,19 +124,29 @@ whitelist_retrieve(void)
         if ('\0' == line[0])
             continue;
 
+		sl_addr = NULL;
 		addr = zero_host_addr;
 		endptr = NULL;
 
-		if (!string_to_host_or_addr(line, &endptr, &addr)) {
+		endptr = is_strprefix(line, "tls:");
+		if (endptr) {
+			use_tls = TRUE;
+			start = endptr;
+		} else {
+			use_tls = FALSE;
+			start = line;
+		}
+
+		if (!string_to_host_or_addr(start, &endptr, &addr)) {
             g_warning("whitelist_retrieve(): "
 				"Line %d: Expected hostname or IP address \"%s\"",
 				linenum, line);
 			continue;
 		}
 
-        if (!is_host_addr(addr)) {
+		{
+			gchar *name;
 			guchar c = *endptr;
-			size_t len;
 
 			switch (c) {
 			case '\0':
@@ -134,22 +165,27 @@ whitelist_retrieve(void)
 				continue;
 			}
 
-			len = endptr - line;
-			line[len] = '\0'; /* Terminate the string for name_to_host_addr() */
-
+			/* Terminate the string for name_to_host_addr() */
+			name = g_strndup(start, endptr - start); 
+			
 			/* @todo TODO: This should use the ADNS resolver. */
-        	addr = name_to_host_addr(line);
-
-        	if (!is_host_addr(addr)) {
-            	g_warning("whitelist_retrieve(): "
-				"Line %d: Could not resolve hostname \"%s\"", linenum, line);
-            	continue;
-        	}
+       		sl_addr = name_to_host_addr(name, settings_dns_net());
+       		if (!sl_addr) {
+           		g_warning("whitelist_retrieve(): "
+					"Line %d: Could not resolve hostname \"%s\"",
+					linenum, line);
+			}
+			G_FREE_NULL(name);
+       		if (!sl_addr) {
+				continue;
+			}
 		}
 
-       	g_assert(is_host_addr(addr));
+       	g_assert(sl_addr);
 		g_assert(NULL != endptr);
-		port = bits = 0;
+		bits = 0;
+		port = 0;
+		item_ok = TRUE;
 
 		/* Ignore trailing items separated by a space */
 		while ('\0' != *endptr && !is_ascii_space(*endptr)) {
@@ -162,7 +198,7 @@ whitelist_retrieve(void)
 				if (0 != port) {
 					g_warning("whitelist_retrieve(): Line %d:"
 						"Multiple colons after host", linenum);
-					addr = zero_host_addr;
+					item_ok = FALSE;
 					break;
 				}
 
@@ -171,7 +207,7 @@ whitelist_retrieve(void)
 				if (0 == port) {
 					g_warning("whitelist_retrieve(): Line %d: "
 						"Invalid port value after host", linenum);
-					addr = zero_host_addr;
+					item_ok = FALSE;
 					break;
 				}
 			} else if ('/' == c) {
@@ -181,7 +217,7 @@ whitelist_retrieve(void)
 				if (0 != bits) {
 					g_warning("whitelist_retrieve(): Line %d:"
 						"Multiple slashes after host", linenum);
-					addr = zero_host_addr;
+					item_ok = FALSE;
 					break;
 				}
 
@@ -189,7 +225,7 @@ whitelist_retrieve(void)
 					if (NET_TYPE_IPV4 != host_addr_net(addr)) {
 						g_warning("whitelist_retrieve(): Line %d: "
 							"IPv4 netmask after non-IPv4 address", linenum);
-						addr = zero_host_addr;
+						item_ok = FALSE;
 						break;
 					}
 					endptr = ep;
@@ -197,7 +233,7 @@ whitelist_retrieve(void)
 					if (0 == (bits = netmask_to_cidr(mask))) {
 						g_warning("whitelist_retrieve(): Line %d: "
 							"IPv4 netmask after non-IPv4 address", linenum);
-						addr = zero_host_addr;
+						item_ok = FALSE;
 						break;
 					}
 
@@ -214,7 +250,7 @@ whitelist_retrieve(void)
 					) {
 						g_warning("whitelist_retrieve(): Line %d: "
 							"Invalid numeric netmask after host", linenum);
-						addr = zero_host_addr;
+						item_ok = FALSE;
 						break;
 					}
 					bits = v;
@@ -222,40 +258,35 @@ whitelist_retrieve(void)
 			} else {
 				g_warning("whitelist_retrieve(): Line %d: "
 					"Unexpected character after host", linenum);
-				addr = zero_host_addr;
+				item_ok = FALSE;
 				break;
 			}
 		}
 
-        if (!is_host_addr(addr)) {
-			continue;
-		}
+		if (item_ok) {
+			for (sl = sl_addr; NULL != sl; sl = g_slist_next(sl)) {
+				host_addr_t *addr_ptr = sl->data;
+				struct whitelist *item;
 
-		if (0 == bits)	{
-			/* Default mask */
-			switch (host_addr_net(addr)) {
-			case NET_TYPE_IPV4:
-        		bits = 32;
-				break;
-			case NET_TYPE_IPV6:
-        		bits = 128;
-				break;
-			case NET_TYPE_NONE:
-				break;
+				g_assert(addr_ptr);
+
+				item = walloc0(sizeof *item);
+				item->addr = *addr_ptr;
+				item->port = port;
+				item->bits = bits ? bits : addr_default_mask(item->addr);
+
+				if (
+					item->port > 0 &&
+					addr_default_mask(item->addr) == item->bits
+				) {
+					tls_cache_insert(item->addr, item->port);
+				}
+
+				sl_whitelist = g_slist_prepend(sl_whitelist, item);
 			}
-			g_assert(0 != bits);
 		}
 
-		{
-    		struct whitelist *item;
-
-        	item = walloc0(sizeof *item);
-			item->addr = addr;
-        	item->port = port;
-        	item->bits = bits;
-
-        	sl_whitelist = g_slist_prepend(sl_whitelist, item);
-		}
+		host_addr_free_list(&sl_addr);
     }
 
     sl_whitelist = g_slist_reverse(sl_whitelist);
