@@ -1,5 +1,5 @@
 /*
- * $Id: ui_core_interface.h,v 1.0
+ * $Id: c2ui.h 12200 2006-11-03 05:05:31Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -43,6 +43,8 @@
 #include "if/core/downloads.h"
 #include "if/core/uploads.h"
 #include "if/core/bitzi.h"
+
+#include "lib/host_addr.h"
 #include "lib/misc.h"
 
 /* Property table includes */
@@ -53,10 +55,11 @@
 
 /* download interface functions */
 void gcu_download_enable_start_now(guint32 running_downloads,
-	guint32 max_downloads);
+	guint32 max_num_downloads);
 void gcu_gui_update_download(struct download *d, gboolean force);
 void gcu_gui_update_download_server(struct download *d);
 void gcu_gui_update_download_range(struct download *d);
+void gcu_gui_update_download_size(struct download *d);
 void gcu_gui_update_download_host(struct download *d);
 void gcu_gui_update_download_abort_resume(void);
 void gcu_gui_update_download_clear(void);
