@@ -1,5 +1,5 @@
 /*
- * $Id: ui2c.h,v 1.28 2005/12/18 06:09:10 cbiere Exp $
+ * $Id: ui2c.h 12175 2006-11-02 00:43:15Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -59,14 +59,15 @@
 #include <glib.h>
 
 /* adns interface functions */
-gboolean guc_adns_resolve(
-	const gchar *hostname, adns_callback_t user_callback, gpointer user_data);
+gboolean guc_adns_resolve(const gchar *hostname,
+			adns_callback_t user_callback, gpointer user_data);
 
 /* download and src interface functions */
-const gchar *guc_build_url_from_download(const struct download *d);
+gchar *guc_download_build_url(const struct download *d);
+gchar *guc_file_info_build_magnet(gnet_fi_t handle);
 gint guc_download_get_http_req_percent(const struct download *d);
-void guc_download_fallback_to_push
-	(struct download *d, gboolean on_timeout, gboolean user_request);
+void guc_download_fallback_to_push(struct download *d, gboolean on_timeout,
+		gboolean user_request);
 gint guc_download_remove_all_from_peer(gchar *guid, const host_addr_t addr,
 	guint16 port, gboolean unavailable);
 gint guc_download_remove_all_named(const gchar *name);
@@ -82,28 +83,26 @@ void guc_download_abort(struct download *d);
 void guc_download_resume(struct download *d);
 void guc_download_freeze_queue(void);
 void guc_download_thaw_queue(void);
-gint guc_download_queue_is_frozen(void);
+gboolean guc_download_queue_is_frozen(void);
 void guc_download_clear_stopped(gboolean complete,
 	gboolean failed, gboolean unavailable, gboolean now);
-void guc_download_auto_new(gchar *file, filesize_t size,
+void guc_download_auto_new(const gchar *file, filesize_t size,
 	guint32 record_index, const host_addr_t addr, guint16 port,
 	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, gboolean file_size_known, fileinfo_t *fi,
+	gboolean file_size_known, fileinfo_t *fi,
 	gnet_host_vec_t *proxies, guint32 flags);
-gboolean guc_download_new(gchar *file, filesize_t size,
+gboolean guc_download_new(const gchar *file, filesize_t size,
 	guint32 record_index, const host_addr_t addr, guint16 port,
 	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, fileinfo_t *fi, gnet_host_vec_t *proxies,
-	guint32 flags);
-gboolean guc_download_new_uri(gchar *file, const gchar *uri, filesize_t size,
-	const host_addr_t addr, guint16 port,
+	fileinfo_t *fi, gnet_host_vec_t *proxies, guint32 flags);
+gboolean guc_download_new_uri(const gchar *file, const gchar *uri,
+	filesize_t size, const host_addr_t addr, guint16 port,
 	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, fileinfo_t *fi, gnet_host_vec_t *proxies,
-	guint32 flags);
-gboolean guc_download_new_unknown_size(gchar *file,
+	fileinfo_t *fi, gnet_host_vec_t *proxies, guint32 flags);
+gboolean guc_download_new_unknown_size(const gchar *file,
 	guint32 record_index, const host_addr_t addr, guint16 port,
 	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, fileinfo_t *fi, gnet_host_vec_t *proxies,
+	fileinfo_t *fi, gnet_host_vec_t *proxies,
 	guint32 flags);
 const gchar *guc_download_get_hostname(const struct download *d);
 const gchar *guc_download_get_country(const struct download *d);
@@ -180,17 +179,14 @@ void guc_node_remove_node_flags_changed_listener
 void guc_node_add(const host_addr_t addr, guint16 port, guint32 flags);
 void guc_node_remove_by_handle(gnet_node_t n);
 void guc_node_remove_nodes_by_handle(GSList *node_list);
-void guc_node_get_status
-	(const gnet_node_t n, gnet_node_status_t *s);
+void guc_node_get_status(const gnet_node_t n, gnet_node_status_t *s);
 gnet_node_info_t *guc_node_get_info(const gnet_node_t n);
 void guc_node_clear_info(gnet_node_info_t *info);
 void guc_node_free_info(gnet_node_info_t *info);
-void guc_node_fill_flags
-	(gnet_node_t n, gnet_node_flags_t *flags);
-void guc_node_fill_info
-	(const gnet_node_t n, gnet_node_info_t *info);
-void guc_node_udp_gui_show(void);
-void guc_node_udp_gui_remove(void);
+void guc_node_fill_flags(gnet_node_t n, gnet_node_flags_t *flags);
+void guc_node_fill_info(const gnet_node_t n, gnet_node_info_t *info);
+const gchar *guc_node_flags_to_string(const gnet_node_flags_t *flags);
+const gchar *guc_node_peermode_to_string(node_peer_t m);
 
 /* parq interface functions */
 gint guc_get_parq_dl_position(const struct download *d);
@@ -214,7 +210,8 @@ gnet_search_t guc_search_new(const gchar *query,
 	guint32 reissue_timeout, flag_t flags);
 gboolean guc_search_browse(gnet_search_t sh,
 	const gchar *hostname, host_addr_t addr, guint16 port,
-	const gchar *guid, gboolean push, const gnet_host_vec_t *proxies);
+	const gchar *guid, const gnet_host_vec_t *proxies, guint32 flags);
+gboolean guc_search_locally(gnet_search_t sh, const gchar *query);
 void guc_search_reissue(gnet_search_t sh);
 void guc_search_close(gnet_search_t sh);
 void guc_search_start(gnet_search_t sh);
@@ -222,6 +219,8 @@ void guc_search_stop(gnet_search_t sh);
 void guc_search_add_kept(gnet_search_t sh, guint32 kept);
 
 /* settings interface functions */
+guint16 guc_listen_port(void);
+host_addr_t guc_listen_addr(enum net_type net);
 const gchar *guc_settings_home_dir(void);
 const gchar *guc_settings_config_dir(void);
 
@@ -262,8 +261,8 @@ void guc_upload_stats_clear_all(void);
 const gchar *guc_version_get_version_string(void);
 
 /* bitzi interface functions*/
-void guc_query_bitzi_by_urn(const gchar *urnsha1);
-bitzi_data_t * guc_querycache_bitzi_by_urn(const gchar *urnsha1);
+void guc_query_bitzi_by_sha1(const gchar *sha1);
+bitzi_data_t *guc_query_cache_bitzi_by_sha1(const gchar *sha1);
 
 /** main functions */
 void guc_gtk_gnutella_exit(gint code);
