@@ -1,5 +1,5 @@
 /*
- * $Id: guid.h,v 1.2 2005/08/27 14:54:27 rmanfredi Exp $
+ * $Id: guid.h 11378 2006-07-27 22:33:55Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -30,6 +30,10 @@
 
 #include "lib/misc.h"		/* For GUID_RAW_SIZE */
 
+typedef struct guid {
+	guchar v[GUID_RAW_SIZE];
+} guid_t;
+
 extern const gchar blank_guid[GUID_RAW_SIZE];
 
 #endif /* _if_core_guid_h_ */
