@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h,v 1.1 2005/12/12 06:59:29 rmanfredi Exp $
+ * $Id: uploads.h 11727 2006-08-26 04:20:24Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,11 +36,6 @@
  * Public interface, visible from the bridge.
  */
 
-#ifdef GUI_SOURCES
-
-void uploads_gui_set_gnet_addr(gnet_upload_t u, host_addr_t addr, guint16 port);
-
-#endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_uploads_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
