@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.c,v 1.21 2005/12/27 16:29:29 cbiere Exp $
+ * $Id: atoms.c 11958 2006-09-13 23:30:37Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,31 +37,46 @@
  * @date 2002-2003
  */
 
-#include <string.h>
-
 #define ATOMS_SOURCE
 #include "common.h"
 
 #include "atoms.h"
 
-RCSID("$Id: atoms.c,v 1.21 2005/12/27 16:29:29 cbiere Exp $");
+RCSID("$Id: atoms.c 11958 2006-09-13 23:30:37Z cbiere $")
 
-#ifdef PROTECT_ATOMS
+#if 0
+#define PROTECT_ATOMS
+#endif
+
+#if 0
+#define ATOMS_HAVE_MAGIC
+#endif
 
 /*
  * With PROTECT_ATOMS all atoms are mapped read-only so that they cannot
  * be modified accidently. This is achieved through mprotect(). The CPU
  * overhead is significant and there is also some memory overhead but it's
  * sufficiently low for testing/debugging purposes.
+ *
+ * TODO: Separate the reference counter (refcount) from the atom itself.
+ *       This would heavily the reduce the amount of mprotect() calls and
+ *       also reduce the memory overhead.
+ *		 We could possibly use the value which we insert into the global
+ *       hashtable as reference counter.
  */
 
-#include <sys/mman.h>
-
+#ifdef ATOMS_HAVE_MAGIC
 typedef enum {
-	ATOM_PROT_MAGIC = 0x3eeb9a27U
+	ATOM_MAGIC = 0x3eeb9a27U
 } atom_prot_magic_t;
 
-#endif /* PROTECT_ATOMS */
+#define ATOM_MAGIC_CHECK(a) g_assert(ATOM_MAGIC == (a)->magic)
+#define ATOM_SET_MAGIC(a) G_STMT_START { (a)->magic = ATOM_MAGIC; } G_STMT_END
+#else
+#define ATOM_MAGIC_CHECK(a)
+#define ATOM_SET_MAGIC(a)
+#endif	/* ATOMS_HAVE_MAGIC */
+
 
 #include "misc.h"
 #include "glib-missing.h"
@@ -79,6 +94,8 @@ union mem_chunk {
   gdouble    d;
 };
 
+#define ATOM_TYPE_MASK	((size_t) 0x07)
+
 /**
  * Atoms are ref-counted.
  *
@@ -91,18 +108,29 @@ typedef struct atom {
 	GHashTable *get;		/**< Allocation spots */
 	GHashTable *free;		/**< Free spots */
 #endif
-#ifdef PROTECT_ATOMS
+#ifdef ATOMS_HAVE_MAGIC
 	atom_prot_magic_t magic;
-	size_t size;
 #endif /* PROTECT_ATOMS */
 	gint refcnt;			/**< Amount of references */
 	/* Ensure `arena' is properly aligned */
 	union mem_chunk arena[1];			/* Start of user arena */
 } atom_t;
 
-
 #define ARENA_OFFSET	G_STRUCT_OFFSET(struct atom, arena)
 
+static inline atom_t *
+atom_from_arena(gconstpointer key)
+{
+	return (gpointer) ((gchar *) key - ARENA_OFFSET);
+}
+
+static inline void
+atom_check(const atom_t *a)
+{
+	g_assert(a);
+	ATOM_MAGIC_CHECK(a);
+}
+
 #ifdef PROTECT_ATOMS
 struct mem_pool {
 	union mem_chunk chunks[1];
@@ -116,7 +144,7 @@ struct mem_cache {
 	size_t hold;
 };
 
-static struct mem_cache *mc[6];
+static struct mem_cache *mc[9];
 
 static struct mem_pool *
 mem_pool_new(size_t size, size_t hold)
@@ -131,7 +159,7 @@ mem_pool_new(size_t size, size_t hold)
   ps = compat_pagesize();
   len = hold * size;
   
-  mp = compat_page_align(len);
+  mp = alloc_pages(len);
   if (mp) {
     size_t i, step = size / sizeof mp->chunks[0];
     
@@ -166,8 +194,6 @@ mem_new(size_t size)
     mc->num_pools = 0;
     mc->avail = NULL;
     mc->pools = NULL;
-
-    g_message("mem_new(): hold=%d, size=%d", (int) mc->hold, size);
   }
   return mc;
 }
@@ -283,12 +309,10 @@ mem_unprotect(gpointer ptr, size_t size)
  * @note All atoms that reside in the same page will be affected.
  */
 static inline void
-atom_unprotect(atom_t *a)
+atom_unprotect(atom_t *a, size_t size)
 {
-	g_assert(a);
-	g_assert(ATOM_PROT_MAGIC == a->magic);
-
-	mem_unprotect(a, a->size);
+	atom_check(a);
+	mem_unprotect(a, size);
 }
 
 /**
@@ -296,12 +320,10 @@ atom_unprotect(atom_t *a)
  * @note All atoms that reside in the same page will be affected.
  */
 static inline void
-atom_protect(atom_t *a)
+atom_protect(atom_t *a, size_t size)
 {
-	g_assert(a);
-	g_assert(ATOM_PROT_MAGIC == a->magic);
-
-	mem_protect(a, a->size);
+	atom_check(a);
+	mem_protect(a, size);
 }
 
 /**
@@ -321,10 +343,9 @@ atom_alloc(size_t size)
 		a = mem_alloc(mc_ptr);
 		mem_unprotect(a, size);
 	} else {
-		a = compat_page_align(size);
+		a = alloc_pages(size);
 	}
-	a->magic = ATOM_PROT_MAGIC;
-	a->size = size;
+	ATOM_SET_MAGIC(a);
 
 	return a;
 }
@@ -336,17 +357,14 @@ atom_alloc(size_t size)
  * with free() and not further protected.
  */
 static void
-atom_dealloc(atom_t *a)
+atom_dealloc(atom_t *a, size_t size)
 {
 	struct mem_cache *mc_ptr;
 
-	g_assert(a);
-	g_assert(ATOM_PROT_MAGIC == a->magic);
+	atom_check(a);
 
-	mc_ptr = atom_get_mc(a->size);
+	mc_ptr = atom_get_mc(size);
 	if (mc_ptr) {
-		size_t size = a->size;
-
 		memset(a, 0, size);
 		mem_free(mc_ptr, a);
 		/*
@@ -359,19 +377,30 @@ atom_dealloc(atom_t *a)
 		 * relies on executable heap-memory because the page is now
 		 * mapped (PROT_READ | PROT_WRITE).
 		 */
-		compat_page_free(a, a->size);
+		FREE_PAGES_NULL(a, size);
 	}
 }
 #else
 
-#define atom_protect(a)
-#define atom_unprotect(a)
-#define atom_alloc(size) g_malloc(size)
-#define atom_dealloc(a) g_free(a)
+#define atom_protect(a, size)
+#define atom_unprotect(a, size)
+
+static inline atom_t *
+atom_alloc(size_t size)
+{
+	atom_t *a = walloc(size);
+	return a;	
+}
+
+static inline void
+atom_dealloc(atom_t *a, size_t size)
+{
+	wfree(a, size);
+}
 
 #endif /* PROTECT_ATOMS */
 
-typedef gint (*len_func_t)(gconstpointer v);
+typedef size_t (*len_func_t)(gconstpointer v);
 typedef const gchar *(*str_func_t)(gconstpointer v);
 
 /**
@@ -386,14 +415,16 @@ typedef struct table_desc {
 	str_func_t str_func;		/**< Atom to human-readable string */
 } table_desc_t;
 
-static gint str_len(gconstpointer v);
+static size_t str_len(gconstpointer v);
 static const gchar *str_str(gconstpointer v);
-static gint guid_len(gconstpointer v);
+static size_t guid_len(gconstpointer v);
 static const gchar *guid_str(gconstpointer v);
-static gint sha1_len(gconstpointer v);
+static size_t sha1_len(gconstpointer v);
 static const gchar *sha1_str(gconstpointer v);
-static gint uint64_len(gconstpointer v);
+static size_t uint64_len(gconstpointer v);
 static const gchar *uint64_str(gconstpointer v);
+static size_t filesize_len(gconstpointer v);
+static const gchar *filesize_str(gconstpointer v);
 
 /**
  * The set of all atom types we know about.
@@ -403,12 +434,16 @@ static table_desc_t atoms[] = {
 	{ "GUID",	NULL, guid_hash,   guid_eq,	    guid_len,   guid_str },	/* 1 */
 	{ "SHA1",	NULL, sha1_hash,   sha1_eq,	    sha1_len,   sha1_str },	/* 2 */
 	{ "uint64",	NULL, uint64_hash, uint64_eq,   uint64_len, uint64_str},/* 3 */
+	{ "filesize",
+		NULL, filesize_hash, filesize_eq, filesize_len, filesize_str},  /* 4 */
 };
 
+static GHashTable *ht_all_atoms;
+
 /**
  * @return length of string + trailing NUL.
  */
-static gint
+static size_t 
 str_len(gconstpointer v)
 {
 	return strlen((const gchar *) v) + 1;
@@ -486,7 +521,7 @@ guid_eq(gconstpointer a, gconstpointer b)
 /**
  * @return length of GUID.
  */
-static gint
+static size_t 
 guid_len(gconstpointer unused_guid)
 {
 	(void) unused_guid;
@@ -544,7 +579,7 @@ sha1_eq(gconstpointer a, gconstpointer b)
 /**
  * @return length of SHA1.
  */
-static gint
+static size_t 
 sha1_len(gconstpointer unused_sha1)
 {
 	(void) unused_sha1;
@@ -561,9 +596,9 @@ sha1_str(gconstpointer sha1)
 }
 
 /**
- * @return length of an int.
+ * @return length of a 64-bit integer.
  */
-static int
+static size_t
 uint64_len(gconstpointer unused_v)
 {
 	(void) unused_v;
@@ -571,7 +606,17 @@ uint64_len(gconstpointer unused_v)
 }
 
 /**
- * @return printable form of an int, as pointer to static data.
+ * @return length of a filesize_t.
+ */
+static size_t
+filesize_len(gconstpointer unused_v)
+{
+	(void) unused_v;
+	return sizeof(filesize_t);
+}
+
+/**
+ * @return printable form of a 64-bit integer, as pointer to static data.
  */
 static const gchar *
 uint64_str(gconstpointer v)
@@ -605,6 +650,42 @@ uint64_hash(gconstpointer p)
 	return v ^ (v >> 32);
 }
 
+/**
+ * @return printable form of a filesize_t, as pointer to static data.
+ */
+static const gchar *
+filesize_str(gconstpointer v)
+{
+	static gchar buf[UINT64_DEC_BUFLEN];
+
+	uint64_to_string_buf(*(const filesize_t *) v, buf, sizeof buf);
+	return buf;
+}
+
+/**
+ * Test two filesize_t for equality.
+ *
+ * @return whether both referenced 64-bit integers are equal.
+ */
+gint
+filesize_eq(gconstpointer a, gconstpointer b)
+{
+	return *(const filesize_t *) a == *(const filesize_t *) b;
+}
+
+/**
+ * Calculate the 32-bit hash of a filesize_t.
+ *
+ * @return the 32-bit hash value for the referenced 64-bit integer.
+ */
+guint
+filesize_hash(gconstpointer p)
+{
+	guint64 v = *(const filesize_t *) p;
+	return v ^ (v >> 32);
+}
+
+
 /**
  * Initialize atom structures.
  */
@@ -613,9 +694,12 @@ atoms_init(void)
 {
 	guint i;
 
+	STATIC_ASSERT(NUM_ATOM_TYPES <= (ATOM_TYPE_MASK + 1));
+	STATIC_ASSERT(NUM_ATOM_TYPES == G_N_ELEMENTS(atoms));
+
 #ifdef PROTECT_ATOMS
 	for (i = 0; i < G_N_ELEMENTS(mc); i++) {
-		mc[i] = mem_new(64 << i);
+		mc[i] = mem_new((2 * sizeof (union mem_chunk)) << i);
 	}
 #endif /* PROTECT_ATOMS */
 
@@ -624,6 +708,29 @@ atoms_init(void)
 
 		td->table = g_hash_table_new(td->hash_func, td->eq_func);
 	}
+	ht_all_atoms = g_hash_table_new(NULL, NULL);
+}
+
+static inline size_t
+atom_is_registered(enum atom_type type, gconstpointer key)
+{
+	gpointer value;
+
+	if (g_hash_table_lookup_extended(ht_all_atoms, key, NULL, &value)) {
+		/* If the address is already registered in the global atom table,
+		 * this is definitely an atom. However, the same memory object
+		 * could be shared by atoms of different types (in theory at least),
+		 * thus we must check whether the types are identical. Otherwise,
+		 * a new atom must be created.
+		 */
+
+		if (((size_t) value & ATOM_TYPE_MASK) == (guint) type) {
+			size_t size = (size_t) value & ~ATOM_TYPE_MASK;
+			g_assert(size >= ARENA_OFFSET);
+			return size;
+		}
+	}
+	return 0;
 }
 
 /**
@@ -633,14 +740,11 @@ atoms_init(void)
  * @return the atom's value.
  */
 gpointer
-atom_get(gint type, gconstpointer key)
+atom_get(enum atom_type type, gconstpointer key)
 {
 	table_desc_t *td;
-	gboolean found;
-	gpointer value;
-	gpointer x;
-	atom_t *a;
-	gint len;
+	gpointer orig_key;
+	size_t size;
 
 #ifdef __GNUC__
 	STATIC_ASSERT(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
@@ -650,45 +754,72 @@ atom_get(gint type, gconstpointer key)
 #endif /* __GNUC__ */
 	
     g_assert(key != NULL);
-	g_assert(type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
 
 	td = &atoms[type];		/* Where atoms of this type are held */
 
+	/* If the address is already registered in the global atom table,
+	 * this is definitely an atom. However, the same memory object
+	 * could be shared by atoms of different types (in theory at least),
+	 * thus we must check whether the types are identical. Otherwise,
+	 * a new atom must be created.
+	 */
+
+	size = atom_is_registered(type, key);
+	if (size > 0) {
+		orig_key = deconstify_gpointer(key);
+	} else {
+		gpointer value;
+		if (g_hash_table_lookup_extended(td->table, key, &orig_key, &value)) {
+			size = (size_t) value;
+			g_assert(size >= ARENA_OFFSET);
+		} else {
+			size = 0;
+		}
+	}
+
 	/*
 	 * If atom exists, increment ref count and return it.
 	 */
 
-	found = g_hash_table_lookup_extended(td->table, key, &value, &x);
-
-	if (found) {
-		a = (atom_t *) ((gchar *) value - ARENA_OFFSET);
+	if (size > 0) {
+		atom_t *a;
 
+		a = atom_from_arena(orig_key);
 		g_assert(a->refcnt > 0);
 
-		atom_unprotect(a);
+		atom_unprotect(a, size);
 		a->refcnt++;
-		atom_protect(a);
-		return value;
+		atom_protect(a, size);
+		return orig_key;
+	} else {
+		size_t len;
+		gpointer value;
+		atom_t *a;
+
+		/*
+		 * Create new atom.
+		 */
+
+		len = (*td->len_func)(key);
+		g_assert(len < ((size_t) -1) - ARENA_OFFSET);
+		size = round_size((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
+
+		a = atom_alloc(size);
+		a->refcnt = 1;
+		memcpy(a->arena, key, len);
+		atom_protect(a, size);
+
+		/*
+		 * Insert atom in table.
+		 */
+
+		value = (gpointer) (size | (guint) type);
+		g_hash_table_insert(ht_all_atoms, a->arena, value);
+		g_hash_table_insert(td->table, a->arena, (gpointer) size);
+
+		return a->arena;
 	}
-
-	/*
-	 * Create new atom.
-	 */
-
-	len = (*td->len_func)(key);
-
-	a = atom_alloc(ARENA_OFFSET + len);
-	a->refcnt = 1;
-	memcpy(a->arena, key, len);
-	atom_protect(a);
-
-	/*
-	 * Insert atom in table.
-	 */
-
-	g_hash_table_insert(td->table, a->arena, GINT_TO_POINTER(1));
-
-	return a->arena;
 }
 
 /**
@@ -696,38 +827,34 @@ atom_get(gint type, gconstpointer key)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free(gint type, gconstpointer key)
+atom_free(enum atom_type type, gconstpointer key)
 {
-	table_desc_t *td;
-	gboolean found;
-	gpointer value;
-	gpointer x;
+	size_t size;
 	atom_t *a;
 
     g_assert(key != NULL);
-	g_assert(type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
 
-	td = &atoms[type];		/* Where atoms of this type are held */
-
-	found = g_hash_table_lookup_extended(td->table, key, &value, &x);
-
-	g_assert(found);
-	g_assert(value == key);
-
-	a = (atom_t *) ((gchar *) key - ARENA_OFFSET);
+	size = atom_is_registered(type, key);
+	g_assert(size > 0);
 
+	a = atom_from_arena(key);
 	g_assert(a->refcnt > 0);
 
 	/*
 	 * Dispose of atom when its reference count reaches 0.
 	 */
 
-	atom_unprotect(a);
+	atom_unprotect(a, size);
 	if (--a->refcnt == 0) {
+		table_desc_t *td;
+
+		td = &atoms[type];		/* Where atoms of this type are held */
 		g_hash_table_remove(td->table, key);
-		atom_dealloc(a);
+		g_hash_table_remove(ht_all_atoms, key);
+		atom_dealloc(a, size);
 	} else {
-		atom_protect(a);
+		atom_protect(a, size);
 	}
 }
 
@@ -744,7 +871,7 @@ struct spot {
  * @returns the atom's value.
  */
 gpointer
-atom_get_track(gint type, gconstpointer key, gchar *file, gint line)
+atom_get_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
 {
 	gpointer atom;
 	atom_t *a;
@@ -754,7 +881,7 @@ atom_get_track(gint type, gconstpointer key, gchar *file, gint line)
 	struct spot *sp;
 
 	atom = atom_get(type, key);
-	a = (atom_t *) ((gchar *) atom - ARENA_OFFSET);
+	a = atom_from_arena(atom);
 
 	/*
 	 * Initialize tracking tables on first allocation.
@@ -808,7 +935,7 @@ destroy_tracking_table(GHashTable *h)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free_track(gint type, gconstpointer key, gchar *file, gint line)
+atom_free_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
 {
 	atom_t *a;
 	gchar buf[512];
@@ -816,7 +943,7 @@ atom_free_track(gint type, gconstpointer key, gchar *file, gint line)
 	gpointer v;
 	struct spot *sp;
 
-	a = (atom_t *) ((gchar *) key - ARENA_OFFSET);
+	a = atom_from_arena(key);
 
 	/*
 	 * If we're going to free the atom, dispose the tracking tables.
@@ -860,9 +987,9 @@ dump_tracking_entry(gpointer key, gpointer value, gpointer user)
 static void
 dump_tracking_table(gpointer atom, GHashTable *h, gchar *what)
 {
-	gint count = g_hash_table_size(h);
+	guint count = g_hash_table_size(h);
 
-	g_warning("all %d %s spot%s for 0x%lx:",
+	g_warning("all %u %s spot%s for 0x%lx:",
 		count, what, count == 1 ? "" : "s", (gulong) atom);
 
 	g_hash_table_foreach(h, dump_tracking_entry, what);
@@ -876,8 +1003,8 @@ dump_tracking_table(gpointer atom, GHashTable *h, gchar *what)
 static gboolean
 atom_warn_free(gpointer key, gpointer unused_value, gpointer udata)
 {
-	atom_t *a = (atom_t *) ((gchar *) key - ARENA_OFFSET);
-	table_desc_t *td = (table_desc_t *) udata;
+	atom_t *a = atom_from_arena(key);
+	table_desc_t *td = udata;
 
 	(void) unused_value;
 
@@ -914,6 +1041,7 @@ atoms_close(void)
 		g_hash_table_foreach_remove(td->table, atom_warn_free, td);
 		g_hash_table_destroy(td->table);
 	}
+	g_hash_table_destroy(ht_all_atoms);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
