@@ -1,5 +1,5 @@
 /*
- * $Id: endian.h,v 1.5 2005/09/10 10:10:36 daichik Exp $
+ * $Id: endian.h 11154 2006-06-24 07:05:31Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -173,6 +173,20 @@ poke_be32(gpointer p, guint32 v)
 	return &q[4];
 }
 
+static inline gpointer
+poke_be64(gpointer p, guint64 v)
+{
+	guint8 *q = p;
+	guint i;
+
+	for (i = 0; i < 8; i++) {
+		q[i ^ 7] = v;
+		v >>= 8;
+	}
+
+	return &q[8];
+}
+
 static inline gpointer
 poke_le16(gpointer p, guint16 v)
 {
@@ -197,5 +211,19 @@ poke_le32(gpointer p, guint32 v)
 	return &q[4];
 }
 
+static inline gpointer
+poke_le64(gpointer p, guint64 v)
+{
+	guint8 *q = p;
+	guint i;
+
+	for (i = 0; i < 8; i++) {
+		q[i] = v;
+		v >>= 8;
+	}
+
+	return &q[8];
+}
+
 #endif /* _endian_h_ */
 /* vi: set ts=4 sw=4 cindent: */
