@@ -1,10 +1,8 @@
 /*
- * $Id: eval.c,v 1.5 2005/01/17 18:54:58 cbiere Exp $
+ * $Id: eval.c 12241 2006-11-06 20:00:57Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * String evaluation.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,26 +23,39 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * String evaluation.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: eval.c,v 1.5 2005/01/17 18:54:58 cbiere Exp $");
+RCSID("$Id: eval.c 12241 2006-11-06 20:00:57Z cbiere $")
 
+#ifdef I_PWD
 #include <pwd.h>
+#endif
 
 #include "eval.h"
 #include "atoms.h"
 #include "misc.h"			/* For g_strlcpy() */
 #include "override.h"		/* Must be the last header included */
 
-#define MAX_STRING	1024	/* Max length for substitution */
+#define MAX_STRING	1024	/**< Max length for substitution */
 
-static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
+static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
 
 static GHashTable *constants;
-static gchar *home;
+static const gchar *home;	/* string atom */
 
-static gchar *get_home(void);
-static gchar *get_variable(gchar *s, gchar **end);
+static const gchar *get_home(void);
+static const gchar *get_variable(const gchar *s, const gchar **end);
+static gboolean initialized;
 
 /**
  * Create a constant string, or reuse an existing one if possible.
@@ -56,7 +67,7 @@ constant_make(gchar *s)
 {
 	gchar *v;
 
-	v = (gchar *) g_hash_table_lookup(constants, s);
+	v = g_hash_table_lookup(constants, s);
 	if (v != NULL)
 		return v;			/* Already exists */
 
@@ -72,8 +83,13 @@ constant_make(gchar *s)
 void
 eval_init(void)
 {
+	g_return_if_fail(!initialized);
+
 	constants = g_hash_table_new(g_str_hash, g_str_equal);
 	home = get_home();
+	g_assert(home);
+
+	initialized = TRUE;
 }
 
 static void
@@ -91,8 +107,10 @@ constants_free_kv(gpointer key,
 void
 eval_close(void)
 {
-	atom_str_free(home);
-
+	if (home) {
+		atom_str_free(home);
+		home = NULL;
+	}
 	g_hash_table_foreach(constants, constants_free_kv, NULL);
 	g_hash_table_destroy(constants);
 }
@@ -107,30 +125,32 @@ eval_close(void)
  * @return the pointer right after the inserted value.
  */
 static gchar *
-insert_value(gchar *val, gchar *start, gint off,
-	gint len, gint maxlen)
+insert_value(const gchar *val, gchar *start, size_t off,
+	size_t len, size_t maxlen)
 {
-	gint vlen = strlen(val);
+	size_t vlen = strlen(val);
 
 	g_assert(len <= maxlen);
-	g_assert(off >= 0 && off <= len);
+	g_assert(off <= len);
 
-	if (len + vlen > maxlen) {
+	if (vlen > maxlen - len) {
 		g_warning("ignoring variable substitution text \"%s\"", val);
 		return start;
 	}
 
-	memmove(start + vlen, start, len + 1 - off);
+	memmove(&start[vlen], start, len + 1 - off);
 	memmove(start, val, vlen);
 
-	return start + vlen;
+	return &start[vlen];
 }
 
 /**
+ * Needs brief description here.
+ *
  * Substitutes variables from string:
  *
- * . The leading "~" is replaced by the home directory.
- * . Variables like "$PATH" or "${PATH}" are replaced by their value, as
+ * - The leading "~" is replaced by the home directory.
+ * - Variables like "$PATH" or "${PATH}" are replaced by their value, as
  *   fetched from the environment, or the empty string if not found.
  *
  * If given a NULL input, we return NULL.
@@ -141,16 +161,18 @@ gchar *
 eval_subst(const gchar *str)
 {
 	gchar buf[MAX_STRING];
+	gchar *end = &buf[sizeof(buf)];
 	gchar *p;
-	gchar *end = buf + sizeof(buf);
 	size_t len;
 	gchar c;
 
+	g_assert(initialized);
+
 	if (str == NULL)
 		return NULL;
 
-	len = g_strlcpy(buf, str, sizeof(buf));
-	if (len >= sizeof(buf)) {
+	len = g_strlcpy(buf, str, sizeof buf);
+	if (len >= sizeof buf) {
 		g_warning("eval_subst: string too large for substitution (%d bytes)",
 			(int) len);
 		return constant_make((gchar *) str);
@@ -160,40 +182,47 @@ eval_subst(const gchar *str)
 	if (common_dbg > 3)
 		printf("eval_subst: on entry: \"%s\"\n", buf);
 
-	for (p =  buf, c = *p++; c; c = *p++) {
-		gchar *val = NULL;
+	for (p = buf, c = *p++; c; c = *p++) {
+		const gchar *val = NULL;
 		gchar *start = p - 1;
 
-		if (c == '~' && start == buf) {		/* Leading ~ only */
-			val = home;
-			memmove(start, start + 1, len - (start - buf));
-			len--;
+		switch (c) {
+		case '~':
+			if (start == buf) {		/* Leading ~ only */
+				val = home;
+				g_assert(val);
+				memmove(start, &start[1], len - (start - buf));
+				len--;
 
-			g_assert((ssize_t) len >= 0);
+				g_assert((ssize_t) len >= 0);
+			}
+			break;
+		case '$':
+			{
+				const gchar *after;
 
-		} else if (c == '$') {
-			gchar *after;
+				val = get_variable(p, &after);
+				g_assert(val);
+				memmove(start, after, len + 1 - (after - buf));
+				len -= after - start;		/* Also removing leading '$' */
 
-			val = get_variable(p, &after);
-			memmove(start, after, len + 1 - (after - buf));
-			len -= after - start;		/* Also removing leading '$' */
-
-			g_assert((ssize_t) len >= 0);
+				g_assert((ssize_t) len >= 0);
+			}
+			break;
 		}
 
-
 		if (val != NULL) {
-			gchar *next = insert_value(
-				val, start, start - buf, len, sizeof(buf) - 1);
-
+			gchar *next;
+			
+			next = insert_value(val, start, start - buf, len, sizeof buf - 1);
 			len += next - start;
 			p = next;
 
-			g_assert(len < sizeof(buf));
+			g_assert(len < sizeof buf);
 			g_assert(p < end);
 		}
 
-		g_assert(p <= (buf + len));
+		g_assert(p <= &buf[len]);
 	}
 
 	if (common_dbg > 3)
@@ -210,22 +239,32 @@ eval_subst(const gchar *str)
  *
  * @return string atom.
  */
-static gchar *
+static const gchar *
 get_home(void)
 {
-	gchar *v;
-	struct passwd *pp;
+	const char *dir;
 
-	v = getenv("HOME");
-	if (v != NULL)
-		return atom_str_get(v);
+	dir = getenv("HOME");
+	
+#if defined(HAS_GETUID)
+	if (!dir) {
+		static struct passwd *pp;
+		
+		pp = getpwuid(getuid());
+		if (pp)
+			dir = pp->pw_dir;
+	}
+#endif /* HAS_GETUID */
 
-	pp = getpwuid(getuid());
-	if (pp != NULL)
-		return atom_str_get(pp->pw_dir);
+	if (!dir)
+		dir = g_get_home_dir();
 
-	g_warning("Could not determine home directory");
-	return atom_str_get("/");
+	if (!dir) {
+		g_warning("Could not determine home directory");
+		dir = "/";
+	}
+
+	return atom_str_get(dir);
 }
 
 /**
@@ -234,12 +273,10 @@ get_home(void)
  * @return variable's value, or "" if not found and set `end' to the address
  * of the character right after the variable name.
  */
-static gchar *get_variable(gchar *s, gchar **end)
+static const gchar *
+get_variable(const gchar *s, const gchar **end)
 {
-	guchar *p = (guchar *) s;
-	guchar c;
-	gchar *name = s;
-	gchar *value;
+	const gchar *value, *p = s;
 	gboolean end_brace = FALSE;
 
 	/*
@@ -248,36 +285,37 @@ static gchar *get_variable(gchar *s, gchar **end)
 
 	if (*p == '{') {
 		p++;
-		name++;
+		s++;
 		end_brace = TRUE;
 	}
 
-	while ((c = *p)) {
-		if (!isalnum(c) && c != '_')
-			break;
+	while (is_ascii_alnum(*p) || *p == '_') {
 		p++;
 	}
 
 	if (end_brace && *p == '}')
-		*end = (gchar *) (p + 1);
+		*end = &p[1];
 	else
-		*end = (gchar *) p;
+		*end = p;
 
 	/*
 	 * Get value from environment.
 	 */
 
-	c = *p;
-	*p = '\0';
-	value = getenv(name);
+	{
+		gchar *name;
 
-	if (value == NULL)
-		value = "";
+		name = g_strndup(s, p - s);
+		value = getenv(name);
 
-	if (common_dbg > 4)
-		printf("variable \"%s\" is \"%s\"\n", name, value);
+		if (value == NULL)
+			value = "";
 
-	*p = c;
+		if (common_dbg > 4)
+			printf("variable \"%s\" is \"%s\"\n", name, value);
+
+		G_FREE_NULL(name);
+	}
 
 	return value;
 }
