@@ -1,5 +1,5 @@
 /*
- * $Id: file.c,v 1.6 2005/05/04 01:29:24 cbiere Exp $
+ * $Id: file.c 12272 2006-11-09 06:14:06Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -24,25 +24,32 @@
  */
 
 /**
+ * @ingroup lib
  * @file
  *
  * Miscellaneous common file routines.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: file.c,v 1.6 2005/05/04 01:29:24 cbiere Exp $");
+RCSID("$Id: file.c 12272 2006-11-09 06:14:06Z cbiere $")
 
 #include "file.h"
 #include "misc.h"
+#include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
+static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
+
 static const gchar orig_ext[] = "orig";
 static const gchar new_ext[] = "new";
 static const gchar instead_str[] = " instead";
 static const gchar empty_str[] = "";
 
-/*
+/**
  * In order to avoid having a dependency between file.c and ban.c,
  * we have ban.c register a callback to reclaim file descriptors
  * at init time.
@@ -66,6 +73,7 @@ file_register_fd_reclaimer(reclaim_fd_t callback)
  * if already present and `renaming' is TRUE.
  * If not found, try with successive alternatives, if supplied.
  *
+ * @attention
  * NB: the supplied `fv' argument is a vector of `fvcnt' elements.
  *
  * @param what is what is being opened, for logging purposes.
@@ -93,17 +101,12 @@ open_read(
 	g_assert(fvcnt >= 1);
 
 	path = make_pathname(fv->dir, fv->name);
-	g_return_val_if_fail(NULL != path, NULL);
 	if (!is_absolute_path(path)) {
 		G_FREE_NULL(path);
 		return NULL;
 	}
 
 	path_orig = g_strdup_printf("%s.%s", path, orig_ext);
-	if (NULL == path_orig)
-        G_FREE_NULL(path);
-    g_return_val_if_fail(NULL != path_orig, NULL);
-
 	in = fopen(path, "r");
 	if (in) {
 		if (renaming && -1 == rename(path, path_orig))
@@ -116,8 +119,8 @@ open_read(
 			g_warning("[%s] failed to retrieve from \"%s\": %s", what, path,
 				g_strerror(errno));
 		}
-        if (fvcnt > 1)
-            g_warning("[%s] trying to load from alternate locations...", what);
+        if (fvcnt > 1 && common_dbg > 0)
+            g_message("[%s] trying to load from alternate locations...", what);
     }
 
 	/*
@@ -158,21 +161,21 @@ open_read(
 		}
 	}
 
-	if (in)
-		g_warning("[%s] retrieving from \"%s\"%s", what, path, instead);
-	else if (instead == instead_str)
+	if (in) {
+		if (common_dbg > 0)
+			g_message("[%s] retrieving from \"%s\"%s", what, path, instead);
+	} else if (instead == instead_str) {
 		g_warning("[%s] unable to retrieve: tried %d alternate location%s",
 			what, fvcnt, fvcnt == 1 ? "" : "s");
-    else
+    } else {
 		g_warning("[%s] unable to retrieve: no alternate locations known",
 			what);
+	}
 
 out:
 
-	if (NULL != path)
-		G_FREE_NULL(path);
-	if (NULL != path_orig)
-		G_FREE_NULL(path_orig);
+	G_FREE_NULL(path);
+	G_FREE_NULL(path_orig);
 	if (in != NULL && chosen != NULL)
 		*chosen = idx;
 
@@ -184,9 +187,10 @@ out:
  * cannot be found, try opening the ".orig" variant if already present.
  * If not found, try with successive alternatives, if supplied.
  *
+ * @attention
  * NB: the supplied `fv' argument is a vector of `fvcnt' elements.
  *
- * Returns opened FILE, or NULL if we were unable to open any.
+ * @returns opened FILE, or NULL if we were unable to open any.
  */
 FILE *
 file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
@@ -199,9 +203,10 @@ file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
  * cannot be found, try opening the ".orig" variant if already present.
  * If not found, try with successive alternatives, if supplied.
  *
+ * @attention
  * NB: the supplied `fv' argument is a vector of `fvcnt' elements.
  *
- * Returns opened FILE, or NULL if we were unable to open any.
+ * @returns opened FILE, or NULL if we were unable to open any.
  */
 FILE *
 file_config_open_read_norename(
@@ -226,7 +231,7 @@ file_config_open_read_norename_chosen(
  * yet and open a ".new" instead.  Renaming will occur afterwards, when
  * file_config_close() is called.
  *
- * Returns opened FILE if success, NULL on error.
+ * @returns opened FILE if success, NULL on error.
  */
 static FILE *
 file_config_open(const gchar *what, const file_path_t *fv)
@@ -258,13 +263,15 @@ file_config_open_write(const gchar *what, const file_path_t *fv)
 
 /**
  * Close configuration file opened for writing, and rename it.
- * Returns TRUE on success.
+ *
+ * @returns TRUE on success.
  */
 gboolean
 file_config_close(FILE *out, const file_path_t *fv)
 {
 	char *path = NULL;
 	char *path_new = NULL;
+	gboolean success = FALSE;
 
 	if (0 != fclose(out)) {
 		g_warning("could not flush \"%s\": %s", fv->name, g_strerror(errno));
@@ -283,17 +290,13 @@ file_config_close(FILE *out, const file_path_t *fv)
 		goto failed;
 	}
 
-	G_FREE_NULL(path_new);
-	G_FREE_NULL(path);
-	return TRUE;
+	success = TRUE;
 
 failed:
 
-	if (NULL != path_new)
-		G_FREE_NULL(path_new);
-	if (NULL != path)
-		G_FREE_NULL(path);
-	return FALSE;
+	G_FREE_NULL(path_new);
+	G_FREE_NULL(path);
+	return success;
 }
 
 /**
@@ -302,7 +305,7 @@ failed:
 void
 file_config_preamble(FILE *out, const gchar *what)
 {
-	time_t now = time((time_t *) NULL);
+	time_t now = tm_time();
 
 	g_assert(out);
 
@@ -316,7 +319,11 @@ file_config_preamble(FILE *out, const gchar *what)
 void
 file_path_set(file_path_t *fp, const char *dir, const char *name)
 {
-	g_assert(NULL != fp);
+	g_assert(fp);
+	g_assert(dir);
+	g_assert(name);
+	g_assert(is_absolute_path(dir));
+
 	fp->dir = dir;
 	fp->name = name;
 }
@@ -337,6 +344,10 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 		return -1;
 	}
 
+#ifdef O_NOCTTY
+	flags |= O_NOCTTY;
+#endif /* O_NOCTTY */
+
 	fd = open(path, flags, mode);
 	if (fd >= 0)
 		return fd;
@@ -361,7 +372,7 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 	) {
 		fd = open(path, flags, mode);
 		if (fd >= 0) {
-			g_warning("had to close a banned fd to %s file", what);
+			g_warning("do_open(): had to close a banned fd to %s file", what);
 			return fd;
 		}
 	}
@@ -374,12 +385,15 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 	 */
 
 	if (errno == 0) {
-		g_warning("open() returned -1 with errno = 0, assuming ENOENT");
+		g_warning("do_open(): "
+			"open() returned -1 with errno = 0, assuming ENOENT");
 		errno = ENOENT;
 	}
 
-	if (!missing || errno != ENOENT)
-		g_warning("can't %s file \"%s\": %s", what, path, g_strerror(errno));
+	if (!missing || errno != ENOENT) {
+		g_warning("do_open(): can't %s file \"%s\": %s",
+			what, path, g_strerror(errno));
+	}
 
 	return -1;
 }
