@@ -1,5 +1,5 @@
 /*
- * $Id: hashlist.c,v 1.14 2005/06/29 14:24:26 daichik Exp $
+ * $Id: hashlist.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -42,7 +42,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hashlist.c,v 1.14 2005/06/29 14:24:26 daichik Exp $");
+RCSID("$Id: hashlist.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "hashlist.h"
 #include "misc.h"
@@ -78,6 +78,11 @@ struct hash_list_iter {
 	guint stamp;
 };
 
+struct hash_list_item {
+	gpointer orig_key;
+	GList *list;
+};
+
 #if 0
 #define USE_HASH_LIST_REGRESSION 1
 #endif
@@ -88,6 +93,7 @@ struct hash_list_iter {
 static void inline hash_list_regression(const hash_list_t *hl)
 {
 	g_assert(NULL != hl->ht);
+	g_assert(hl->len >= 0);
 	g_assert(g_list_first(hl->l) == hl->l);
 	g_assert(g_list_first(hl->last) == hl->l);
 	g_assert(g_list_last(hl->l) == hl->last);
@@ -128,11 +134,11 @@ static void inline hash_list_regression(const hash_list_t *hl)
  * Create a new hash list.
  */
 hash_list_t *
-hash_list_new(void)
+hash_list_new(GHashFunc hash_func, GEqualFunc eq_func)
 {
-	hash_list_t *hl = walloc(sizeof(*hl));
+	hash_list_t *hl = walloc(sizeof *hl);
 	hl->l = NULL;
-	hl->ht = g_hash_table_new(NULL, NULL);
+	hl->ht = g_hash_table_new(hash_func, eq_func);
 	hl->last = NULL;
 	hl->refcount = 1;
 	hl->len = 0;
@@ -149,6 +155,8 @@ hash_list_new(void)
 void
 hash_list_free(hash_list_t *hl)
 {
+	GList *iter;
+
 	g_assert(NULL != hl);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
@@ -159,69 +167,47 @@ hash_list_free(hash_list_t *hl)
 			"(hl=%p, hl->refcount=%d)",
 			cast_to_gconstpointer(hl), hl->refcount);
 	}
+
 	g_hash_table_destroy(hl->ht);
-	g_list_free(hl->l);
 	hl->ht = NULL;
+
+	for (iter = hl->l; NULL != iter; iter = g_list_next(iter)) {
+		struct hash_list_item *item = iter->data;
+		wfree(item, sizeof *item);
+	}
+	g_list_free(hl->l);
 	hl->l = NULL;
-	hl->len = 0;
+
 	hl->magic = 0;
 
-	wfree(hl, sizeof(*hl));
+	wfree(hl, sizeof *hl);
 }
 
 /**
- * Hash table removal iterator -- always returns TRUE.
- */
-static gboolean
-hash_list_rt(gpointer uu_key, gpointer uu_value, gpointer uu_udata)
-{
-	(void) uu_key;
-	(void) uu_value;
-	(void) uu_udata;
-
-	return TRUE;
-}
-
-/**
- * Clear the structure.  Items are simply de-referenced, not freed.
+ * Append `key' to the list.
  */
 void
-hash_list_clear(hash_list_t *hl)
+hash_list_append(hash_list_t *hl, gpointer key)
 {
-	g_assert(NULL != hl);
-	g_assert(HASH_LIST_MAGIC == hl->magic);
-	g_assert(equiv(hl->len == 0, hl->last == NULL));
-	g_assert(1 == hl->refcount);
-	hash_list_regression(hl);
-
-	if (hl->len == 0)
-		return;
-
-	g_hash_table_foreach_remove(hl->ht, hash_list_rt, NULL);
-	g_list_free(hl->l);
-	hl->l = hl->last = NULL;
-	hl->len = 0;
-	hl->stamp++;
-}
-
-/**
- * Append `data' to the list.
- */
-void
-hash_list_append(hash_list_t *hl, gpointer data)
-{
-	g_assert(NULL != data);
+	struct hash_list_item *item;
+	
 	g_assert(NULL != hl);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(1 == hl->refcount);
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
 	hash_list_regression(hl);
 
-	hl->last = g_list_last(g_list_append(hl->last, data));
+	item = walloc(sizeof *item);
+	item->orig_key = key;
+
+	hl->last = g_list_last(g_list_append(hl->last, item));
 	if (NULL == hl->l)
 		hl->l = hl->last;
-	g_assert(NULL == g_hash_table_lookup(hl->ht, data));
-	g_hash_table_insert(hl->ht, data, hl->last);
+	item->list = hl->last;
+
+	g_assert(NULL == g_hash_table_lookup(hl->ht, key));
+	g_hash_table_insert(hl->ht, deconstify_gpointer(key), item);
+
 	hl->len++;
 	hl->stamp++;
 
@@ -229,23 +215,79 @@ hash_list_append(hash_list_t *hl, gpointer data)
 }
 
 /**
- * Prepend `data' to the list.
+ * Prepend `key' to the list.
  */
 void
-hash_list_prepend(hash_list_t *hl, gpointer data)
+hash_list_prepend(hash_list_t *hl, gpointer key)
 {
-	g_assert(NULL != data);
+	struct hash_list_item *item;
+
 	g_assert(NULL != hl);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(1 == hl->refcount);
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
 	hash_list_regression(hl);
 
-	hl->l = g_list_prepend(hl->l, data);
+	item = walloc(sizeof *item);
+	item->orig_key = key;
+
+	hl->l = g_list_prepend(hl->l, item);
 	if (NULL == hl->last)
 		hl->last = hl->l;
-	g_assert(NULL == g_hash_table_lookup(hl->ht, data));
-	g_hash_table_insert(hl->ht, data, hl->l);
+	item->list = hl->l;
+
+	g_assert(NULL == g_hash_table_lookup(hl->ht, key));
+	g_hash_table_insert(hl->ht, deconstify_gpointer(key), item);
+
+	hl->len++;
+	hl->stamp++;
+
+	hash_list_regression(hl);
+}
+
+/**
+ * Insert `key' into the list.
+ */
+void
+hash_list_insert_sorted(hash_list_t *hl, gpointer key, GCompareFunc func)
+{
+	struct hash_list_item *item;
+	GList *iter;
+
+	g_assert(NULL != hl);
+	g_assert(NULL != func);
+	g_assert(HASH_LIST_MAGIC == hl->magic);
+	g_assert(1 == hl->refcount);
+	g_assert(equiv(hl->len == 0, hl->last == NULL));
+	hash_list_regression(hl);
+
+	g_assert(NULL == g_hash_table_lookup(hl->ht, key));
+
+	for (iter = hl->l; iter; iter = g_list_next(iter)) {
+		item = iter->data;
+		if (func(key, item->orig_key) <= 0)
+			break;
+	}
+	
+	item = walloc(sizeof *item);
+	item->orig_key = key;
+	item->list = g_list_alloc();
+	item->list->data = item;
+	item->list->prev = g_list_previous(iter);
+	item->list->next = iter;
+	if (item->list->prev) {
+		item->list->prev->next = item->list;
+	}
+
+	if (hl->l == iter) {
+		hl->l = item->list;
+	}
+	if (!hl->last) {
+		hl->last = item->list;
+	}
+
+	g_hash_table_insert(hl->ht, deconstify_gpointer(key), item);
+
 	hl->len++;
 	hl->stamp++;
 
@@ -254,25 +296,27 @@ hash_list_prepend(hash_list_t *hl, gpointer data)
 
 /**
  * Remove `data' from the list.
+ * @return The data that associated with the given key.
  */
 void
-hash_list_remove(hash_list_t *hl, gpointer data)
+hash_list_remove(hash_list_t *hl, gpointer key)
 {
-	GList *l;
+	struct hash_list_item *item;
 
-	g_assert(NULL != data);
 	g_assert(1 == hl->refcount);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
 	g_assert(hl->len > 0);
 	hash_list_regression(hl);
 
-	l = (GList *) g_hash_table_lookup(hl->ht, data);
-	g_assert(NULL != l);
-	if (hl->last == l)
+	item = g_hash_table_lookup(hl->ht, key);
+	g_assert(item);
+	if (hl->last == item->list)
 		hl->last = g_list_previous(hl->last);
-	hl->l = g_list_delete_link(hl->l, l);
-	g_hash_table_remove(hl->ht, data);
+	hl->l = g_list_delete_link(hl->l, item->list);
+	g_hash_table_remove(hl->ht, key);
+	wfree(item, sizeof *item);
+
 	hl->len--;
 	hl->stamp++;
 
@@ -280,7 +324,7 @@ hash_list_remove(hash_list_t *hl, gpointer data)
 }
 
 /**
- * @returns the last item of the list, or NULL if none.
+ * @returns The data associated with the last item, or NULL if none.
  */
 gpointer
 hash_list_last(const hash_list_t *hl)
@@ -291,7 +335,14 @@ hash_list_last(const hash_list_t *hl)
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
 	hash_list_regression(hl);
 
-	return NULL != hl->last ? hl->last->data : NULL;
+	if (hl->last) {
+		struct hash_list_item *item;
+
+		item = hl->last->data;
+		g_assert(item);
+		return item->orig_key;
+	}
+	return NULL;
 }
 
 /**
@@ -306,13 +357,107 @@ hash_list_first(const hash_list_t *hl)
 	g_assert(equiv(hl->len == 0, hl->l == NULL));
 	hash_list_regression(hl);
 
-	return NULL != hl->l ? hl->l->data : NULL;
+	if (hl->l) {
+		struct hash_list_item *item;
+
+		item = hl->l->data;
+		g_assert(item);
+		return item->orig_key;
+	}
+	return NULL;
+}
+
+/**
+ * Move entry to the head of the list.
+ */
+void
+hash_list_moveto_head(hash_list_t *hl, gpointer key)
+{
+	struct hash_list_item *item;
+
+	g_assert(1 == hl->refcount);
+	g_assert(HASH_LIST_MAGIC == hl->magic);
+	g_assert(equiv(hl->len == 0, hl->last == NULL));
+	g_assert(hl->len > 0);
+	hash_list_regression(hl);
+
+
+	item = g_hash_table_lookup(hl->ht, key);
+	g_assert(item);
+
+	if (hl->l == item->list)
+		goto done;				/* Item already at the head */
+
+	/*
+	 * Remove item from list
+	 */
+
+	if (hl->last == item->list)
+		hl->last = g_list_previous(hl->last);
+	hl->l = g_list_delete_link(hl->l, item->list);
+
+	g_assert(hl->l != NULL);		/* Or item would be at the head */
+	g_assert(hl->last != NULL);		/* Item not the head and not sole entry */
+
+	/*
+	 * Insert link back at the head.
+	 */
+
+	hl->l = g_list_prepend(hl->l, item);
+	item->list = hl->l;
+
+done:
+	hl->stamp++;
+
+	hash_list_regression(hl);
+}
+
+/**
+ * Move entry to the tail of the list.
+ */
+void
+hash_list_moveto_tail(hash_list_t *hl, gpointer key)
+{
+	struct hash_list_item *item;
+
+	g_assert(1 == hl->refcount);
+	g_assert(HASH_LIST_MAGIC == hl->magic);
+	g_assert(equiv(hl->len == 0, hl->last == NULL));
+	g_assert(hl->len > 0);
+	hash_list_regression(hl);
+
+
+	item = g_hash_table_lookup(hl->ht, key);
+	g_assert(item);
+
+	if (hl->last == item->list)
+		goto done;				/* Item already at the tail */
+
+	/*
+	 * Remove item from list
+	 */
+
+	hl->l = g_list_delete_link(hl->l, item->list);
+
+	g_assert(hl->l != NULL);		/* Or item would be at the tail */
+
+	/*
+	 * Insert link back at the tail.
+	 */
+
+	hl->last = g_list_last(g_list_append(hl->last, item));
+	item->list = hl->last;
+
+done:
+	hl->stamp++;
+
+	hash_list_regression(hl);
 }
 
 /**
  * @returns the length of the list.
  */
-gint
+guint
 hash_list_length(const hash_list_t *hl)
 {
 	g_assert(NULL != hl);
@@ -332,20 +477,23 @@ hash_list_iterator(hash_list_t *hl)
 {
 	hash_list_iter_t *i;
 
-	g_assert(NULL != hl);
-	g_assert(hl->refcount > 0);
-	g_assert(HASH_LIST_MAGIC == hl->magic);
-	g_assert(equiv(hl->len == 0, hl->last == NULL));
+	if (hl) {
+		g_assert(hl->refcount > 0);
+		g_assert(HASH_LIST_MAGIC == hl->magic);
+		g_assert(equiv(hl->len == 0, hl->last == NULL));
 
-	i = walloc(sizeof(*i));
+		i = walloc(sizeof(*i));
 
-	i->magic = HASH_LIST_ITER_MAGIC;
-	i->hl = hl;
-	i->l = hl->l;
-	i->stamp = hl->stamp;
-	i->pos = -1;				/* Before first item */
-	i->move = +1;
-	hl->refcount++;
+		i->magic = HASH_LIST_ITER_MAGIC;
+		i->hl = hl;
+		i->l = hl->l;
+		i->stamp = hl->stamp;
+		i->pos = -1;				/* Before first item */
+		i->move = +1;
+		hl->refcount++;
+	} else {
+		i = NULL;
+	}
 
 	return i;
 }
@@ -359,20 +507,23 @@ hash_list_iterator_last(hash_list_t *hl)
 {
 	hash_list_iter_t *i;
 
-	g_assert(NULL != hl);
-	g_assert(HASH_LIST_MAGIC == hl->magic);
-	g_assert(hl->refcount > 0);
-	g_assert(equiv(hl->len == 0, hl->last == NULL));
+	if (hl) {
+		g_assert(HASH_LIST_MAGIC == hl->magic);
+		g_assert(hl->refcount > 0);
+		g_assert(equiv(hl->len == 0, hl->last == NULL));
 
-	i = walloc(sizeof(*i));
+		i = walloc(sizeof(*i));
 
-	i->magic = HASH_LIST_ITER_MAGIC;
-	i->hl = hl;
-	i->l = hl->last;
-	i->stamp = hl->stamp;
-	i->pos = hl->len;			/* After last item */
-	i->move = -1;
-	hl->refcount++;
+		i->magic = HASH_LIST_ITER_MAGIC;
+		i->hl = hl;
+		i->l = hl->last;
+		i->stamp = hl->stamp;
+		i->pos = hl->len;			/* After last item */
+		i->move = -1;
+		hl->refcount++;
+	} else {
+		i = NULL;
+	}
 
 	return i;
 }
@@ -395,7 +546,12 @@ hash_list_next(hash_list_iter_t *i)
 
 	g_assert(i->l != NULL || i->pos >= i->hl->len);
 
-	return NULL != i->l ? i->l->data : NULL;
+	if (i->l) {
+		struct hash_list_item *item;
+		item = i->l->data;
+		return item->orig_key;
+	}
+	return NULL;
 }
 
 /**
@@ -431,7 +587,12 @@ hash_list_previous(hash_list_iter_t *i)
 
 	g_assert(i->l != NULL || i->pos < 0);
 
-	return NULL != i->l ? i->l->data : NULL;
+	if (i->l) {
+		struct hash_list_item *item;
+		item = i->l->data;
+		return item->orig_key;
+	}
+	return NULL;
 }
 
 /**
@@ -482,33 +643,36 @@ hash_list_has_follower(const hash_list_iter_t *i)
 void
 hash_list_release(hash_list_iter_t *i)
 {
-	g_assert(NULL != i);
-	g_assert(HASH_LIST_ITER_MAGIC == i->magic);
-	g_assert(i->hl->refcount > 0);
+	if (i) {
+		g_assert(HASH_LIST_ITER_MAGIC == i->magic);
+		g_assert(i->hl->refcount > 0);
 
-	i->hl->refcount--;
-	i->magic = 0;
+		i->hl->refcount--;
+		i->magic = 0;
 
-	wfree(i, sizeof(*i));
+		wfree(i, sizeof *i);
+	}
 }
 
 /**
  * Check whether hashlist contains the `data'.
  */
 gboolean
-hash_list_contains(hash_list_t *hl, gpointer data)
+hash_list_contains(hash_list_t *hl, gconstpointer key, gpointer *orig_key_ptr)
 {
-	GList *l;
+	struct hash_list_item *item;
 
 	g_assert(NULL != hl);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(NULL != hl->ht);
-	g_assert(NULL != data);
 	g_assert(hl->refcount > 0);
 	hash_list_regression(hl);
 
-	l = g_hash_table_lookup(hl->ht, data);
-	return NULL != l && l->data == data;
+	item = g_hash_table_lookup(hl->ht, key);
+	if (item && orig_key_ptr) {
+		*orig_key_ptr = item->orig_key;
+	}
+	return NULL != item;
 }
 
 /**
@@ -517,6 +681,8 @@ hash_list_contains(hash_list_t *hl, gpointer data)
 void
 hash_list_foreach(const hash_list_t *hl, GFunc func, gpointer user_data)
 {
+	GList *list;
+	
 	g_assert(NULL != hl);
 	g_assert(HASH_LIST_MAGIC == hl->magic);
 	g_assert(NULL != func);
@@ -524,7 +690,12 @@ hash_list_foreach(const hash_list_t *hl, GFunc func, gpointer user_data)
 	g_assert(equiv(hl->len == 0, hl->last == NULL));
 	hash_list_regression(hl);
 
-	G_LIST_FOREACH(hl->l, func, user_data);
+	for (list = hl->l; NULL != list; list = g_list_next(list)) {
+		struct hash_list_item *item;
+
+		item = list->data;
+		func(item->orig_key, user_data);
+	}
 
 	hash_list_regression(hl);
 }
