@@ -1,5 +1,5 @@
 /*
- * $Id: header.c,v 1.10 2005/11/09 19:16:53 cbiere Exp $
+ * $Id: header.c 12140 2006-10-27 00:41:49Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,21 +35,68 @@
 
 #include "common.h"
 
-RCSID("$Id: header.c,v 1.10 2005/11/09 19:16:53 cbiere Exp $");
+RCSID("$Id: header.c 12140 2006-10-27 00:41:49Z cbiere $")
 
 #include "header.h"
 #include "glib-missing.h"
 #include "misc.h"
 #include "walloc.h"
 #include "getline.h"		/* For MAX_LINE_SIZE */
+#include "slist.h"
+
 #include "override.h"		/* Must be the last header included */
 
+/*
+ * The `headers' field is a hash table indexed by field name (normalized).
+ * Each value (GString) holds a private copy of the string making that header,
+ * with all continuations removed (leading spaces collapsed into one), and
+ * indentical fields concatenated using ", " separators, per RFC2616.
+ *
+ * The `fields' field holds a list of all the fields, in the order they
+ * appeared.  The value is a header_field_t structure.  It allows one to
+ * dump the header exactly as it was read.
+ */
+
+struct header {
+	GHashTable *headers;		/**< Indexed by name, normalized */
+	slist_t *fields;			/**< Ordered list of header_field_t */
+	gint flags;					/**< Various operating flags */
+	gint size;					/**< Total header size, in bytes */
+	gint num_lines;				/**< Total header lines seen */
+};
+
+/**
+ * A header field.
+ *
+ * It holds the field name, and all the lines that make up that field.
+ * The first line has the field name and the ":" stripped, as well as
+ * all the leading spaces.  Continuations also have their leading spaces
+ * stripped out.
+ *
+ * For instance, assume the following header field:
+ *
+ *    - X-Comment: first line
+ *         and continuation of first line
+ *
+ * Then the structure would contain, with () denoting a list:
+ *
+ *    - name = "X-Comment"
+ *    - lines = ("first line", "and continuation of first line")
+ */
+
+typedef struct {
+	gchar *name;				/**< Field name */
+	slist_t *lines;				/**< List of lines making this header */
+} header_field_t;
+
 /***
  *** Operating flags
  ***/
 
-#define HEAD_F_EOH		0x00000001			/**< EOH reached */
-#define HEAD_F_SKIP		0x00000002			/**< Skip continuations */
+enum {
+	HEAD_F_EOH	= 0x00000001,	/**< EOH reached */
+	HEAD_F_SKIP	= 0x00000002	/**< Skip continuations */
+};
 
 /***
  *** Error code management
@@ -79,6 +126,12 @@ header_strerror(guint errnum)
 	return error_str[errnum];
 }
 
+gint
+header_num_lines(const header_t *h)
+{
+	return h->num_lines;
+}
+
 /*
  * XXX share normalized header strings into hash table?
  */
@@ -126,28 +179,31 @@ hfield_make(const gchar *name)
 {
 	header_field_t *h;
 
-	h = (header_field_t *) g_malloc0(sizeof(header_field_t));
+	h = walloc0(sizeof *h);
 	h->name = g_strdup(name);
 
 	return h;
 }
 
+static void
+hfield_free_item(gpointer p, gpointer unused_data)
+{
+	(void) unused_data;
+	G_FREE_NULL(p);
+}
+
 /**
- * hfield_free
- *
  * Dispose of the header field.
  */
 static void
 hfield_free(header_field_t *h)
 {
-	GSList *l;
-
-	for (l = h->lines; l; l = l->next)
-		G_FREE_NULL(l->data);
-	g_slist_free(h->lines);
-
+	if (h->lines) {
+		slist_foreach(h->lines, hfield_free_item, NULL);
+		slist_free(&h->lines);
+	}
 	G_FREE_NULL(h->name);
-	G_FREE_NULL(h);
+	wfree(h, sizeof *h);
 }
 
 /**
@@ -157,7 +213,10 @@ hfield_free(header_field_t *h)
 static void
 hfield_append(header_field_t *h, const gchar *text)
 {
-	h->lines = g_slist_append(h->lines, g_strdup(text));
+	if (!h->lines) {
+		h->lines = slist_new();
+	}
+	slist_append(h->lines, g_strdup(text));
 }
 
 /**
@@ -166,24 +225,39 @@ hfield_append(header_field_t *h, const gchar *text)
 static void
 hfield_dump(const header_field_t *h, FILE *out)
 {
-	GSList *l;
-
-	fprintf(out, "%s: ", h->name);
+	slist_iter_t *iter;
+	gboolean first;
 
 	g_assert(h->lines);
 
-	for (l = h->lines; l; l = l->next) {
-		if (l != h->lines)
+	fprintf(out, "%s: ", h->name);
+
+	first = TRUE;
+	iter = slist_iter_on_head(h->lines);
+	for (/* NOTHING */; slist_iter_has_item(iter); slist_iter_next(iter)) {
+		if (!first) {
+			first = FALSE;
 			fputs("    ", out);			/* Continuation line */
-		fputs(l->data, out);
+		}
+		fputs(slist_iter_current(iter), out);
 		fputc('\n', out);
 	}
+	slist_iter_free(&iter);	
 }
 
 /***
  *** header object
  ***/
 
+static GHashTable *
+header_get_table(header_t *o)
+{
+	if (!o->headers) {
+		o->headers = g_hash_table_new(g_str_hash, g_str_equal);
+	}
+	return o->headers;
+}
+
 /**
  * Create a new header object.
  */
@@ -192,15 +266,11 @@ header_make(void)
 {
 	header_t *o;
 
-	o = (header_t *) g_malloc0(sizeof(header_t));
-	o->headers = g_hash_table_new(g_str_hash, g_str_equal);
-
+	o = walloc0(sizeof *o);
 	return o;
 }
 
 /**
- * htable callback
- *
  * Frees the key/values from the headers hash.
  */
 static gboolean
@@ -209,7 +279,7 @@ free_header_data(gpointer key, gpointer value, gpointer unused_udata)
 	(void) unused_udata;
 
 	G_FREE_NULL(key);		/* XXX if shared, don't do that */
-	g_string_free((GString *) value, TRUE);
+	g_string_free(value, TRUE);
 	return TRUE;
 }
 
@@ -222,9 +292,14 @@ header_free(header_t *o)
 	g_assert(o);
 
 	header_reset(o);
+	wfree(o, sizeof *o);
+}
 
-	g_hash_table_destroy(o->headers);
-	G_FREE_NULL(o);
+static void
+header_reset_item(gpointer p, gpointer unused_data)
+{
+	(void) unused_data;
+	hfield_free(p);
 }
 
 /**
@@ -233,18 +308,20 @@ header_free(header_t *o)
 void
 header_reset(header_t *o)
 {
-	GSList *l;
+	static const header_t zero_header;
 
 	g_assert(o);
 
-	g_hash_table_foreach_remove(o->headers, free_header_data, NULL);
-
-	for (l = o->fields; l; l = l->next)
-		hfield_free((header_field_t *) l->data);
-	g_slist_free(o->fields);
-	o->fields = NULL;
-
-	o->size = o->lines = o->flags = 0;
+	if (o->headers) {
+		g_hash_table_foreach_remove(o->headers, free_header_data, NULL);
+		g_hash_table_destroy(o->headers);
+		o->headers = NULL;
+	}
+	if (o->fields) {
+		slist_foreach(o->fields, header_reset_item, NULL);
+		slist_free(&o->fields);
+	}
+	*o = zero_header;
 }
 
 /**
@@ -260,8 +337,11 @@ header_get(const header_t *o, const gchar *field)
 {
 	GString *v;
 
-	v = g_hash_table_lookup(o->headers, (gpointer) field);
-
+	if (o->headers) {
+		v = g_hash_table_lookup(o->headers, deconstify_gchar(field));
+	} else {
+		v = NULL;
+	}
 	return v ? v->str : NULL;
 }
 
@@ -270,15 +350,10 @@ header_get(const header_t *o, const gchar *field)
  * copy of the internal value, so it may be kept around, but must be
  * freed by the caller.
  */
-gchar *header_getdup(const header_t *o, const gchar *field)
+gchar *
+header_getdup(const header_t *o, const gchar *field)
 {
-	GString *v;
-
-	v = g_hash_table_lookup(o->headers, (gpointer) field);
-	if (!v)
-		return NULL;
-
-	return g_strdup(v->str);
+	return g_strdup(header_get(o, field));
 }
 
 /**
@@ -288,11 +363,11 @@ gchar *header_getdup(const header_t *o, const gchar *field)
 static void
 add_header(header_t *o, const gchar *field, const gchar *text)
 {
-	GHashTable *h = o->headers;
+	GHashTable *ht;
 	GString *v;
 
-	v = g_hash_table_lookup(h, field);
-
+	ht = header_get_table(o);
+	v = g_hash_table_lookup(ht, field);
 	if (v) {
 		/*
 		 * Header already exists, according to RFC2616 we need to append
@@ -311,7 +386,7 @@ add_header(header_t *o, const gchar *field, const gchar *text)
 
 		key = g_strdup(field);
 		v = g_string_new(text);
-		g_hash_table_insert(h, (gpointer) key, (gpointer) v);
+		g_hash_table_insert(ht, (gpointer) key, (gpointer) v);
 	}
 }
 
@@ -322,10 +397,10 @@ add_header(header_t *o, const gchar *field, const gchar *text)
 static void
 add_continuation(header_t *o, const gchar *field, const gchar *text)
 {
-	GHashTable *h = o->headers;
 	GString *v;
 
-	v = g_hash_table_lookup(h, field);
+	g_assert(o->headers);
+	v = g_hash_table_lookup(o->headers, field);
 	g_assert(v);
 	g_string_append_c(v, ' ');
 	g_string_append(v, text);
@@ -337,7 +412,8 @@ add_continuation(header_t *o, const gchar *field, const gchar *text)
  *
  * @return an error code, or HEAD_OK if appending was successful.
  */
-gint header_append(header_t *o, const gchar *text, gint len)
+gint
+header_append(header_t *o, const gchar *text, gint len)
 {
 	gchar buf[MAX_LINE_SIZE];
 	const gchar *p = text;
@@ -363,7 +439,7 @@ gint header_append(header_t *o, const gchar *text, gint len)
 	if (o->size >= HEAD_MAX_SIZE)
 		return HEAD_TOO_LARGE;
 
-	if (++(o->lines) >= HEAD_MAX_LINES)
+	if (++(o->num_lines) >= HEAD_MAX_LINES)
 		return HEAD_MANY_LINES;
 
 	/*
@@ -380,7 +456,7 @@ gint header_append(header_t *o, const gchar *text, gint len)
 		 * an unexpected continuation line.
 		 */
 
-		if (o->fields == 0)
+		if (NULL == o->fields)
 			return HEAD_CONTINUATION;		/* Unexpected continuation */
 
 		/*
@@ -418,7 +494,7 @@ gint header_append(header_t *o, const gchar *text, gint len)
 		 * field we handled.
 		 */
 
-		hf = (header_field_t *) g_slist_last(o->fields)->data;
+		hf = slist_tail(o->fields);
 		hfield_append(hf, p);
 		add_continuation(o, hf->name, p);
 		o->size += len - (p - text);	/* Count only effective text */
@@ -499,23 +575,31 @@ gint header_append(header_t *o, const gchar *text, gint len)
 
 		hfield_append(hf, p);
 		add_header(o, buf, p);
-		o->fields = g_slist_append(o->fields, (gpointer) hf);
+		if (!o->fields) {
+			o->fields = slist_new();
+		}
+		slist_append(o->fields, hf);
 		o->size += len - (p - text);	/* Count only effective text */
 	}
 
 	return HEAD_OK;
 }
 
+static void
+header_dump_item(gpointer p, gpointer user_data)
+{
+	hfield_dump(p, user_data);
+}
+
 /**
  * Dump whole header on specified file.
  */
 void
 header_dump(const header_t *o, FILE *out)
 {
-	GSList *l;
-
-	for (l = o->fields; l; l = l->next)
-		hfield_dump((header_field_t *) l->data, out);
+	if (o->fields) {
+		slist_foreach(o->fields, header_dump_item, out);
+	}
 }
 
 /***
@@ -532,14 +616,14 @@ header_dump(const header_t *o, FILE *out)
  */
 struct header_fmt {
 	guint32 magic;
-	gint maxlen;				/**< Maximum line length before continuation */
-	GString *header;			/**< Header being built */
-	gchar sep[257];				/**< Optional separator */
-	gint seplen;				/**< Length of separator string */
-	gint stripped_seplen;		/**< Length of separator without trailing space */
-	gint current_len;			/**< Length of currently built line */
-	gboolean data_emitted;		/**< Whether data was ever emitted */
-	gboolean frozen;			/**< Header terminated */
+	gint maxlen;			/**< Maximum line length before continuation */
+	GString *header;		/**< Header being built */
+	gchar sep[257];			/**< Optional separator */
+	gint seplen;			/**< Length of separator string */
+	gint stripped_seplen;	/**< Length of separator without trailing space */
+	gint current_len;		/**< Length of currently built line */
+	gboolean data_emitted;	/**< Whether data was ever emitted */
+	gboolean frozen;		/**< Header terminated */
 };
 
 /**
@@ -616,7 +700,7 @@ header_fmt_make(const gchar *field, const gchar *separator, gint len_hint)
 void
 header_fmt_set_line_length(gpointer o, gint maxlen)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 	g_assert(maxlen > 0);
@@ -630,12 +714,13 @@ header_fmt_set_line_length(gpointer o, gint maxlen)
 void
 header_fmt_free(gpointer o)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 
 	g_string_free(hf->header, TRUE);
-	wfree(hf, sizeof(*hf));
+	hf->magic = 0;
+	wfree(hf, sizeof *hf);
 }
 
 /**
@@ -650,7 +735,7 @@ header_fmt_free(gpointer o)
 gboolean
 header_fmt_value_fits(gpointer o, gint len, gint maxlen)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 	gint final_len;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
@@ -684,13 +769,14 @@ static void
 header_fmt_append_full(struct header_fmt *hf, const gchar *str,
 	const gchar *separator, gint slen, gint sslen)
 {
-	gint len;
+	size_t len;
 	gint curlen;
 
 	len = strlen(str);
+	g_assert(len <= INT_MAX);	/* Legacy bug */
 	curlen = hf->current_len;
 
-	if (curlen + len + slen > hf->maxlen) {
+	if (curlen + len + slen > (size_t) hf->maxlen) {
 		/*
 		 * Emit sperator, if any and data was already emitted.
 		 */
@@ -728,7 +814,7 @@ header_fmt_append_full(struct header_fmt *hf, const gchar *str,
 void
 header_fmt_append(gpointer o, const gchar *str, const gchar *separator)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 	gint seplen;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
@@ -751,7 +837,7 @@ header_fmt_append(gpointer o, const gchar *str, const gchar *separator)
 void
 header_fmt_append_value(gpointer o, const gchar *str)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 	g_assert(!hf->frozen);
@@ -765,7 +851,7 @@ header_fmt_append_value(gpointer o, const gchar *str)
 gint
 header_fmt_length(gpointer o)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 
@@ -779,7 +865,7 @@ header_fmt_length(gpointer o)
 void
 header_fmt_end(gpointer o)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 	g_assert(!hf->frozen);
@@ -794,7 +880,7 @@ header_fmt_end(gpointer o)
 gchar *
 header_fmt_string(gpointer o)
 {
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 
@@ -811,7 +897,7 @@ gchar *
 header_fmt_to_string(gpointer o)
 {
 	static gchar line[HEADER_FMT_MAX_SIZE + 1];
-	struct header_fmt *hf = (struct header_fmt *) o;
+	struct header_fmt *hf = o;
 
 	g_assert(hf->magic == HEADER_FMT_MAGIC);
 
