@@ -1,5 +1,5 @@
 /*
- * $Id: header.h,v 1.5 2005/11/09 19:16:53 cbiere Exp $
+ * $Id: header.h 12140 2006-10-27 00:41:49Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -41,51 +41,12 @@
 
 /**
  * Header parsing and holding data structures.
- *
- * The `headers' field is a hash table indexed by field name (normalized).
- * Each value (GString) holds a private copy of the string making that header,
- * with all continuations removed (leading spaces collapsed into one), and
- * indentical fields concatenated using ", " separators, per RFC2616.
- *
- * The `fields' field holds a list of all the fields, in the order they
- * appeared.  The value is a header_field_t structure.  It allows one to
- * dump the header exactly as it was read.
  */
 
-typedef struct header {
-	GHashTable *headers;		/**< Indexed by name, normalized */
-	GSList *fields;				/**< Ordered list of header_field_t */
-	gint flags;					/**< Various operating flags */
-	gint size;					/**< Total header size, in bytes */
-	gint lines;					/**< Total header lines seen */
-} header_t;
+struct header;
+typedef struct header header_t;
 
-#define HEADER_SIZE(h)		((h)->size)
-#define HEADER_LINES(h)		((h)->lines)
-
-/**
- * A header field.
- *
- * It holds the field name, and all the lines that make up that field.
- * The first line has the field name and the ":" stripped, as well as
- * all the leading spaces.  Continuations also have their leading spaces
- * stripped out.
- *
- * For instance, assume the following header field:
- *
- *    - X-Comment: first line
- *         and continuation of first line
- *
- * Then the structure would contain, with () denoting a list:
- *
- *    - name = "X-Comment"
- *    - lines = ("first line", "and continuation of first line")
- */
-
-typedef struct {
-	gchar *name;				/**< Field name */
-	GSList *lines;				/**< List of lines making this header */
-} header_field_t;
+gint header_num_lines(const header_t *h);
 
 /*
  * Error codes.
