@@ -1,5 +1,5 @@
 /*
- * $Id: iprange.c,v 1.6 2005/04/24 23:03:16 cbiere Exp $
+ * $Id: iprange.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -24,15 +24,21 @@
  */
 
 /**
+ * @ingroup lib
  * @file
  *
+ * Needs brief description here.
+ *
  * Load, parse and lookup IP addresses from a set of IP ranges defined
  * by a list of addresses in CIDR (Classless Internet Domain Routing) format.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: iprange.c,v 1.6 2005/04/24 23:03:16 cbiere Exp $");
+RCSID("$Id: iprange.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "iprange.h"
 #include "misc.h"
@@ -58,7 +64,7 @@ RCSID("$Id: iprange.c,v 1.6 2005/04/24 23:03:16 cbiere Exp $");
  * the time the range was added to the database.
  */
 
-#define IPRANGE_MARK	1		/* Magic tagging of user values */
+#define IPRANGE_MARK	1		/**< Magic tagging of user values */
 
 /*
  * A "database" descriptor, holding the CIDR networks and their attached value.
@@ -68,43 +74,43 @@ RCSID("$Id: iprange.c,v 1.6 2005/04/24 23:03:16 cbiere Exp $");
 #define IPAGESZ			256
 
 struct iprange_db {
-	guint32 magic;			/* Magic number */
-	gpointer lvl1[IPAGESZ];	/* Level-1 index */
+	guint32 magic;			/**< Magic number */
+	gpointer lvl1[IPAGESZ];	/**< Level-1 index */
 	/* Allocation */
-	iprange_clone_t clone;	/* Cloning callback for values */
-	iprange_free_t free;	/* Freeing callback for values */
+	iprange_clone_t clone;	/**< Cloning callback for values */
+	iprange_free_t free;	/**< Freeing callback for values */
 	/* Statistics */
-	gint range_count;		/* Amount of network ranges registered */
-	gint lvl2_count;		/* Amount of level-2 pages, for stats */
-	gint head_count;		/* Amount of heads in level-2 pages, for stats */
-	gint list_count;		/* Amount of items held in lists */
+	gint range_count;		/**< Amount of network ranges registered */
+	gint lvl2_count;		/**< Amount of level-2 pages, for stats */
+	gint head_count;		/**< Amount of heads in level-2 pages, for stats */
+	gint list_count;		/**< Amount of items held in lists */
 };
 
-/*
+/**
  * A CIDR network description.  Note that the network IP is already masked,
  * so to determine whether some IP falls within that network, one needs
  * to compare (IP & netmask) with the ip_masked field.
  */
 struct iprange_net {
-	guint32 ip_masked;	/* The IP of the network (meaningful bits) */
-	guint32 netmask;	/* The network bit mask, selecting meaningful bits */
-	gpointer value;		/* Value held */
-};
-
-/*
- * Error code stings.
- */
-static const gchar *iprange_errstr[] = {
-	"OK",									/* IPR_ERR_OK */
-	"Incorrect network prefix",				/* IPR_ERR_BAD_PREFIX */
-	"CIDR range clash",						/* IPR_ERR_RANGE_CLASH */
-	"Duplicate range",						/* IPR_ERR_RANGE_DUP */
-	"Range is subnet of existing range",	/* IPR_ERR_RANGE_SUBNET */
-	"Range is overlapping existing range",	/* IPR_ERR_RANGE_OVERLAP */
+	guint32 ip_masked;	/**< The IP of the network (meaningful bits) */
+	guint32 netmask;	/**< The network bit mask, selecting meaningful bits */
+	gpointer value;		/**< Value held */
 };
 
 /**
- * Returns human-readable error string for given error code.
+ * Error code stings.
+ */
+static const gchar *iprange_errstr[] = {
+	"OK",									/**< IPR_ERR_OK */
+	"Incorrect network prefix",				/**< IPR_ERR_BAD_PREFIX */
+	"CIDR range clash",						/**< IPR_ERR_RANGE_CLASH */
+	"Duplicate range",						/**< IPR_ERR_RANGE_DUP */
+	"Range is subnet of existing range",	/**< IPR_ERR_RANGE_SUBNET */
+	"Range is overlapping existing range",	/**< IPR_ERR_RANGE_OVERLAP */
+};
+
+/**
+ * @return human-readable error string for given error code.
  */
 const gchar *
 iprange_strerror(iprange_err_t errnum)
