@@ -1,5 +1,5 @@
 /*
- * $Id: url.c,v 1.12 2005/04/27 15:30:21 cbiere Exp $
+ * $Id: url.c 12290 2006-11-09 22:07:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -23,48 +23,69 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * URL handling of specific formats.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: url.c,v 1.12 2005/04/27 15:30:21 cbiere Exp $");
+RCSID("$Id: url.c 12290 2006-11-09 22:07:21Z cbiere $")
 
+#include "glib-missing.h"
+#include "host_addr.h"
 #include "url.h"
 #include "misc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
-/* XXX -- needs to initialze the debug level of the lib functions -- RAM */
-static gboolean url_debug = FALSE;
+extern guint32 lib_debug;
 
 #define ESCAPE_CHAR		'%'
-#define TRANSPARENT_CHAR(x,m) \
-	((x) >= 32 && (x) < 128 && (is_transparent[(x)-32] & (m)))
 
-/*
- * Reserved chars: ";", "/", "?", ":", "@", "=" and "&"
- * Unsafe chars  : " ", '"', "<", ">", "#", and "%"
- * Misc chars    : "{", "}", "|", "\", "^", "~", "[", "]" and "`"
+/**
+ * - Reserved chars: ";", "/", "?", ":", "@", "=" and "&"
+ * - Unsafe chars  : " ", '"', "<", ">", "#", and "%"
+ * - Misc chars    : "{", "}", "|", "\", "^", "~", "[", "]" and "`"
  *
- * Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
- * Bit 1 encodes regular transparent set minus '+' (query string).
+ * - Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
+ * - Bit 1 encodes regular transparent set minus '+' (query string).
+ * - Bit 2 encodes the set for fixing an incomplete escaping.
  */
 static const guint8 is_transparent[96] = {
 /*  0 1 2 3 4 5 6 7 8 9 a b c d e f */	/* 0123456789abcdef -            */
-    0,3,0,0,3,0,0,3,3,3,3,1,3,3,3,3,	/*  !"#$%&'()*+,-./ -  32 -> 47  */
-    3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,0,	/* 0123456789:;<=>? -  48 -> 63  */
-    0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,	/* @ABCDEFGHIJKLMNO -  64 -> 79  */
-    3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,3,	/* PQRSTUVWXYZ[\]^_ -  80 -> 95  */
-    0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,	/* `abcdefghijklmno -  96 -> 111 */
-    3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,	/* pqrstuvwxyz{|}~  - 112 -> 127 */
+    0,7,0,0,7,0,4,7,7,7,7,0,7,7,7,7,	/*  !"#$%&'()*+,-./ -  32 -> 47  */
+    7,7,7,7,7,7,7,7,7,7,4,0,0,4,0,4,	/* 0123456789:;<=>? -  48 -> 63  */
+    0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* @ABCDEFGHIJKLMNO -  64 -> 79  */
+    7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,7,	/* PQRSTUVWXYZ[\]^_ -  80 -> 95  */
+    0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* `abcdefghijklmno -  96 -> 111 */
+    7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,0,	/* pqrstuvwxyz{|}~  - 112 -> 127 */
 };
 
-#define PATH_MASK		0x1
-#define QUERY_MASK		0x2
+enum escape_mask {
+	PATH_MASK	= (1 << 0),
+	QUERY_MASK	= (1 << 1),
+	FIX_MASK	= (1 << 2)
+};
+
+static inline gboolean
+is_transparent_char(const gint c, const enum escape_mask m)
+{
+	return c >= 32 && c < 128 && (is_transparent[c - 32] & (guint8) m);
+}
 
 static const char hex_alphabet[] = "0123456789ABCDEF";
 
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
- * `mask' tells us whether we're escaping an URL path or a query string.
+ *
+ * @param `url' no brief description.
+ * @param `mask' tells us whether we're escaping an URL path or a query string.
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
@@ -78,7 +99,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 	gchar *new;
 
 	for (p = url, c = *p++; c; c = *p++)
-		if (!TRANSPARENT_CHAR(c, mask))
+		if (!is_transparent_char(c, mask))
 			need_escape++;
 
 	if (need_escape == 0)
@@ -87,7 +108,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 	new = g_malloc(p - url + (need_escape << 1));
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
-		if (TRANSPARENT_CHAR(c, mask))
+		if (is_transparent_char(c, mask))
 			*q++ = c;
 		else {
 			*q++ = ESCAPE_CHAR;
@@ -117,7 +138,7 @@ url_escape_mask_into(const gchar *url, gchar *target, gint len, guint8 mask)
 	gchar *end = target + len;
 
 	for (q = target, c = *p++; c && q < end; c = *p++) {
-		if (TRANSPARENT_CHAR(c, mask))
+		if (is_transparent_char(c, mask))
 			*q++ = c;
 		else if (end - q >= 3) {
 			*q++ = ESCAPE_CHAR;
@@ -172,6 +193,37 @@ url_escape_into(const gchar *url, gchar *target, gint len)
 	return url_escape_mask_into(url, target, len, PATH_MASK);
 }
 
+/**
+ * Don't touch '?', '&', '=', ':', %HH.
+ */
+gchar *
+url_fix_escape(const gchar *url)
+{
+	const gchar *p;
+	GString *gs;
+	guchar c;
+
+	gs = g_string_new(NULL);
+
+	for (p = url; '\0' != (c = *p); p++) {
+		if (
+			is_transparent_char(c, FIX_MASK) ||
+			('%' == c && is_ascii_xdigit(p[1]) && is_ascii_xdigit(p[2]))
+		) {
+			gs = g_string_append_c(gs, c);
+		} else {
+			gchar buf[3];
+
+			buf[0] = ESCAPE_CHAR;
+			buf[1] = hex_alphabet[c >> 4];
+			buf[2] = hex_alphabet[c & 0xf];
+			gs = g_string_append_len(gs, buf, sizeof buf);
+		}
+	}
+
+	return gm_string_finalize(gs);
+}
+
 /**
  * Escape control characters using %xx, where xx is an hex code.
  *
@@ -262,9 +314,9 @@ url_unescape(gchar *url, gboolean inplace)
 			*q++ = c;
 		else {
 			if ((c = *p++)) {
-				gint v = (hex2dec(c) << 4);
+				gint v = hex2int_inline(c) << 4;
 				if ((c = *p++))
-					v += hex2dec(c);
+					v += hex2int_inline(c);
 				else
 					g_assert_not_reached();	/* Handled in pre-scan above */
 
@@ -301,7 +353,7 @@ url_params_parse(gchar *query)
 	gchar *value = NULL;
 	gboolean in_value = FALSE;
 
-	up = walloc(sizeof(*up));
+	up = walloc(sizeof *up);
 	up->params = g_hash_table_new(g_str_hash, g_str_equal);
 	up->count = 0;
 
@@ -313,8 +365,7 @@ url_params_parse(gchar *query)
 				*q = '\0';
 				value = url_unescape(start, FALSE);
 				if (!value) {
-					if (name)
-						G_FREE_NULL(name);
+					G_FREE_NULL(name);
 					url_params_free(up);
 					return NULL;
 				}
@@ -380,42 +431,38 @@ url_params_free(url_params_t *up)
 	g_hash_table_foreach(up->params, free_params_kv, NULL);
 	g_hash_table_destroy(up->params);
 
-	wfree(up, sizeof(*up));
+	wfree(up, sizeof *up);
 }
 
 static gboolean
-url_safe_char(gint c, url_policy_t p)
+url_safe_char(gchar c, url_policy_t p)
 {
-	if (p & URL_POLICY_GWC_RULES) {
-		/* Reject allow anything unreasonable */
-		if (!isascii(c)) {
-			if (url_debug)
-				g_warning("Non-ASCII character in GWC URL; rejected");
-			return FALSE;
-		}
+	if (!isascii(c) || is_ascii_cntrl(c))
+		return FALSE;
 
-    	if (!isalnum(c)
-        	&& c != '/' && c != '.' && c != '_' && c != '-' && c != '~'
-      	) {
-			if (url_debug)
+	if (!(p & URL_POLICY_ALLOW_ANY_CHAR)) {
+    	if (
+			!is_ascii_lower(c) &&
+			!is_ascii_digit(c) &&
+			NULL == strchr("/._-~", c)
+		) {
+			if (lib_debug)
       			g_warning("Unsafe character in GWC URL; rejected");
       		return FALSE;
     	}
-	} else {
-		/* XXX: Figure out what's the correct set for this */
-		if (!isascii(c) || iscntrl(c)) {
-			return FALSE;
-		}
 	}
+
 	return TRUE;
 }
 
 /**
+ * @attention
  * NB: May modify ``url'' in all cased; pass a copy if necessary!
  *
- * Returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
+ * @returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
  * it returns either a pointer to the original URL or a g_malloc()ed
  * modified copy which has to be g_free()ed.
+ *
  * The URL is validated according to the specified policy. Unnecessary
  * parts such as ":80" (for the port) and "/./" are removed: the hostname
  * is forced to lowercase; a base URI "/" is appended if missing.
@@ -425,77 +472,81 @@ url_safe_char(gint c, url_policy_t p)
 gchar *
 url_normalize(gchar *url, url_policy_t pol)
 {
-	gint c, dots = 0;
-	gchar *endptr;
-	const gchar *tld = NULL;
-	guint16 port = 0;
-	gchar *p, *q = url;
-	const gchar *uri;
 	static const char http_prefix[] = "http://";
-	gchar *warn = NULL;
+	const gchar *p, *uri, *endptr, *tld = NULL, *warn = NULL;
+	gchar c, *q;
+	host_addr_t addr;
 
 	g_assert(url);
 
-	if (NULL == (q = is_strprefix(q, http_prefix))) {
-		if (url_debug)
+	if (NULL == (q = is_strcaseprefix(url, http_prefix))) {
+		if (lib_debug)
 			g_warning("URL \"%s\" isn't preceded by \"%s\"", url, http_prefix);
 		return NULL;
 	}
+	/* Make sure the prefix is all-lowercase */
+	memcpy(url, http_prefix, CONST_STRLEN(http_prefix));
 
 	if (!is_ascii_alnum(*q)) {
 		warn = "HTTP prefix MUST be followed by an alphanum";
 		goto bad;
 	}
 
-	if (gchar_to_ip_strict(q, NULL, (const gchar **) &endptr)) {
+	if (
+		string_to_host_or_addr(q, &endptr, &addr) &&
+		is_host_addr(addr) &&
+		('/' == *endptr || ':' == *endptr || '\0' == *endptr)
+	) {
 		if (!(pol & URL_POLICY_ALLOW_IP_AS_HOST)) {
 			warn = "URLs without hostnames have been disabled";
 			goto bad;
 		}
-		q = endptr;
+		q = deconstify_gchar(endptr);
 
 	} else {
+		size_t dots = 0;
+
 		/* The ``host'' part is not an IP address */
 
 		for (/* NOTHING */; *q != '\0'; q++) {
 
-		    for (/* NOTHING */; (c = (guchar) *q) != '\0'; q++) {
+		    for (/* NOTHING */; '\0' != (c = *q); q++) {
         		if (is_ascii_alnum(c)) {
           			*q = ascii_tolower(c);
-        		} else if (c != '-') {
+        		} else if ('-' != c) {
           			break;
         		}
       		}
 
-			if (c == '\0') {
+			if ('\0' == c) {
 				if (dots < 1 && !(pol & URL_POLICY_ALLOW_LOCAL_HOSTS)) {
 					warn = "current URL policy forbids local hosts";
 					goto bad;
 				}
 				break;
-			} else if (c == '.') {
+			} else if ('.' == c) {
 
-				if (!(isalnum((guchar) *(q - 1)) && isalnum((guchar) q[1]))) {
+				if ( !(is_ascii_alnum(*(q - 1)) && is_ascii_alnum(q[1]))) {
 					warn = "a dot must be preceded and followed by an alphanum";
 					goto bad;
 				}
 				dots++;
-				if (q[1] == '\0' || q[1] == ':' || q[1] == '/')
+				if ('\0' == q[1] || ':' == q[1] || '/' == q[1])
 					break;
 
 				tld = &q[1];
-			} else if (c == '\0' || c == '/' || c == ':') {
+			} else if ('\0' == c || '/' == c || ':' == c) {
 				break;
 			} else {
-				if (url_debug)
+				if (lib_debug)
 					g_warning("Bad URL \"%s\": "
 						"invalid character '%c' in ``host:port'' part.",
-						url, isprint(c) ? c : '?');
+						url, is_ascii_print(c) ? c : '?');
 				return NULL;
 			}
 		}
 
-		if (!tld || !(isalpha((guchar) tld[0]) && isalpha((guchar) tld[1]))) {
+		if (!tld || !(is_ascii_alpha(tld[0]) && is_ascii_alpha(tld[1]))) {
 			warn = "no or invalid top-level domain";
 			goto bad;
 		}
@@ -503,96 +554,84 @@ url_normalize(gchar *url, url_policy_t pol)
 	}
 
 	p = q;
-	if (*q == ':' ) {
-		gulong v = 0;
+	if (':' == *q) {
+		guint32 port;
+		gint error;
 
-		q++;
-		/* strtoul() allows leading spaces, +, - and zeroes */
-		if (NULL != strchr("123456789", *q)) {
-			errno = 0;
-			v = strtoul(q, &endptr, 10);
+		q++; /* Skip ':' */
+
+		/* Reject port numbers with leading zeros */
+		if (!is_ascii_digit(*q) || '0' == *q) {
+			error = EINVAL;
+			port = 0;
+		} else {
+			port = parse_uint32(q, &endptr, 10, &error);
 		}
-		if (v < 1 || v > 65535 || errno) {
+
+		if (error || port < 1 || port > 65535) {
 			warn = "':' MUST be followed a by port value (1-65535)";
 			goto bad;
 		}
-		port = (guint16) v;
+
+		if (
+			!(URL_POLICY_ALLOW_ANY_PORT & pol) &&
+			port < 1024 &&
+			80 != port &&
+			443 != port
+		) {
+			warn = "Ports below 1024 other than 80 and 443 are disallowed";
+			goto bad;
+		}
+
 		p = endptr;
 		if (port == /* HTTP_PORT */ 80) {
 			/* We don't want the default port in a URL;
 			 * this does also prevents duplicates. */
 			q--;
 		} else {
-			q = endptr;
+			q = deconstify_gchar(endptr);
 		}
 	}
 
-	if (*p != '/' && *p != '\0') {
+	if ('/' != *p && '\0' != *p) {
 		warn = "host must be followed by ':', '/' or NUL";
 		goto bad;
 	}
 
-	uri = q;
+	uri = p;
 
 	/* Scan path */
-	for (/* NOTHING */; (c = *(guchar *) p) != '\0'; q++, p++) {
+	for (/* NOTHING */; '\0' != (c = *p); p++) {
 		if (!url_safe_char(c, pol)) {
 			warn = "URL contains characters prohibited by policy";
 			goto bad;
 		}
-
-		/* Handle relative paths i.e., /. and /.. */
-		if (c != '/') {
-				*q = c;
-			continue;
-		}
-
-		/* Special handling for '/' follows */
-		do {
-
-			*q = '/';
-
-			while (p[1] == '/')
-				p++;
-
-			if (0 == strcmp(p, "/.")) {
-				p++;
-				if (url_debug)
-					g_message("ignoring trailing \"/.\" in URI");
-			} else if (0 == strcmp(p, "/..")) {
-				if (url_debug)
-					g_message("trailing \"/..\" in URI; rejected");
-				return NULL;
-			} else if (is_strprefix(p, "/./")) {
-				p += 2;
-				if (url_debug)
-					g_message("ignoring unnecessary \"/./\" in URI");
-			} else if (is_strprefix(p, "/../")) {
-				p += 3;
-				if (url_debug)
-					g_message("ascending one component in URI");
-				while (*--q != '/')
-					if (q <= uri) {
-						warn = "URI ascends beyond root per \"/../\"";
-						goto bad;
-					}
-			} else {
-				break;
-			}
-
-		} while (*p == '/' && (p[1] == '/' || p[1] == '.'));
-
 	}
-	*q = '\0';
 
-	if (pol & URL_POLICY_GWC_RULES) {
+	if (q != uri) {
+		size_t len;
+
+		len = strlen(uri);
+		memmove(q, uri, len);
+		q[len] = '\0';
+	}
+	uri = q;
+
+	if (0 != canonize_path(q, q)) {
+		warn = "Could not canonize URI";
+		goto bad;
+	}
+
+	if (!(URL_POLICY_ALLOW_STATIC_FILES & pol)) {
 		static const struct {
-			ssize_t len;
 			const gchar *ext;
+			size_t len;
 		} static_types[] = {
-			{ 5, ".html" },
-			{ 4, ".htm" },
-			{ 4, ".txt" }
+#define D(x) { (x), CONST_STRLEN(x) }
+			D(".html"),
+			D(".htm"),
+			D(".txt"),
+#undef D
 		};
 		guint i;
 
@@ -607,26 +646,25 @@ url_normalize(gchar *url, url_policy_t pol)
 	}
 
 	/* Add a trailing slash; if the URI is empty (to prevent dupes) */
-	if (*uri == '\0') {
+	if ('\0' == uri[0]) {
 		ssize_t len = q - url;
+		gchar *s;
 
 		g_assert(len > 0);
-		p = g_malloc(len + sizeof "/");
-		if (p) {
-			memcpy(p, url, len);
-			p[len] = '/';
-			p[len + 1] = '\0';
-		}
-		url = p;
+		s = g_malloc(len + sizeof "/");
+		memcpy(s, url, len);
+		s[len] = '/';
+		s[len + 1] = '\0';
+		url = s;
 	}
 
-	if (url_debug)
+	if (lib_debug > 1)
 		g_message("url=\"%s\"", url);
 
 	return url;
 
 bad:
-	if (url_debug)
+	if (lib_debug)
 		g_warning("invalid URL \"%s\": %s", url, warn);
 	return NULL;
 }
