@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.c,v 1.22 2005/12/19 11:19:24 cbiere Exp $
+ * $Id: fileinfo.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2003, Richard Eckart
  *
@@ -35,13 +35,14 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c,v 1.22 2005/12/19 11:19:24 cbiere Exp $");
+RCSID("$Id: fileinfo.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/filter.h"
 #include "gtk/statusbar.h"
 #include "gtk/visual_progress.h"
 #include "gtk/columns.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
@@ -135,10 +136,11 @@ drag_data_get(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
 
 	g_assert(url_ptr != NULL);
 	if (*url_ptr) {
-		const guchar *drag_data = *url_ptr;
+		const gchar *drag_data = *url_ptr;
 		
     	gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
-			8 /* CHAR_BIT */, drag_data, strlen(drag_data));
+			8 /* CHAR_BIT */, cast_to_gconstpointer(drag_data),
+			strlen(drag_data));
 		G_FREE_NULL(*url_ptr);
 	}
 }
@@ -209,18 +211,22 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
     guc_fi_get_status(fih, &s);
 
     gm_snprintf(fi_sources, sizeof(fi_sources), "%d/%d/%d",
-        s.recvcount, s.aqueued_count+s.pqueued_count, s.lifecount);
+        s.recvcount, s.aqueued_count + s.pqueued_count, s.lifecount);
     titles[c_fi_sources] = fi_sources;
 
     if (s.done) {
+		gdouble d;
+
+		d = s.size > 0 ? ((gdouble) s.done / s.size) * 100.0 : 100.0;
         gm_snprintf(fi_done, sizeof(fi_done), "%s (%.1f%%)",
-            short_size(s.done), ((float) s.done / s.size)*100.0);
+            short_size(s.done, show_metric_units()), d);
         titles[c_fi_done] = fi_done;
     } else {
         titles[c_fi_done] = "-";
     }
 
-    g_strlcpy(fi_size, short_size(s.size), sizeof(fi_size));
+    g_strlcpy(fi_size, short_size(s.size, show_metric_units()),
+		sizeof(fi_size));
     titles[c_fi_size]    = fi_size;
 
     if (s.recvcount) {
@@ -231,7 +237,7 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 
         gm_snprintf(fi_status, sizeof(fi_status),
             _("Downloading (%s)  TR: %s"),
-			short_rate(s.recv_last_rate),
+			short_rate(s.recv_last_rate, show_metric_units()),
 			secs ? short_time(secs) : "-");
 
         titles[c_fi_status] = fi_status;
@@ -251,15 +257,15 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 			else
 				rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
 						"; %s %s (%.1f%%)", _("Computing SHA1"),
-						short_size(s.sha1_hashed),
-						((float) s.sha1_hashed / s.size) * 100.0);
+						short_size(s.sha1_hashed, show_metric_units()),
+						((gdouble) s.sha1_hashed / s.size) * 100.0);
 		}
 
 		if (s.copied > 0 && s.copied < s.size) 
 			rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
 					"; %s %s (%.1f%%)", _("Moving"),
-					short_size(s.copied),
-					((float) s.copied / s.size) * 100.0);
+					short_size(s.copied, show_metric_units()),
+					((gdouble) s.copied / s.size) * 100.0);
 
         titles[c_fi_status] = fi_status;
     } else if (s.lifecount == 0) {
@@ -282,7 +288,7 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 static void
 fi_gui_set_details(gnet_fi_t fih)
 {
-    gnet_fi_info_t *fi = NULL;
+    gnet_fi_info_t *fi;
     gnet_fi_status_t fis;
     gchar **aliases;
     guint n;
@@ -295,15 +301,20 @@ fi_gui_set_details(gnet_fi_t fih)
     guc_fi_get_status(fih, &fis);
     aliases = guc_fi_get_aliases(fih);
 
-    cl_aliases = GTK_CLIST(lookup_widget(main_window, "clist_fi_aliases"));
+    cl_aliases = GTK_CLIST(gui_main_window_lookup("clist_fi_aliases"));
 
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_filename")),
+    gtk_entry_printf(
+		GTK_ENTRY(gui_main_window_lookup("entry_fi_sha1")),
+		"%s%s",
+		fi->sha1 ? "urn:sha1:" : _("<none>"),
+		fi->sha1 ? sha1_base32(fi->sha1) : "");
+    gtk_entry_set_text(
+        GTK_ENTRY(gui_main_window_lookup("entry_fi_filename")),
         fi->file_name);
 	uint64_to_string_buf(fis.size, bytes, sizeof bytes);
-    gtk_label_printf(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_size")),
-        _("%s (%s bytes)"), short_size(fis.size), bytes);
+    gtk_entry_printf(
+        GTK_ENTRY(gui_main_window_lookup("entry_fi_size")),
+        _("%s (%s bytes)"), short_size(fis.size, show_metric_units()), bytes);
 
     gtk_clist_freeze(cl_aliases);
     gtk_clist_clear(cl_aliases);
@@ -319,7 +330,7 @@ fi_gui_set_details(gnet_fi_t fih)
 
 	vp_draw_fi_progress(last_shown_valid, last_shown);
 
-	gtk_widget_set_sensitive(lookup_widget(main_window, "button_fi_purge"),
+	gtk_widget_set_sensitive(gui_main_window_lookup("button_fi_purge"),
 							 TRUE);
 }
 
@@ -329,17 +340,23 @@ fi_gui_set_details(gnet_fi_t fih)
 static void
 fi_gui_clear_details(void)
 {
+	static const gchar *widgets[] = {
+		"entry_fi_filename",
+		"entry_fi_sha1",
+		"entry_fi_size",
+	};
+	guint i;
+		
     last_shown_valid = FALSE;
 
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_filename")),
-        "");
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_size")),
-        "");
+	for (i = 0; i < G_N_ELEMENTS(widgets); i++) {
+		GtkEntry *entry = GTK_ENTRY(gui_main_window_lookup(widgets[i]));
+		gtk_entry_set_text(entry, "");
+	}
+
     gtk_clist_clear(
-        GTK_CLIST(lookup_widget(main_window, "clist_fi_aliases")));
-    gtk_widget_set_sensitive(lookup_widget(main_window, "button_fi_purge"),
+        GTK_CLIST(gui_main_window_lookup("clist_fi_aliases")));
+    gtk_widget_set_sensitive(gui_main_window_lookup("button_fi_purge"),
         FALSE);
 
     vp_draw_fi_progress(last_shown_valid, last_shown);
@@ -669,12 +686,12 @@ fi_gui_init(void)
     guc_fi_add_listener(fi_gui_fi_status_changed_transient,
 		EV_FI_STATUS_CHANGED_TRANSIENT, FREQ_SECS, 0);
 
-    clist_fileinfo = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
+    clist_fileinfo = GTK_CLIST(gui_main_window_lookup("clist_fileinfo"));
 
     gtk_clist_set_column_justification(clist_fileinfo,
         c_fi_size, GTK_JUSTIFY_RIGHT);
 
-    gtk_clist_column_titles_passive(clist_fileinfo);
+    gtk_clist_column_titles_active(clist_fileinfo);
 
     /* Initialize the row filter */
     fi_gui_set_filter_regex(NULL);
@@ -756,4 +773,171 @@ fi_gui_update_display(time_t now)
 #endif
 }
 
+static inline guint
+fi_gui_relative_done(const gnet_fi_status_t *s, guint base)
+{
+	filesize_t x;
+
+	/**
+	 * Use integer arithmetic because float or double might be too small
+	 * for 64-bit values.
+	 */
+	if (s->size == s->done) {
+		return base;
+	}
+	if (s->size > base) {
+		x = s->size / base;
+		x = s->done / MAX(1, x);
+	} else {
+		x = (s->done * base) / MAX(1, s->size);
+	}
+	base--;
+	return MIN(x, base);
+}
+
+static inline guint
+fi_gui_numeric_status(const gnet_fi_status_t *s)
+{
+	guint v;
+
+	v = fi_gui_relative_done(s, 100);
+	v |= (s->lifecount > 0)						? (1 <<  7) : 0;
+	v |= (s->aqueued_count || s->pqueued_count)	? (1 <<  8) : 0;
+	v |= (s->recvcount > 0)						? (1 <<  9) : 0;
+	v |= (s->size > 0 && s->size == s->done)	? (1 << 10) : 0;
+
+	return v;
+}
+
+static gint 
+fi_gui_cmp_filename(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_info_t *a_fi, *b_fi;
+    gnet_fi_t a, b;
+	gint r;
+   
+	(void) unused_clist;
+	a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+	a_fi = guc_fi_get_info(a);
+	b_fi = guc_fi_get_info(b);
+	r = strcmp(a_fi->file_name, b_fi->file_name);
+	guc_fi_free_info(b_fi);
+	guc_fi_free_info(a_fi);
+
+	return r;
+}
+
+static gint 
+fi_gui_cmp_size(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	return CMP(a.size, b.size);
+}
+
+static gint 
+fi_gui_cmp_done(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+	gint ret;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	ret = CMP(fi_gui_relative_done(&a, 1000), fi_gui_relative_done(&b, 1000));
+	return 0 == ret ? CMP(a.done, b.done) : ret;
+}
+
+static gint 
+fi_gui_cmp_sources(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+	gint r;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	r = CMP(a.recvcount, b.recvcount);
+	if (0 == r) {
+		r = CMP(a.aqueued_count + a.pqueued_count,
+				b.aqueued_count + b.pqueued_count);
+		if (0 == r) {
+			r = CMP(a.lifecount, b.lifecount);
+		}
+	}
+	return r;
+}
+
+static gint 
+fi_gui_cmp_status(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	return CMP(fi_gui_numeric_status(&a), fi_gui_numeric_status(&b));
+}
+
+void
+on_clist_fileinfo_click_column(GtkCList *clist, gint column,
+	gpointer unused_udata)
+{
+	static gint sort_col = c_fi_num;
+	static gboolean sort_invert;
+
+	(void) unused_udata;
+	
+	g_assert(column >= 0 && column < c_fi_num);
+
+	switch ((enum c_fi) column) {
+#define CASE(x) case c_fi_ ## x : \
+		gtk_clist_set_compare_func(clist, fi_gui_cmp_ ## x ); break;
+	CASE(filename)
+	CASE(size)
+	CASE(done)
+	CASE(sources)
+	CASE(status)
+#undef CASE
+	case c_fi_num:
+		g_assert_not_reached();
+	}
+
+	sort_invert = sort_col == column && !sort_invert;
+	sort_col = column;
+	gtk_clist_set_sort_type(clist,
+		sort_invert ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
+	gtk_clist_sort(clist);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
