@@ -1,5 +1,5 @@
 /*
- * $Id: monitor.c,v 1.10 2005/07/26 21:20:16 cbiere Exp $
+ * $Id: monitor.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: monitor.c,v 1.10 2005/07/26 21:20:16 cbiere Exp $");
+RCSID("$Id: monitor.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/monitor.h"
 #include "gtk/monitor_cb.h"
@@ -48,7 +48,6 @@ RCSID("$Id: monitor.c,v 1.10 2005/07/26 21:20:16 cbiere Exp $");
 #include "lib/glib-missing.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-static guint32 monitor_items = 0;
 static GtkListStore *monitor_model = NULL;
 
 enum {
@@ -63,44 +62,56 @@ enum {
  ***/
 
 static void
-monitor_gui_append(query_type_t type, const gchar *item,
-	guint32 ip, guint16 port)
+monitor_gui_add(query_type_t type, const gchar *item,
+	const host_addr_t addr, guint16 port)
 {
-	(void) ip;
+	gint n;
+
+	(void) addr;
 	(void) port;
 
 	/* The user might have changed the max. number of items to
 	 * show, that's why we don't just the remove the first item. */
-	for (/* empty */; monitor_items >= monitor_max_items; monitor_items--) {
+	n = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(monitor_model), NULL);
+
+	if (n > 0 && (guint) n >= monitor_max_items) {
 		GtkTreeIter iter;
+		gboolean ok;
 
-        /* Get the first iter in the list */
-        if (
-			!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(monitor_model),
-					&iter)
-			) {
-				break;
-			}
-
-		gtk_list_store_remove(monitor_model, &iter);
-    }
+		/* Children are enumerated from 0 to number-1 */
+		ok = gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(monitor_model),
+					&iter, NULL, MAX(1, monitor_max_items) - 1);
+		while (ok) {
+			GtkTreeIter next = iter;
+			ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(monitor_model), &next);
+			gtk_list_store_remove(monitor_model, &iter);
+			iter = next;
+		}
+	}
 
 	if (monitor_max_items > 0) {
+		const gchar *charset_ptr, *s;
+		gchar *dbuf;
 		GtkTreeIter iter;
-		gchar buf[128];
-		const gchar *s;
+		gchar buf[1024];
 
     	/* Aquire an iterator */
-    	gtk_list_store_append(monitor_model, &iter);
-    	monitor_items++;
+    	gtk_list_store_prepend(monitor_model, &iter);
 
 		/* If the query is empty and we have a SHA1 extension,
 	 	 * we print a urn:sha1-query instead. */
 		concat_strings(buf, sizeof buf,
 			QUERY_SHA1 == type ? "urn:sha1:" : "", item, (void *) 0);
 
-		s = lazy_locale_to_utf8(buf);
-   		gtk_list_store_set(monitor_model, &iter, QUERY_COLUMN, s, (-1));
+		s = lazy_unknown_to_utf8_normalized(buf, UNI_NORM_GUI, &charset_ptr);
+		if (s != buf) {
+			dbuf = g_strconcat("<", charset_ptr, "> ", s, (void *) 0);
+			s = dbuf;
+		} else {
+			dbuf = NULL;
+		}
+		gtk_list_store_set(monitor_model, &iter, QUERY_COLUMN, s, (-1));
+		G_FREE_NULL(dbuf);
 	}
 }
 
@@ -120,10 +131,9 @@ monitor_gui_init(void)
     monitor_model = gtk_list_store_new(MONITOR_COLUMNS, G_TYPE_STRING);
 
     /* Get the monitor widget */
-    tree = lookup_widget(main_window, "treeview_monitor");
+    tree = gui_main_window_lookup("treeview_monitor");
 
-    gtk_tree_view_set_model
-        (GTK_TREE_VIEW(tree), GTK_TREE_MODEL(monitor_model));
+    gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(monitor_model));
 
     /* The view now holds a reference.  We can get rid of our own
      * reference */
@@ -138,7 +148,7 @@ monitor_gui_init(void)
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 
     /* Add the column to the view. */
-    gtk_tree_view_append_column (GTK_TREE_VIEW(tree), column);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	g_signal_connect(G_OBJECT(tree), "button_press_event",
 		G_CALLBACK(on_treeview_monitor_button_press_event), NULL);
@@ -158,22 +168,22 @@ void
 share_gui_clear_monitor(void)
 {
     gtk_list_store_clear(monitor_model);
-	monitor_items = 0;
 }
 #endif
 
 /**
  * Enable/disable monitor.
  */
-void monitor_gui_enable_monitor(const gboolean val)
+void
+monitor_gui_enable_monitor(const gboolean val)
 {
     static gboolean registered = FALSE;
 
     if (val != registered) {
         if (val)
-            guc_share_add_search_request_listener(monitor_gui_append);
+            guc_share_add_search_request_listener(monitor_gui_add);
         else
-            guc_share_remove_search_request_listener(monitor_gui_append);
+            guc_share_remove_search_request_listener(monitor_gui_add);
         registered = val;
     }
 }
