@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c,v 1.56 2005/12/27 13:05:51 cbiere Exp $
+ * $Id: nodes.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: nodes.c,v 1.56 2005/12/27 13:05:51 cbiere Exp $");
+RCSID("$Id: nodes.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "interface-glade.h"
 
@@ -178,17 +178,20 @@ create_nodes_model(void)
 }
 
 static void
-cell_renderer_func(GtkTreeViewColumn *unused_column,
+cell_renderer_func(GtkTreeViewColumn *column,
 	GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter,
 	gpointer udata)
 {
 	static const GValue zero_value;
-	GValue value = zero_value;
 	const struct node_data *data;
-	const gchar *s, *attr = "text";
+	const gchar *s, *attr;
+	GValue value;
 
-	(void) unused_column;
+	if (!gtk_tree_view_column_get_visible(column))
+		return;
 
+	attr = "text";
+	value = zero_value;
 	gtk_tree_model_get_value(model, iter, 0, &value);
 	data = g_value_get_pointer(&value);
 	switch (GPOINTER_TO_UINT(udata)) {
@@ -221,12 +224,15 @@ cell_renderer_func(GtkTreeViewColumn *unused_column,
 		s = NULL;
 	}
 
-	if (data->fg)
+	if (data->fg) {
 		g_object_set(cell,
+			attr, s,
 			"foreground-gdk", data->fg,
 			"foreground-set", TRUE,
 			(void *) 0);
-	g_object_set(cell, attr, s, (void *) 0);
+	} else {
+		g_object_set(cell, attr, s, (void *) 0);
+	}
 }
 
 /**
@@ -264,7 +270,7 @@ nodes_gui_create_treeview_nodes(void)
     /*
      * Get the monitor widget
      */
-	tree = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_nodes"));
+	tree = GTK_TREE_VIEW(gui_main_window_lookup("treeview_nodes"));
 	treeview_nodes = tree;
 	gtk_tree_view_set_model(tree, GTK_TREE_MODEL(nodes_model));
 	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(tree),
@@ -317,8 +323,7 @@ nodes_gui_update_node_info(struct node_data *data, gnet_node_info_t *info)
     guc_node_get_status(info->node_handle, &status);
     gm_snprintf(data->version, sizeof data->version, "%u.%u",
 		info->proto_major, info->proto_minor);
-	if (data->user_agent)
-		atom_str_free(data->user_agent);
+	atom_str_free_null(&data->user_agent);
 	data->user_agent = info->vendor ? atom_str_get(info->vendor) : NULL;
 	data->country = info->country;
 }
@@ -334,35 +339,13 @@ nodes_gui_update_node_flags(struct node_data *data, gnet_node_flags_t *flags)
 	g_assert(NULL != data);
 
 	concat_strings(data->flags, sizeof data->flags,
-		"<tt>", nodes_gui_common_flags_str(flags), "</tt>", (void *) 0);
+		"<tt>", guc_node_flags_to_string(flags), "</tt>", (void *) 0);
 
 	ultra = NODE_P_ULTRA == flags->peermode;
     data->fg = &(gtk_widget_get_style(GTK_WIDGET(treeview_nodes))
 					->fg[ultra ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
 
-static const gchar *
-peermode_to_string(node_peer_t m)
-{
-	switch (m) {
-	case NODE_P_LEAF:
-		return _("Leaf");
-	case NODE_P_ULTRA:
-		return _("Ultrapeer");
-	case NODE_P_NORMAL:
-		return _("Legacy");
-	case NODE_P_CRAWLER:
-		return _("Crawler");
-	case NODE_P_UDP:
-		return _("UDP");
-	case NODE_P_AUTO:
-	case NODE_P_UNKNOWN:
-		break;
-	}
-
-	return _("Unknown");
-}
-
 static void
 update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 {
@@ -411,9 +394,9 @@ update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 			"%s %s (%s)\n"
 			"%s %.64s",
 			_("Peer:"),
-			host_addr_port_to_string(info.addr, info.port),
+			host_addr_port_to_string(info.gnet_addr, info.gnet_port),
 			_("Peermode:"),
-			peermode_to_string(flags.peermode),
+			guc_node_peermode_to_string(flags.peermode),
 			flags.incoming ? _("incoming") : _("outgoing"),
 			_("Country:"),
 			iso3166_country_name(info.country),
@@ -457,7 +440,7 @@ host_lookup_callback(const gchar *hostname, gpointer key)
 
 	g_hash_table_remove(ht_pending_lookups, key);
 
-	tv = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_nodes"));
+	tv = GTK_TREE_VIEW(gui_main_window_lookup("treeview_nodes"));
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(tv));
 	data = find_node(n);
 	if (!data)
@@ -526,7 +509,6 @@ on_cursor_changed(GtkTreeView *tv, gpointer unused_udata)
 void
 nodes_gui_early_init(void)
 {
-	popup_nodes = create_popup_nodes();
 	nodes_gui_create_treeview_nodes();
 }
 
@@ -536,8 +518,7 @@ nodes_gui_early_init(void)
 void
 nodes_gui_init(void)
 {
-	treeview_nodes = GTK_TREE_VIEW(lookup_widget(
-		main_window, "treeview_nodes"));
+	treeview_nodes = GTK_TREE_VIEW(gui_main_window_lookup( "treeview_nodes"));
 
 	tree_view_restore_widths(treeview_nodes, PROP_NODES_COL_WIDTHS);
 	tree_view_restore_visibility(treeview_nodes, PROP_NODES_COL_VISIBLE);
@@ -762,7 +743,7 @@ nodes_gui_update_nodes_display(time_t now)
 	 */
 
 	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
+		notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 
     current_page = gtk_notebook_get_current_page(notebook);
 	if (
@@ -862,7 +843,7 @@ nodes_gui_remove_selected(void)
 	GtkTreeSelection *selection;
 	GSList *node_list = NULL;
 
-	treeview = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_nodes"));
+	treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_nodes"));
 	selection = gtk_tree_view_get_selection(treeview);
 	gtk_tree_selection_selected_foreach(selection,
 		nodes_gui_remove_selected_helper, &node_list);
@@ -913,7 +894,7 @@ nodes_gui_reverse_lookup_selected(void)
 	GtkTreeView *tv;
 	GtkTreeSelection *selection;
 
-	tv = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_nodes"));
+	tv = GTK_TREE_VIEW(gui_main_window_lookup("treeview_nodes"));
 	selection = gtk_tree_view_get_selection(tv);
 	gtk_tree_selection_selected_foreach(selection,
 		nodes_gui_reverse_lookup_selected_helper, NULL);
@@ -933,7 +914,7 @@ nodes_gui_browse_selected_helper(GtkTreeModel *model,
 	info = guc_node_get_info(data->handle);
 	if (!info->is_pseudo) {
 		search_gui_new_browse_host(NULL, info->gnet_addr, info->gnet_port,
-			info->gnet_guid, FALSE, NULL);
+			info->gnet_guid, NULL, 0);
 	}
 	guc_node_free_info(info);
 }
@@ -944,7 +925,7 @@ nodes_gui_browse_selected(void)
 	GtkTreeView *tv;
 	GtkTreeSelection *selection;
 
-	tv = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_nodes"));
+	tv = GTK_TREE_VIEW(gui_main_window_lookup("treeview_nodes"));
 	selection = gtk_tree_view_get_selection(tv);
 	gtk_tree_selection_selected_foreach(selection,
 		nodes_gui_browse_selected_helper, NULL);
