@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c,v 1.32 2005/12/11 13:52:52 rmanfredi Exp $
+ * $Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -46,6 +46,7 @@
 #include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
 #include "gtk/notebooks.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
@@ -59,7 +60,7 @@
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: uploads.c,v 1.32 2005/12/11 13:52:52 rmanfredi Exp $");
+RCSID("$Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #define UPDATE_MIN	300		/**< Update screen every 5 minutes at least */
 
@@ -135,7 +136,7 @@ on_button_press_event(GtkWidget *unused_widget, GdkEventButton *event,
 
 	if (3 == event->button) {
         /* Right click section (popup menu) */
-		gtk_menu_popup(GTK_MENU(popup_uploads), NULL, NULL, NULL, NULL,
+		gtk_menu_popup(GTK_MENU(gui_popup_uploads()), NULL, NULL, NULL, NULL,
 			event->button, event->time);
 		return TRUE;
     }
@@ -264,8 +265,15 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 
 	rd->last_update  = tm_time();
 
-	if (!host_addr_equal(u->addr, rd->addr)) {
+	if (
+		!host_addr_equal(u->addr, rd->addr) ||
+		!host_addr_equal(u->gnet_addr, rd->gnet_addr) ||
+		u->gnet_port != rd->gnet_port
+	) {
 		rd->addr = u->addr;
+		rd->gnet_addr = u->gnet_addr;
+		rd->gnet_port = u->gnet_port;
+
 		gtk_list_store_set(store_uploads, &rd->iter,
 			c_ul_host, uploads_gui_host_string(u), (-1));
 	}
@@ -280,14 +288,16 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 			g_strlcpy(str, "...", sizeof str);
 		else {
 			range_len = gm_snprintf(str, sizeof str, "%s%s",
-				u->partial ? "*" : "",
-				short_size(u->range_end - u->range_start + 1));
+				short_size(u->range_end - u->range_start + 1,
+					show_metric_units()),
+				u->partial ? _(" (partial)") : "");
 
 			if ((guint) range_len < sizeof str) {
 				if (u->range_start)
 					range_len += gm_snprintf(&str[range_len],
 									sizeof str - range_len,
-									" @ %s", short_size(u->range_start));
+									" @ %s", short_size(u->range_start,
+												show_metric_units()));
 				g_assert((guint) range_len < sizeof str);
 			}
 		}
@@ -298,7 +308,7 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	if (u->file_size != rd->size) {
 		rd->size = u->file_size;
 		gtk_list_store_set(store_uploads, &rd->iter,
-			c_ul_size, short_size(rd->size),
+			c_ul_size, short_size(rd->size, show_metric_units()),
 			(-1));
 	}
 
@@ -376,8 +386,8 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 						? atom_str_get(u->user_agent) : NULL;
 	rd->push		= u->push;
 	rd->valid		= TRUE;
-    rd->gnet_addr   = zero_host_addr;
-    rd->gnet_port   = 0;
+    rd->gnet_addr   = u->gnet_addr;
+    rd->gnet_port   = u->gnet_port;
 
 	guc_upload_get_status(u->upload_handle, &status);
     rd->status = status.status;
@@ -388,20 +398,22 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 		static gchar range_tmp[256];	/* MUST be static! */
 
         range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
-			u->partial ? "*" : "",
-            short_size(u->range_end - u->range_start + 1));
+            short_size(u->range_end - u->range_start + 1,
+				show_metric_units()),
+			u->partial ? _(" (partial)") : "");
 
         if (u->range_start)
             range_len += gm_snprintf(
                 &range_tmp[range_len], sizeof range_tmp - range_len,
-                " @ %s", short_size(u->range_start));
+                " @ %s", short_size(u->range_start, show_metric_units()));
 
         g_assert((guint) range_len < sizeof range_tmp);
 
         titles[c_ul_range] = range_tmp;
     }
 
-	g_strlcpy(size_tmp, short_size(u->file_size), sizeof size_tmp);
+	g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
+		sizeof size_tmp);
     titles[c_ul_size] = size_tmp;
 
    	titles[c_ul_agent] = u->user_agent ? u->user_agent : "...";
@@ -521,7 +533,6 @@ create_uploads_model(void)
 void
 uploads_gui_early_init(void)
 {
-    popup_uploads = create_popup_uploads();
 }
 
 void
@@ -540,17 +551,16 @@ uploads_gui_init(void)
 		{ c_ul_progress, 	NULL },
 		{ c_ul_status, 		NULL },
 	};
-	GtkTreeView *treeview;
 	size_t i;
 
 	STATIC_ASSERT(G_N_ELEMENTS(cols) == UPLOADS_GUI_VISIBLE_COLUMNS);
 	store_uploads = create_uploads_model();
 
-	button_uploads_clear_completed = lookup_widget(main_window,
-		"button_uploads_clear_completed");
-	treeview_uploads = treeview =
-		GTK_TREE_VIEW(lookup_widget(main_window, "treeview_uploads"));
-	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store_uploads));
+	button_uploads_clear_completed =
+		gui_main_window_lookup("button_uploads_clear_completed");
+	treeview_uploads =
+		GTK_TREE_VIEW(gui_main_window_lookup("treeview_uploads"));
+	gtk_tree_view_set_model(treeview_uploads, GTK_TREE_MODEL(store_uploads));
 
 	for (i = 0; i < G_N_ELEMENTS(cols); i++)
 		add_column(cols[i].id, cols[i].sortfunc,
@@ -568,15 +578,13 @@ uploads_gui_init(void)
     guc_upload_add_upload_info_changed_listener
 		(upload_info_changed);
 
-	g_signal_connect(GTK_OBJECT(treeview), "button_press_event",
+	g_signal_connect(GTK_OBJECT(treeview_uploads), "button_press_event",
 		G_CALLBACK(on_button_press_event), NULL);
 }
 
 static inline void
-free_row_data(upload_row_data_t *rd, gpointer user_data)
+free_row_data(upload_row_data_t *rd)
 {
-	(void) user_data;
-
 	if (NULL != rd->user_agent) {
 		atom_str_free(rd->user_agent);
 		rd->user_agent = NULL;
@@ -595,7 +603,7 @@ free_handle(gpointer key, gpointer value,
 	(void) key;
 	(void) user_data;
 
-	free_row_data(value, NULL);
+	free_row_data(value);
 }
 
 static inline void
@@ -604,7 +612,7 @@ remove_row(upload_row_data_t *rd, remove_row_ctx_t *ctx)
 	g_assert(NULL != rd);
 	if (ctx->force || upload_should_remove(ctx->now, rd)) {
     	gtk_list_store_remove(store_uploads, &rd->iter);
-		free_row_data(rd, NULL);
+		free_row_data(rd);
 	} else
 		ctx->sl_remaining = g_slist_prepend(ctx->sl_remaining, rd);
 }
@@ -650,7 +658,7 @@ uploads_gui_update_display(time_t now)
 	 */
 
 	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
+		notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 
 	current_page = gtk_notebook_get_current_page(notebook);
 	if (
@@ -673,7 +681,7 @@ uploads_gui_update_display(time_t now)
 		locked = TRUE;
 
 		/* Remove all rows with `removed' uploads. */
-		G_SLIST_FOREACH(sl_removed_uploads, remove_row, &ctx);
+		G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
 		g_slist_free(sl_removed_uploads);
 		sl_removed_uploads = ctx.sl_remaining;
 
@@ -701,7 +709,7 @@ uploads_clear_helper(gpointer user_data)
 
 	/* Remove all rows with `removed' uploads. */
 
-	G_SLIST_FOREACH(sl_removed_uploads, remove_row, &ctx);
+	G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
 	g_slist_free(sl_removed_uploads);
 	sl_removed_uploads = ctx.sl_remaining;
 
@@ -757,7 +765,7 @@ uploads_gui_shutdown(void)
 	g_hash_table_foreach(upload_handles, free_handle, NULL);
 	g_hash_table_destroy(upload_handles);
 	upload_handles = NULL;
-	G_SLIST_FOREACH(sl_removed_uploads, free_row_data, NULL);
+	G_SLIST_FOREACH(sl_removed_uploads, free_row_data);
 	g_slist_free(sl_removed_uploads);
 	sl_removed_uploads = NULL;
 }
