@@ -1,5 +1,5 @@
 /*
- * $Id: gui.h,v 1.6 2005/06/29 14:24:28 daichik Exp $
+ * $Id: gui.h 11913 2006-09-08 04:28:59Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -28,6 +28,44 @@
 
 #include "common.h"
 
+#ifdef USE_TOPLESS
+
+/* Diverse dummy definitions */
+#define settings_gui_save_if_dirty()
+
+#define main_gui_early_init(argc, argv)
+#define main_gui_init()
+#define main_gui_timer(x)
+#define main_gui_update_coords()
+#define main_gui_shutdown()
+#define main_gui_shutdown_tick(remain)
+#define settings_gui_shutdown()
+
+#define drop_init()
+#define drop_close()
+
+#define search_gui_store_searches()
+
+#define icon_timer()
+
+static inline void
+main_gui_run(const gchar *unused_geometry_spec)
+{
+	GMainLoop *ml;
+
+	(void) unused_geometry_spec;
+
+#if defined(USE_GLIB1)
+	ml = g_main_new(FALSE);
+	g_main_run(ml);
+#elif defined(USE_GLIB2)
+	ml = g_main_loop_new(NULL, FALSE);
+	g_main_loop_run(ml);
+#endif /* GLIB */
+}
+
+#else	/* !USE_TOPLESS */
+
 #include <gtk/gtk.h>
 
 #ifdef USE_GTK1
@@ -59,6 +97,7 @@ enum sorting_order {
 	SORT_NO_COL = 2		/**< No column chosen yet */
 };
 
+#endif	/* USE_TOPLESS */
 #endif /* _gtk_ui_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
