@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_common.c,v 1.13 2005/06/29 14:24:28 daichik Exp $
+ * $Id: nodes_common.c 11755 2006-08-26 21:08:47Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,18 +36,21 @@
 
 #include "gui.h"
 
-RCSID("$Id: nodes_common.c,v 1.13 2005/06/29 14:24:28 daichik Exp $");
+RCSID("$Id: nodes_common.c 11755 2006-08-26 21:08:47Z cbiere $")
 
 #include "nodes_common.h"
+#include "settings.h"
 
 #include "gtk/statusbar.h"
 
 #include "if/bridge/ui2c.h"
 #include "if/core/nodes.h"
+#include "if/core/sockets.h"
 #include "if/gui_property_priv.h"
 
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
 static gchar gui_tmp[4096];
@@ -105,13 +108,14 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 				if (show_gnet_info_tx_wire) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_size(n->tx_written));
+					"%s", compact_size(n->tx_written, show_metric_units()));
 					is_first = FALSE;
 				}
 
 				if (show_gnet_info_tx_speed)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s%s", is_first ? "" : ", ", compact_rate(n->tx_bps));
+						"%s%s", is_first ? "" : ", ",
+						compact_rate(n->tx_bps, show_metric_units()));
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ ")");
@@ -133,13 +137,14 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 				if (show_gnet_info_rx_wire) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_size(n->rx_given));
+						"%s", compact_size(n->rx_given, show_metric_units()));
 					is_first = FALSE;
 				}
 
 				if (show_gnet_info_rx_speed)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s%s", is_first ? "" : ", ", compact_rate(n->rx_bps));
+						"%s%s", is_first ? "" : ", ",
+						compact_rate(n->rx_bps, show_metric_units()));
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ ")");
@@ -223,8 +228,9 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 				if (show_gnet_info_shared_size && n->gnet_info_known) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_kb_size(
-							n->gnet_files_count ? n->gnet_kbytes_count : 0));
+						"%s",
+						compact_kb_size(n->gnet_files_count
+							? n->gnet_kbytes_count : 0, show_metric_units()));
 					is_first = FALSE;
 				}
 				if (show_gnet_info_shared_files && n->gnet_info_known)
@@ -244,15 +250,17 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 				if (n->qrt_slots != 0)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						" QRT(%s, g=%d, f=%d%%, t=%d%%, e=%d%%)",
-						compact_size(n->qrt_slots), n->qrt_generation,
+						compact_size(n->qrt_slots, show_metric_units()),
+						n->qrt_generation,
 						n->qrt_fill_ratio, n->qrt_pass_throw,
 						(guint) (n->qrp_efficiency * 100.0));
 			}
 
 			if (show_gnet_info_dbw)
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-				" Dup=%u Bad=%u W=%u H=%u",
-				n->n_dups, n->n_bad, n->n_weird, n->n_hostile);
+				" Dup=%u Bad=%u W=%u H=%u S=%u E=%u",
+				n->n_dups, n->n_bad, n->n_weird,
+				n->n_hostile, n->n_spam, n->n_evil);
 
 			if (show_gnet_info_rt) {
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
@@ -302,126 +310,131 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 	return a;
 }
 
-/**
- * Display a summary of the node flags.
- *
- * The stuff in the Flags column means:
- *
- *  - 012345678AB (offset)
- *  - NIrwqxZPFhT
- *  - ^^^^^^^^^^^
- *  - ||||||||||+ T indicates a TLS-tunneled connection
- *  - |||||||||+  hops flow triggerd (h), or total query flow control (f)
- *  - ||||||||+   flow control (F), or pending data in queue (d)
- *  - |||||||+    indicates whether we're a push proxy (P) / node is proxy (p)
- *  - ||||||+     indicates whether RX, TX or both (Z) are compressed
- *  - |||||+      indicates whether we sent our last-hop QRT to remote UP
- *  - ||||+       indicates whether we sent/received a QRT, or send/receive one
- *  - |||+        indicates whether node is writable
- *  - ||+         indicates whether node is readable
- *  - |+          indicates connection type (Incoming, Outgoing, Ponging)
- *  - +           indicates peer mode (Normal, Ultra, Leaf)
- */
-const gchar *
-nodes_gui_common_flags_str(const gnet_node_flags_t *flags)
-{
-	static gchar status[] = "NIrwqTRPFhS";
-
-	switch (flags->peermode) {
-		case NODE_P_UNKNOWN:	status[0] = '-'; break;
-		case NODE_P_ULTRA:		status[0] = 'U'; break;
-		case NODE_P_NORMAL:		status[0] = 'N'; break;
-		case NODE_P_LEAF:		status[0] = 'L'; break;
-		case NODE_P_CRAWLER:	status[0] = 'C'; break;
-		case NODE_P_UDP:		status[0] = 'P'; break;
-		default:				g_assert(0); break;
-	}
-
-	status[1] = flags->incoming ? 'I' : 'O';
-	status[2] = flags->readable ? 'r' : '-';
-	status[3] = flags->writable ? 'w' : '-';
-
-	switch (flags->qrt_state) {
-		case QRT_S_SENT: case QRT_S_RECEIVED:		status[4] = 'Q'; break;
-		case QRT_S_SENDING: case QRT_S_RECEIVING:	status[4] = 'q'; break;
-		case QRT_S_PATCHING:						status[4] = 'p'; break;
-		default:									status[4] = '-';
-	}
-
-	switch (flags->uqrt_state) {
-		case QRT_S_SENT:		status[5] = 'X'; break;
-		case QRT_S_SENDING:		status[5] = 'x'; break;
-		case QRT_S_PATCHING:	status[5] = 'p'; break;
-		default:				status[5] = '-';
-	}
-
-	status[6] =
-		flags->tx_compressed && flags->rx_compressed ? 'Z' :
-		flags->tx_compressed ? 'T' :
-		flags->rx_compressed ? 'R' : '-';
-
-	if (flags->is_push_proxied)  status[7] = 'P';
-	else if (flags->is_proxying) status[7] = 'p';
-	else status[7] = '-';
-
-	if (flags->in_tx_swift_control) status[8]     = 'S';
-	else if (flags->in_tx_flow_control) status[8] = 'F';
-	else if (flags->mqueue_above_lowat) status[8] = 'D';
-	else if (!flags->mqueue_empty) status[8]      = 'd';
-	else status[8]                                = '-';
-
-	if (flags->hops_flow == 0)
-		status[9] = 'f';
-	else if (flags->hops_flow < GTA_NORMAL_TTL)
-		status[9] = 'h';
-	else
-		status[9] = '-';
-
-	status[10] = flags->tls ? 'T' : '-';
-
-	status[sizeof(status) - 1] = '\0';
-	return status;
-}
+struct add_node_context {
+	guint32 flags;
+	guint16 port;
+};
 
 static void
-add_node_helper(guint32 ip, gpointer port)
+add_node_helper(const host_addr_t *addrs, size_t n, gpointer data)
 {
-	guc_node_add(ip, GPOINTER_TO_UINT(port));
+	struct add_node_context *ctx = data;
+
+	g_assert(addrs);
+	g_assert(ctx);
+	g_assert(0 != ctx->port);
+
+	if (n > 0) {
+		guc_node_add(addrs[random_raw() % n], ctx->port, ctx->flags);
+	}
+
+	wfree(ctx, sizeof *ctx);
 }
 
 /**
- * Try to connect to the node given by the addr string in the form
- * [ip]:[port]. Port may be omitted.
+ * Try to connect to the list of nodes given by in following form:
+ *
+ * list = <node> | <node>, 1*<node>
+ * port = 1..65535
+ * hostname = 1*[a-zA-Z0-9.-]
+ * node = hostname [":" <port>]
+ *       | <IPv4 address>[":" <port>]
+ *		 | <IPv6 address>
+ *		 | "[" <IPv6 address> "]:" <port>
+ * peer = ["tls:"]<node>
+ *
+ * If the port is omitted, the default port (GTA_PORT: 6346) is used.
+ * The case-insensitive prefix "tls:" requests a TLS (encrypted) connection.
  */
 void
-nodes_gui_common_connect_by_name(const gchar *addr)
+nodes_gui_common_connect_by_name(const gchar *line)
 {
-    guint32 port = GTA_PORT;
-    gchar *e, *p;
+	const gchar *q;
 
-    g_assert(addr != NULL);
+    g_assert(line);
 
-    e = g_strdup(addr);
-	g_strstrip(e);
+	q = line;
+	while ('\0' != *q) {
+		const gchar *endptr, *hostname;
+		size_t hostname_len;
+		host_addr_t addr;
+		guint32 flags;
+    	guint16 port;
 
-	p = strchr(e, ':');
-	if (p) {
-		const gchar *endptr;
-		guint32 v;
-		gint error;
+		q = skip_ascii_spaces(q);
+		if (',' == *q) {
+			q++;
+			continue;
+		}
 
-		*p++ = '\0';
-		v = parse_uint32(p, &endptr, 10, &error);
-		port = (v > 0 && v < 65536 && *endptr == '\0') ? v : 0;
+		addr = zero_host_addr;
+		port = GTA_PORT;
+		flags = SOCK_F_FORCE;
+		endptr = NULL;
+		hostname = NULL;
+		hostname_len = 0;
+
+		endptr = is_strcaseprefix(q, "tls:");
+		if (endptr) {
+			flags |= SOCK_F_TLS;
+			q = endptr;
+		}
+
+		if (!string_to_host_or_addr(q, &endptr, &addr)) {
+			g_message("Expected hostname or IP address");
+			break;
+		}
+
+		if (!is_host_addr(addr)) {
+			hostname = q;
+			hostname_len = endptr - q;
+		}
+
+		q = endptr;
+
+		if (':' == *q) {
+			gint error;
+
+			port = parse_uint16(&q[1], &endptr, 10, &error);
+			if (error || 0 == port) {
+				g_message("Cannot parse port");
+				break;
+			}
+
+			q = skip_ascii_spaces(endptr);
+		} else {
+			q = skip_ascii_spaces(endptr);
+			if ('\0' != *q && ',' != *q) {
+				g_message("Expected \",\" or \":\"");
+				break;
+			}
+		}
+
+		if (!hostname) {
+			guc_node_add(addr, port, flags);
+		} else {
+			struct add_node_context *ctx;
+			gchar *p;
+
+			if ('\0' == hostname[hostname_len])	{
+				p = NULL;
+			} else {
+				size_t n = 1 + hostname_len;
+
+				g_assert(n > hostname_len);
+				p = g_malloc(n);
+				g_strlcpy(p, hostname, n);
+				hostname = p;
+			}
+
+			ctx = walloc(sizeof *ctx);
+			ctx->port = port;
+			ctx->flags = flags;
+			guc_adns_resolve(hostname, add_node_helper, ctx);
+
+			G_FREE_NULL(p);
+		}
 	}
-
-	if (port < 1 || port > 65535) {
-		statusbar_gui_warning(15, _("Port must be between 1 and 65535"));
-    } else {
-		guc_adns_resolve(e, add_node_helper, GUINT_TO_POINTER((guint) port));
-	}
-
-    G_FREE_NULL(e);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
