@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats_cb.c,v 1.5 2005/06/25 01:37:44 daichik Exp $
+ * $Id: upload_stats_cb.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  * Copyright (c) 2001-2003, Richard Eckart
@@ -38,7 +38,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: upload_stats_cb.c,v 1.5 2005/06/25 01:37:44 daichik Exp $");
+RCSID("$Id: upload_stats_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "upload_stats_cb.h"
 #include "upload_stats.h"
@@ -89,7 +89,7 @@ compare_ul_size(GtkCList *unused_clist, gconstpointer ptr1, gconstpointer ptr2)
 }
 
 /**
- * first by complete, then by attempts
+ * First by complete, then by attempts.
  */
 static gint
 compare_ul_complete(GtkCList *unused_clist,
@@ -110,7 +110,7 @@ compare_ul_complete(GtkCList *unused_clist,
 }
 
 /**
- * first by normalized, then by complete
+ * First by normalized, then by complete.
  */
 gint
 compare_ul_norm(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
@@ -126,7 +126,7 @@ compare_ul_norm(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 }
 
 /**
- * first by attempts, then by complete
+ * First by attempts, then by complete.
  */
 static gint
 compare_ul_attempts(GtkCList *unused_clist,
@@ -195,7 +195,7 @@ on_clist_ul_stats_resize_column(GtkCList *unused_clist,
 	*(gint *) &ul_stats_col_widths[column] = width;
 }
 
-#endif /* USE_GTK1*/
+#endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
 void
@@ -206,8 +206,8 @@ on_popup_upload_stats_config_cols_activate(GtkMenuItem *unused_menuitem,
 
 	(void) unused_menuitem;
 	(void) unused_udata;
-    cc = gtk_column_chooser_new(
-			lookup_widget(main_window, "treeview_ul_stats"));
+
+    cc = gtk_column_chooser_new(gui_main_window_lookup("treeview_ul_stats"));
     gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
 }
 #endif /* USE_GTK2 */
