@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_common.c,v 1.16 2005/07/01 09:28:06 daichik Exp $
+ * $Id: uploads_common.c 11727 2006-08-26 04:20:24Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,28 +25,33 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_common.c,v 1.16 2005/07/01 09:28:06 daichik Exp $");
+RCSID("$Id: uploads_common.c 11727 2006-08-26 04:20:24Z cbiere $")
 
 #include "uploads.h"			/* For upload_row_data_t */
 #include "uploads_common.h"
+#include "search_common.h"
+#include "settings.h"
 
 #include "if/gui_property.h"
 #include "if/gnet_property.h"
+#include "if/core/uploads.h"
+#include "if/bridge/ui2c.h"
 
+#include "lib/host_addr.h"
 #include "lib/misc.h"
 #include "lib/glib-missing.h"	/* For gm_snprintf() */
+#include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-#define IO_STALLED		60		/**< If nothing exchanged after that many secs */
+#define IO_STALLED		60	/**< If nothing exchanged after that many secs */
 
 /**
- * uploads_gui_progress
  *
  * @returns a floating point value from [0:1] which indicates
  * the total progress of the upload.
  */
-gfloat uploads_gui_progress(
-	const gnet_upload_status_t *u,
+gfloat
+uploads_gui_progress(const gnet_upload_status_t *u,
 	const upload_row_data_t *data)
 {
 	gfloat progress = 0.0;
@@ -57,6 +62,7 @@ gfloat uploads_gui_progress(
 
 	switch (u->status) {
     case GTA_UL_HEADERS:
+    case GTA_UL_EXPECTING:
     case GTA_UL_WAITING:
     case GTA_UL_PFSP_WAITING:
     case GTA_UL_ABORTED:
@@ -105,13 +111,15 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 
     case GTA_UL_COMPLETE:
 		{
-	        filesize_t requested = data->range_end - data->range_start + 1;
 			gint t = delta_time(u->last_update, data->start_date);
-			
+	        filesize_t requested = data->range_end - data->range_start + 1;
+
 			gm_snprintf(tmpstr, sizeof(tmpstr),
-				_("Completed (%s) %s"),
-				t > 0 ? short_rate(requested / t) : _("< 1s"),
-				t > 0 ? short_time(t) : "");
+				_("Completed (%s) %s%s"),
+				t > 0 ? short_rate(requested / t, show_metric_units())
+						: _("< 1s"),
+				t > 0 ? short_time(t) : "",
+				u->parq_quick ? " (quick)" : "");
 		}
         break;
 
@@ -119,25 +127,30 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 		{
 			/* Time Remaining at the current rate, in seconds  */
 			filesize_t tr = (data->range_end + 1 - u->pos) / MAX(1, u->avg_bps);
-			gint slen = gm_snprintf(tmpstr, sizeof(tmpstr), "%.02f%%",
-							uploads_gui_progress(u, data) * 100.0);
-			time_t now = time(NULL);
+			gdouble p = uploads_gui_progress(u, data);
+			time_t now = tm_time();
 			gboolean stalled = delta_time(now, u->last_update) > IO_STALLED;
+			gchar pbuf[32];
 
-			slen += gm_snprintf(&tmpstr[slen], sizeof(tmpstr)-slen, " (%s)",
-							stalled ? _("stalled") : short_rate(u->bps));
-
-			gm_snprintf(&tmpstr[slen], sizeof(tmpstr)-slen,
-				" TR: %s", short_time(tr));
+			gm_snprintf(pbuf, sizeof pbuf, "%5.02f%% ", p * 100.0);
+			gm_snprintf(tmpstr, sizeof tmpstr, _("%s(%s) TR: %s %s"),
+				p > 1.0 ? pbuf : "",
+				stalled ? _("stalled")
+					: short_rate(u->bps, show_metric_units()),
+				short_time(tr),
+				u->parq_quick ? " (quick)" : "");
 		}
 		break;
 
     case GTA_UL_HEADERS:
         return _("Waiting for headers...");
 
-    case GTA_UL_WAITING:
+    case GTA_UL_EXPECTING:
         return _("Waiting for further request...");
 
+    case GTA_UL_WAITING:
+        return _("Reading follow-up request...");
+
     case GTA_UL_PFSP_WAITING:
         return _("Unavailable range, waiting retry...");
 
@@ -151,7 +164,7 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 		{
 			guint32 max_up, cur_up;
 			gboolean queued;
-			size_t slen = 0;
+			gchar tbuf[64];
 
 			/*
 			 * Status: GTA_UL_QUEUED. When PARQ is enabled, and all upload
@@ -165,22 +178,23 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 			gnet_prop_get_guint32_val(PROP_MAX_UPLOADS, &max_up);
 			gnet_prop_get_guint32_val(PROP_UL_RUNNING, &cur_up);
 			queued = u->parq_position > max_up - cur_up;
-				
-			slen += gm_snprintf(tmpstr, sizeof(tmpstr),
-						_("%s [%d] (slot %d/%d)"),
+
+			/* position 1 should always get an upload slot */
+			if (u->parq_retry > 0) {
+				gm_snprintf(tbuf, sizeof tbuf,
+							" %s,", short_time(u->parq_retry));
+			} else {
+				tbuf[0] = '\0';
+			}
+			gm_snprintf(tmpstr, sizeof tmpstr,
+						_("%s [%d] (slot %d/%d)%s %s %s"),
 						queued ? _("Queued") : _("Waiting"),
 						u->parq_queue_no,
 						u->parq_position,
-						u->parq_size);
-						
-			/* position 1 should always get an upload slot */
-			if (u->parq_retry > 0) {
-				slen += gm_snprintf(&tmpstr[slen], sizeof(tmpstr) - slen,
-							" %s,", short_time(u->parq_retry));
-			}
-					
-			slen += gm_snprintf(&tmpstr[slen], sizeof(tmpstr) - slen,
-						" %s %s", _("lifetime:"), short_time(u->parq_lifetime));
+						u->parq_size,
+						tbuf,
+						_("lifetime:"),
+						short_time(u->parq_lifetime));
 
 		}
 		break;
@@ -228,6 +242,7 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 	case GTA_UL_SENDING:
 	case GTA_UL_HEADERS:
 	case GTA_UL_WAITING:
+	case GTA_UL_EXPECTING:
 	case GTA_UL_QUEUED:
 	case GTA_UL_QUEUE:
 	case GTA_UL_QUEUE_WAITING:
@@ -236,18 +251,57 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 	}
 
 	if (0 != prop) {
-		guint32 grace;
+		guint32 val;
+		time_delta_t grace;
+
+		gnet_prop_get_guint32_val(PROP_ENTRY_REMOVAL_TIMEOUT, &val);
+		grace = val;
 
-		gnet_prop_get_guint32_val(PROP_ENTRY_REMOVAL_TIMEOUT, &grace);
 		if (delta_time(now, ul->last_update) > grace) {
-			gboolean val;
+			gboolean auto_remove;
 
-			gui_prop_get_boolean_val(prop, &val);
-			return val;
+			gui_prop_get_boolean_val(prop, &auto_remove);
+			return auto_remove;
 		}
 	}
 
 	return FALSE;
 }
 
+/**
+ * @return A pointer to a static buffer holding the host address as string.
+ */
+const gchar * 
+uploads_gui_host_string(const gnet_upload_info_t *u)
+{
+	static gchar buf[1024];
+	const gchar *peer;
+
+	if (u->gnet_port && is_host_addr(u->gnet_addr)) {
+		peer = host_addr_port_to_string(u->gnet_addr, u->gnet_port);
+	} else {
+		peer = NULL;
+	}
+
+	concat_strings(buf, sizeof buf,
+		host_addr_to_string(u->addr),
+		u->encrypted ? " (E) " : " ",
+		peer ? " <" : "",
+		peer ? peer : "",
+		peer ? ">"  : "",
+		(void *) 0);
+
+	return buf;
+}
+
+/**
+ * Initiate a browse host of the uploading host.
+ */
+void
+uploads_gui_browse_host(host_addr_t addr, guint16 port)
+{
+	if (host_addr_is_routable(addr) && port != 0)
+		search_gui_new_browse_host(NULL, addr, port, NULL, NULL, 0);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
